#include "BuildDataOrganizer.h"

// Called from model == SpatialTemperatureHydro
void BuildDataOrganizer::AddBulkAreaSpatialCoverData(DataFolder *folder, Inputs *input, int locationIndex)
{
	if (input->NLCDLandCoverData[locationIndex] == input->LocationParams["NODATA"])
	{
		folder->ParamStringDict["Type"] = "NODATA";
		return;
	}
	folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] = input->TreeOnImpervious[locationIndex];
	folder->ParamDict["ImperviousCover_frac"] = input->ImperCoverdata[locationIndex] / 100.0; 
	folder->ParamDict["TotalImperviousCover_frac"] = input->TreeOnImpervious[locationIndex] + folder->ParamDict["ImperviousCover_frac"];

	folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] = input->TreeOnPervious[locationIndex];
	folder->ParamDict["ShortVegetationCover_frac"] = input->ShortVegCover[locationIndex];
	folder->ParamDict["SoilCover_frac"] = input->SoilCover[locationIndex];
	folder->ParamDict["TotalPerviousCover_frac"] = input->TreeOnPervious[locationIndex] + input->ShortVegCover[locationIndex] + input->SoilCover[locationIndex];

	folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] = input->ConnectedImpervious[locationIndex] * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"] = (1 - input->ConnectedImpervious[locationIndex]) * folder->ParamDict["TotalImperviousCover_frac"];

	folder->ParamDict["NLCDLandCoverData"]= input->NLCDLandCoverData[locationIndex];
	LandCoverPCal::CalLandCover(input, folder, locationIndex);
}

//Called from model = StatisticalHydro
void BuildDataOrganizer::AddBulkAreaStatisticalCoverData(DataFolder *folder)
{
	folder->ParamDict["TotalImperviousCover_frac"] = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"];
	folder->ParamDict["TotalPerviousCover_frac"] = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];
	folder->ParamDict["TotalTreeCover"] = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] = folder->ParamDict["DCIA_frac"] * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"] = (1.0 - folder->ParamDict["DCIA_frac"]) * folder->ParamDict["TotalImperviousCover_frac"];
}

//Called from model == LCScalingTempSpatial
void BuildDataOrganizer::AddBulkAreaAlternativeCoverData(DataFolder *folder, Inputs *input, int locationIndex)
{
	folder->ParamDict["TotalImperviousCover_frac"] = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"];
	folder->ParamDict["TotalPerviousCover_frac"] = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];
	folder->ParamDict["TotalTreeCover"] = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] = folder->ParamDict["DCIA_frac"] * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"] = (1 - folder->ParamDict["DCIA_frac"]) * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["TreeP"] = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	folder->ParamDict["SoilP"]= folder->ParamDict["SoilCover_frac"];
	folder->ParamDict["ShortVegP"]= folder->ParamDict["ShortVegetationCover_frac"];
	folder->ParamDict["ImperP"]= folder->ParamDict["TotalImperviousCover_frac"];
	input->NLCDLandCoverData.push_back(folder->ParamDict["NLCDLandCoverData"]);
	folder->ParamDict["WaterP"] = 0;

}

void BuildDataOrganizer::SetGIMissingParams(DataOrganizer *organizer, Inputs *input)
{ 
	if (input->SimulationStringParams["Model"] == "CoolRiver") return;
	for(int i = 0; i < organizer->DataDrawers[0].size(); ++i)
	{ 
		DataFolder *giFolder = organizer->DataDrawers[0][i];	
		if (giFolder->ParamStringDict["Type"] == "BulkArea")
			continue;
		for (const auto &myPair : input->RefParams) {
			if (giFolder->ParamDict.find(myPair.first) == giFolder->ParamDict.end()) {
				giFolder->ParamDict[myPair.first] = myPair.second;
			}
		}
	}
}

//Called from model == SpatialTemperatureHydro
void BuildDataOrganizer::BuildSpatialorganizer(DataOrganizer *organizer, Inputs *input)
{

	for (int i = 0; i < input->LocationParams["NumberOfLocations"]; ++i)
	{
		int index = 0;
		std::vector<DataFolder*> dataDrawer;
		while (index < input->DataDrawers[0].size())
		{
			std::map<std::string, double> folderInfo = input->DataDrawers[0][index];
			if (input->StringDataDrawers[0][index]["Type"] == "BulkArea")
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[0][index];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				AddBulkAreaSpatialCoverData(folder, input, i);
				dataDrawer.push_back(folder);
			}
			++index;
		}
		organizer->DataDrawers.push_back(dataDrawer);
	}
}

//Called from model = StatisticalHydro
void BuildDataOrganizer::BuildStatisticalDataOrganizer(DataOrganizer *organizer, Inputs *input)
{
	int index = 0;
	for (int j = 0; j < input->DataDrawers.size(); ++j)
	{
		std::vector<DataFolder*> dataDrawer;
		for (int i = 0; i < input->DataDrawers[j].size(); ++i)
		{
			std::map<std::string, double> folderInfo = input->DataDrawers[j][i];
		
			if (input->StringDataDrawers[j][i]["Type"] == "BulkArea" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				// Reza: I will take care of this later in SemiAveSMDCalc.cpp
				//folder->VarDict["aveSMD"] = input->AveSMD;
				AddBulkAreaStatisticalCoverData(folder);
				dataDrawer.push_back(folder);
			}

			if (input->StringDataDrawers[j][i]["Type"] == "RainGarden" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				//folder->Type = "RainGarden";
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				//folder->ParamDict["StorageZoneDepthMax"] = folder->ParamDict["soilThickness_m"] * (folder->ParamDict["soilPorosity"]);
				dataDrawer.push_back(folder);

			}
			if (input->StringDataDrawers[j][i]["Type"] == "BioRetention" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				// Not sure about this:
				//folder->ParamDict["StorageZoneDepthMax"] = rainGarden->ParamDict["StorageZoneDepth"] * (rainGarden->ParamDict["StorageZonePorosity"] / 100);
				dataDrawer.push_back(folder);

			}
			if (input->StringDataDrawers[j][i]["Type"] == "InfilTrench" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				// Not sure about this:
				//folder->ParamDict["StorageZoneDepthMax"] = rainGarden->ParamDict["StorageZoneDepth"] * (rainGarden->ParamDict["StorageZonePorosity"] / 100);
				dataDrawer.push_back(folder);

			}
			if (input->StringDataDrawers[j][i]["Type"] == "Swale" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				// Not sure about this:
				//folder->ParamDict["StorageZoneDepthMax"] = rainGarden->ParamDict["StorageZoneDepth"] * (rainGarden->ParamDict["StorageZonePorosity"] / 100);
				dataDrawer.push_back(folder);

			}
			if (input->StringDataDrawers[j][i]["Type"] == "RoofDiscon" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				// Not sure about this:
				//folder->ParamDict["StorageZoneDepthMax"] = rainGarden->ParamDict["StorageZoneDepth"] * (rainGarden->ParamDict["StorageZonePorosity"] / 100);
				dataDrawer.push_back(folder);

			}
			if (input->StringDataDrawers[j][i]["Type"] == "GreenRoof" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				//folder->Type = "GreenRoof";
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				//folder->ParamDict["StorageZoneDepthMax"] = folder->ParamDict["StorageZoneDepth"] * folder->ParamDict["StorageZonePorosity"];
				dataDrawer.push_back(folder); 
			}
			if (input->StringDataDrawers[j][i]["Type"] == "PermeablePavement" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				//folder->Type = "PermeablePavement";
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				//folder->ParamDict["StorageZoneDepthMax"] = folder->ParamDict["StorageZoneDepth"] * folder->ParamDict["StorageZonePorosity"];
				dataDrawer.push_back(folder); 
			}
			if (input->StringDataDrawers[j][i]["Type"] == "RainBarrel" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				//folder->Type = "RainBarrel";
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				//folder->ParamDict["StorageZoneDepthMax"] = folder->ParamDict["StorageZoneDepth"] * folder->ParamDict["StorageZonePorosity"];
				dataDrawer.push_back(folder);
			}
			if (input->StringDataDrawers[j][i]["Type"] == "TreePit" && folderInfo["Area"] != 0.0)
			{
				DataFolder *folder = new DataFolder;
				folder->ParamDict = folderInfo;
				//folder->Type = "TreePit";
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;
				//folder->ParamDict["StorageZoneDepthMax"] = folder->ParamDict["StorageZoneDepth"] * folder->ParamDict["StorageZonePorosity"];
				dataDrawer.push_back(folder);
			}
			++index;
		}
		organizer->DataDrawers.push_back(dataDrawer);
	}
}


//Called from model == LCScalingTempSpatial
void BuildDataOrganizer::BuildLCScaling(DataOrganizer *organizer, Inputs *input)
{
	/*	UnifiedHydro model = LCScaling only runs two scenarios, the reference station and the alternative.
		The UnifiedHydroConfig.xml used with LCScaling must give parameters for these two scenarios with the two <DataFolders>

		LCScaling uses LCScaling\BatchRunsParamAdjustor\LCScalingForTempSpatial.cs to define the gap filling when TC + IC < 1 and generate the file 100Runs_Parameters.txt,
		100Runs_Parameters.txt has TC, IC, and gap (SVeg and Soil) that sums to 1,
		LCScaling\100_20_Runs_Code\20Runs_Jun2014_Sutherland_TT_2012\20Runs_Jun2014_Sutherland_TT\ChangeParameter.cpp changes the DCIA

		LCScaling in a 121 or other multiple run sceneario overwrites the UnifiedHydroConfig.xml, and uses ConfigIC#TC#.txt to keep a record of each UnifiedHydroConfig.xml
		LCScaling generates and overwrites the BlockGroup1.txt and BlockGroup2.txt files with each run, and converts those files into IC#TC#.txt for permanent output.
	*/
	
		//Likely folder for reference cell
		std::vector<DataFolder*> dataDrawer;
	    std::map<std::string, double> folderInfo = input->DataDrawers[0][0];
	    DataFolder *folder = new DataFolder;
		folder->ParamDict = folderInfo;
		folder->ParamStringDict = input->StringDataDrawers[0][0];
		folder->VarDict["aveSMD"] = input->AveSMD;
		folder->VarDict["position"] = 0;
		AddBulkAreaAlternativeCoverData(folder, input, 0);
		dataDrawer.push_back(folder);
		organizer->DataDrawers.push_back(dataDrawer);

		//Likely folder for non-reference cell
		std::vector<DataFolder*> dataDrawer1;
		DataFolder *folder1 = new DataFolder;
		folder1->ParamDict = input->DataDrawers[0][1];;
		folder1->ParamStringDict = input->StringDataDrawers[0][1];
		folder1->VarDict["aveSMD"] = input->AveSMD;
		folder1->VarDict["position"] = 1;
		AddBulkAreaAlternativeCoverData(folder1, input, 1);
		dataDrawer1.push_back(folder1);
		organizer->DataDrawers.push_back(dataDrawer1);
}

void BuildDataOrganizer::Go(DataOrganizer *organizer, Inputs *input)
{
	if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro")
	{
			BuildSpatialorganizer(organizer, input);
	}
	else if (input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
			BuildLCScaling(organizer, input);
	}
	else if (input->SimulationStringParams["Model"] == "StatisticalHydro")
	{
			BuildStatisticalDataOrganizer(organizer, input);
	}
}

