
#include <iostream>
#include "Inputs\inputs.h"
#include "DataOrganizer.h"
#include "TestSimulation.h"
#include "BuildDataOrganizer.h"
#include <chrono>
#include "LogMessage.h"

typedef std::chrono::high_resolution_clock Clock;


int main(int argc, char* argv[])
{
	try
	{
		auto t1 = Clock::now();
		LogMessage::log("Starting new hydro simulation");
		std::string directory = argv[1];
		Inputs *input = new Inputs(directory);
		DataOrganizer *organizer = new DataOrganizer;
		BuildDataOrganizer::Go(organizer, input);
		BuildDataOrganizer::SetGIMissingParams(organizer, input);
		TestSimulation::Go(organizer, input, directory);	
		delete input;
		delete organizer;
		auto t2 = Clock::now();
		int dur = std::chrono::duration_cast<std::chrono::seconds>(t2 - t1).count();
		std::cout << "Program Runtime: "
			<< dur
			<< "seconds" << std::endl;
		LogMessage::log("Program Runtime secs: "+std::to_string(dur));
		LogMessage::log("Simulation Complete.\n\n");
		//cin.get();
	}
	catch (std::exception ex)
	{
		std::cout << "\n  " << ex.what() << "\n\n";
		std::string s(ex.what());
		LogMessage::log("Error in simulation occurred:\n" + s);
		LogMessage::log("\n\n");
	}
	catch (...)
	{
		std::cout << "Error in simulation run.\n\n";
		LogMessage::log("Error in simulation occurred. No exception message:\n");
		LogMessage::log("\n\n");
	}
}