#ifndef GIOUTLETS_H
#define GIOUTLETS_H
//////////////////////////////////////////////////////////////////////////////////
//																		        //
// GI_Outlets.h - Green Infrastructure algorithms designed for the i-Tree Hydro //
//                       addupted from SWMM v5.1 LID controls                   //																		   
// ver 1.0                                                                      //
// Language:    C++ (Visual Studio 2019)                                        //
// Platform:    Win10           										        //
// Application: Unified Hydro, Winter, Spring, and Summer 2019                  //
// Programmer:  Reza Abdi, reabdi@syr.edu               				        //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////
/*
Origin:
==================
Program: SWMM v5.1

Source Code File(s): Lid.c & LidProc.c in SWMM v5.1 source code

Publication(s): Reza Abdi PHDThesis

Module Operations:
==================
...
/*
Config file inputs for generic GI device, named for Bioretention with bioretention parameter values
	<DataFolder> <!--Terms in this folder replace those in the BulkArea folder; code could be changed to delete these datafolder terms that should be the same as the BulkArea terms-->
		<Type>BioRetention</Type> <!--GI type, selected from: BioRetention, InfilTrench, PermeablePavement, RainGarden, Swale-->
		<Area>46.45</Area> <!--GI area in square meters, for each unit in the type-->
		<giNumber>30.0</giNumber> <!--GI number of similar units in the type-->
		<upSlopeArea>26206</upSlopeArea> <!--Contributing area in square meters within the catchment that drains to the GI units-->
		<upSlopeAreaImpFrac>0.499</upSlopeAreaImpFrac> <!--Impervious fraction within the contributing area-->
		<PerviousCoverUnderTreeCanopy_frac>0.0</PerviousCoverUnderTreeCanopy_frac> <!--Tree cover fraction within the GI area that has pervious cover under the canopy-->
		<ImperviousCoverUnderTreeCanopy_frac>0.0</ImperviousCoverUnderTreeCanopy_frac> <!--Tree cover fraction within the GI area that has impervious cover under the canopy-->
		<ShortVegetationCover_frac>0.5</ShortVegetationCover_frac> <!--Short vegetation fraction within the GI area-->
		<SoilCover_frac>0.5</SoilCover_frac> <!--Bare soil fraction within the GI area-->
		<ImperviousCover_frac>0.0</ImperviousCover_frac> <!--Impervious fraction within the GI area-->
		<ImpAreaTreated_frac>0.8</ImpAreaTreated_frac> <!--Fraction of upslope impervious area runoff sent to the GI, remaininder goes to DCIA-->
		<PerAreaTreated_frac>0.8</PerAreaTreated_frac> <!--Fraction of pervious area runoff sent to the GI, remaininder goes to pervious inflow-->
		<DCIA_frac>0.0</DCIA_frac> <!-- Directly connected impervious area in GI area that drains directly to GI outlet over connected impervious surfaces-->
		<ImperviousDepressionStorage_mm>1.14</ImperviousDepressionStorage_mm> <!--Impervious depression storage of the GI area-->
		<PerviousDepressionStorage_mm>0.0</PerviousDepressionStorage_mm> <!--Pervious depression storage of the GI area-->
		<T0_m2ph>0.33</T0_m2ph> <!--Soil transmissivity at saturation units of meters squared per time step, not necessarily per hour; time was 5min when model was developed and value was converted back to hour = 0.33 * 12-->
		<n>2</n> <!-- Scale parameter of power function infiltration; infiltration set to exponentional or power function decay based on SimulationStringParams-->
		<m>0.2</m> <!-- Scale parameter of soil transmissivity -->
		<MSD_m>0.08</MSD_m> <!-- Maximum root zone storage deficit in meters, is used in GI to define soil thickness = MSD_m / effectivePorosity -->
		<K0_mph>0.0025</K0_mph> <!-- Soil hydraulic conductivity units of meters per time step, not necessarily per hour; time was 5min when model was developed and value was converted back to hour = 0.0025 * 12-->
		<WFS_m>0.0613</WFS_m> <!--Wetting front suction in meters-->
		<DWSMI_m>0.1975</DWSMI_m> <!--Wetted moisture content: Delta Wettingfront Saturated soil moisture content Minus Initial soil moisture content-->
		<Td_h>10</Td_h> <!--Time delay for unsaturated zone in time step units, not necessarily per hour; time was 5min when model was developed and value was converted back to hour = 10 * 12-->
		<Q0_mph>8.33e-07</Q0_mph> <!--Initial stream discharge in meters per time step, not necessarily per hour; time was 5min when model was developed and value was converted back to hour = 8.33e-7 * 12-->
		<SD0_percent>0.0</SD0_percent> <!--Initial root zone saturation deficit in percent, 0 is fully saturated-->
		<PAQ_RT_A_h>0.04</PAQ_RT_A_h> <!--Time constant for pervious area flow; when model was developed this was calibrated to match SWMM example project in Ft Collins, CO-->
		<PAQ_RT_B_h>37.36</PAQ_RT_B_h> <!--Time constant for pervious area flow; when model was developed this was calibrated to match SWMM example project in Ft Collins, CO-->
		<DCIAQ_RT_A_h>0.04</DCIAQ_RT_A_h> <!--Time constant for directly connected impervious area flow; when model was developed this was calibrated to match SWMM example project in Ft Collins, CO-->
		<DCIAQ_RT_B_h>37.36</DCIAQ_RT_B_h> <!--Time constant for directly connected impervious area flow; when model was developed this was calibrated to match SWMM example project in Ft Collins, CO-->
		<SSQ_RT_h>120.0</SSQ_RT_h> <!--Time constant for subsurface flow in time step, not necessarily per hour; when model was developed this was kept at the same magnitude of the hourly value = 120*1-->
		<PMacro_frac>0.0</PMacro_frac> <!--Soil macropore fraction for GI area -->
		<InfiltExcessGovernedArea_frac>1.0</InfiltExcessGovernedArea_frac> <!-- Fraction of GI area where rainfall rate can exceed infiltration rate -->
		<TreeLAI>0.0</TreeLAI> <!-- Leaf area index for tree cover in the GI area -->
		<ShortVegLAI>0.0</ShortVegLAI> <!-- Leaf area index for short vegetation in the GI area -->
		<LeafOnDay>1</LeafOnDay> <!-- Leaf on day in Julian counter in the GI area-->
		<LeafOffDay>365</LeafOffDay> <!-- Leaf off day in Julian counter in the GI area-->
		<LeafTransDays>28</LeafTransDays>  <!-- Leaf transition from on to off and off to on in the GI area-->
		<EvergreenTreeCover_frac>0.002</EvergreenTreeCover_frac> <!--Fraction of tree cover that is evergreen in the GI area-->
		<TreeBAI>0.0</TreeBAI> <!-- Bark area index in the GI area-->
		<EverGreenShrubCover_frac>0.002</EverGreenShrubCover_frac> <!--Fraction of shrub cover that is evergreen in the GI area-->
		<ShrubBAI>0.0</ShrubBAI> <!-- Bark area index in the GI area-->
		<TreeLeafStorage_mm>0.0</TreeLeafStorage_mm> <!-- Depth of water storage in meters per unit area of leaf-->
		<BF_flag>0.0</BF_flag> <!-- Baseflow Flag for GI area; set to 0 when there is no baseflow from GI and baseflow is computed in BulkArea; we developed model with BF_flag = 0 -->
		<startDayOfYr>128.0</startDayOfYr> <!-- Start of simulation in Julian day for use in RainBarrel GI device to get drytime variable, and updated by tstep of units second-->
		<tstep>300.0</tstep>  <!--time step in seconds used with GI devices, to adjust drytime variable, SurfaceOutflow variable, and Manning Eqn based outflow variable-->
		<soilFieldCapacity>0.2</soilFieldCapacity> <!--Soil field capacity for GI soil (distinct layer in GI device that is shared with upper layer in Hydro), units of m^3/m^3; this variable was used in SWMM source code-->
		<soilCondSlope>10.0</soilCondSlope> <!-- Soil conductivity slope for GI soil to compute percolation, gradient units; this variable was used in SWMM source code-->
		<soilWiltPoint>0.1</soilWiltPoint> <!--Soil wilting point for GI soil (distinct layer in GI device that is shared with upper layer in Hydro), units of m^3/m^3; this variable was used in SWMM source code-->
		<storThickness_m>0.8</storThickness_m> <!--Storage layer thickness in meters, is distinct layer in GI device, typically higher porosity -->
		<storVoidFrac>0.4</storVoidFrac> <!--Storage layer void fraction, unit fraction; for storage layer -->
		<storClogFactor>0.0</storClogFactor> <!-- Clogging factor for storage layer-->
		<seepageRate_mph>0.000423</seepageRate_mph> <!-- Seepage rate (e.g., conductivity) for storage layer, in meters per time step, not necessarily per hour; when model was developed this was 5 min time step-->
		<paveThickness_m>0.0</paveThickness_m> <!-- Permeable pavement layer thickness in units of meters, this layer sits above all other porous layers -->
		<drainMatThickness_m>0.0</drainMatThickness_m> <!-- Drainage material layer thickness in units of meters, this layer used in Green Roofs -->
		<drainVoidFrac>0.0</drainVoidFrac> <!-- Drainage material void fraction, this layer used in Green Roofs -->
		<drainOffset>0.05</drainOffset> <!-- Drain offset for storage layer drain, offset in units of meters, is height to drain invert (bottom) from the bottom of the storage layer -->
		<drainOrificeQ_coef>0.3<drainOrificeQ_coef> <!--Drain orifice coefficeint for storage layer drain, units are from SWMM and likely BG emperical formula; coefficient value of 0 sets drainage to 0-->
		<drainOrificeQ_exponent>0.9<drainOrificeQ_exponent> <!--Drain orifice exponent for storage layer drain, units are from SWMM and likely BG emperical formula -->
		<surfaceEmergencyH_m>0.8</surfaceEmergencyH_m> <!--Surface emergency spillway above outlet berm, in meters above ground surface; water above this height releases water to DCIA-->
		<surfaceBermH_m>0.15</surfaceBermH_m> <!--Surface berm height, in meters above ground surface, allows for water to leave the GI device as controlled outflow, except for Swale-->
		<surfaceSlope_frac>0.015</surfaceSlope_frac> <!--Surface slope of GI device in streamwise direction, units m/m or fraction, used with Manning Eqn to compute velocity-->
		<surfaceRoughness>0.1</surfaceRoughness> <!--Surface roughness of GI device, traditionally unitless, used with Manning Eqn to compute velocity-->
		<surfaceVoidRatio>1.0</surfaceVoidRatio> <!--Surface void ratio to represent loss of volume vegetation volume fraction -->
		<giWidthAtOutlet_m>4.0</giWidthAtOutlet_m> <!--Width of GI devices at outlet for BioRetention, RainGarden, PermeablePavement, InfiltrationTrench, but not for Swale-->
		<effPorosity>0.401<effPorosity> <!-- effective porosity in units of m^3/m^3, is equal to soil porosity-->
		<binNumber>24</binNumber> <!--Topographic index bin number used for GI devices; model orders bins TI values from largest to smallest in the bins-->
		<surfaceSideSlope>0.0</surfaceSideSlope> <!--Surface side slope which is required for the Swale GI type (H:V) -->
		<pavementImpervFrac>0.0</pavementImpervFrac> <!-- The impervious fraction of the pavement layer in the GI type, reguired for Permeable Pavement GI type, the unit is fraction -->
		<paveThickness_m>0.2</paveThickness_m> <!-- The thickness of the of the pavement layer in the GI type, reguired for Permeable Pavement GI type ,in m -->
		<pavementVoidFrac>0.3</pavementVoidFrac> <!-- The void fraction of the pavement layer in the permaeable pavemnet GI type. -->
		<paveClogFactor>0.0</paveClogFactor> <!-- Clogging factor for permeable layer, for Permeable Pavement GI type-->
		<pavePermeability_mph>2.12</pavePermeability_mph> <!-- The permeability of the pavement layer in Permeable Pavement GI type. -->
		<paveRegenDays>0.0</paveRegenDays> <!-- The days required for regeneration for the pavement layer, for Permeable Pavement GI type. -->
	</DataFolder>
Public Interface:
=================


Related changes on other routines:
==============
Inputs.cpp - Set SimulationNumericalParams["TotalTimeSteps"] = 144.0 or other #, from = daysDiff * 24 to account for partial day and non-hourly time steps
ImperviousDepressionStorageCalc.cpp
InfiltrationExpDecayCalc.cpp
InfiltrationPowDecayCalc.cpp
TestSimulation.cpp
UnsatZonePerviousFlowCalc.cpp
WaterOnPerviousAreaCalc.cpp
FlowSumCalc.cpp
RootZoneETCalc.cpp
SemiAveSMDCalc.cpp

Maintenance History:
====================

*/


#include "../Inputs/Inputs.h"
#include "../DataFolder.h"

//Define constants used in GI_Outlets.cpp
// 4 layers are: surface, soil, storage, and pavement
#define MAX_LAYERS 4		
#define BIG 1.E10
// Effective zero value
#define ZERO 1.E-10         
// Integration error tolerance set to 0.001 m = 1 mm
#define STOPTOL  0.001      
// Value for TRUE state
#define TRUE     1              
// Value for FALSE state
#define FALSE    0              
#define SECperDAY   86400
// Set runoff minumum from SWMM 0.001 in/hr to m/sec
#define MIN_RUNOFF  7.06e-9     
// Set flow cutoff for dry conditions from SWMM 0.001 in/hr to m/sec
#define MINFLOW  7.06e-9        

// Function for finding minimum of x and y    
#define MIN(x,y) (((x)<=(y)) ? (x) : (y))        
// Function for finding maximum of x and y    
#define MAX(x,y) (((x)>=(y)) ? (x) : (y))        



class GI_Outlets
{
public:

	// Get the water balance (runoff, drainage, infil, evap) for the GI units; Call evalGI_Unit
	static void gi_getRunoff(Inputs* input, DataFolder* folder, int timestep);
	// Get water balance (runoff, drainage, infil, evap) for GI units; Call giProc_getOutflow
	static void evalGI_Unit(DataFolder* folder, double impDep_BA, int timestep);
	// Get water balance (runoff, drainage, infil, evap) for GI units; Call modpuls_solve 
	static void giProc_getOutflow(DataFolder* folder, double* giEvap, double* giInfil, double* giDrain, int timestep);
	// Call flux rate function for the specific GI device, Get water levels and flux rates over the time step
	static void modpuls_solve(DataFolder* folder, int n, double* x, double* xOld, double* xPrev,
		double* xMin, double* xMax, double* xTol, double* qOld, double* q, double dt);

	// Called by modpuls_solve: computes flux rates from the layers of a bioretention
	static void bioretentionFluxRates(DataFolder* folder, double x[], double f[]);
	// Called by modpuls_solve: computes flux rates from the layers of a green roof
	static void greenRoofFluxRates(DataFolder* folder, double x[], double f[]);
	// Called by modpuls_solve: computes flux rates from the layers of a infiltration trench
	static void trenchFluxRates(DataFolder* folder, double x[], double f[]);
	// Called by modpuls_solve: computes flux rates from the layers of a permeable pavement
	static void pavementFluxRates(DataFolder* folder, double x[], double f[]);
	// Called by modpuls_solve: computes flux rates from the layers of a rain barrel
	static void barrelFluxRates(DataFolder* folder, double x[], double f[]);
	// Called by modpuls_solve: computes flux rates from the layers of a roof disconnect
	static void roofFluxRates(DataFolder* folder, double x[], double f[]);
	// Called by modpuls_solve: computes flux rates from the layers of a swale
	static void swaleFluxRates(DataFolder* folder, double x[], double f[]);

	// Not used by i-Tree Hydro; was SWWM function to get GI evaporation rates.
	static void getEvapRates(DataFolder* folder, double surfaceVol, double paveVol, double soilVol, double storageVol, double pervFrac);
	// Get percolation rate of water through a GI's soil layer; using SWMM function; Hydro rootZoneQ in GI area = percolation
	static double getSoilPercRate(DataFolder* folder, double theta);
	// Get exfiltration rate from storage zone into the native soil; this water decreases the AveSMD_GI, which updates AveSMD of the BulkArea and affects baseflow
	static double getStorageExfilRate(DataFolder* folder);
	// Get underdrain flow rate in a GI's storage layer; this water increases the AveSMD_GI, which updates AveSMD of the BulkArea and affects baseflow
	static double getStorageDrainRate(DataFolder* folder, double storageDepth,
		double soilTheta, double paveDepth, double surfaceDepth);
	// Get outflow rate from a GI's surface layer.
	static double getSurfaceOutflowRate(DataFolder* folder, double surfaceDepth);
	// The function finds/computes surface overflow rate from a GI unit.
	static double getSurfaceOverflowRate(DataFolder* folder, double surfaceDepth);
	// In this function the code updates the mass balance for a GI unit and saves
	//  current flux rates to the GI report file.
	static void giproc_saveResults(DataFolder* folder);
	// The function updates components of the water mass balance for a GI unit
	// The vectors then are used in the output writer functions
	//Li:Added the argument double total_Evap
	static void updateWaterBalance(DataFolder* folder, double inflow, double infil, double soilPerc, double exfil,
		double surfFlow, double drainFlow, double paveDepth, double surfaceDepth,
		double soilMoisture, double storageDepth, double total_Evap);
	// The function updates components of the water mass balance for a GI unit
	//  over the current time step.
	//Li:Added the argument double total_Evap
	static void updateWaterBalancePermeablePavement(DataFolder* folder, double inflow, double infil, double soilPerc, double exfil,
		double surfFlow, double drainFlow, double paveDepth, double surfaceDepth,
		double soilMoisture, double storageDepth, double pavePerc, double total_Evap);

	// The function computes flow rate through a green roof's drainage mat.
	static double getDrainMatOutflow(DataFolder* folder, double depth);
	// The function initializes the water balance components of a GI unit
	static void giproc_initWaterBalance(DataFolder* folder);
	// The function computes reduced permeability of a pavement layer due to
	//  clogging.
	static double getPavementPermRate(DataFolder* folder);
	// computes runoff and drain flows from the GIs for the available data folders.
	static void getGI_Outflow(Inputs* input, DataFolder* folder, int timestep);

};





#endif