#include "IndexCalculator.h"
#include <utility>
#include <iomanip>


// IndexCalculator constructor - Power Decay - directory is initiliazed to the passed directory, N is initialized to N (the scale
// parameter of the soild decay power function - reaad in from the param.dat, 
// success is initilized to 1 - an indication of a successful run - it is reinitialized to 0 if readDEMData fails
IndexCalculator::IndexCalculator(std::string directory, double N, std::string simulationMode, std::string routingAlgorithm)
	: directory(directory), N(N), simulationMode(simulationMode), routingAlgorithm(routingAlgorithm), pi(3.14159265359), success(1)
{
	// This function initializes private data members of the Indexcalculator class like so:
	//  rows = 0; cols = 0;  NATB = 0; NAC =0; DX = 0.0; fArea = 1.0; totalFImpArea = 0.0;
	init();
	// This function clears all the vectors used in the index calculations
	clear();

	// Defining the number of TI bins
	NAC = 30;
	// Resizing the following vectors to contain 30 elements
	y.resize(NAC);    // 
	AC.resize(NAC);   // Proportion of catchment area associated with ST(IA)
	ST.resize(NAC);   // Ln(a/tanB) or (a/tanB)^n value in summary histogram



					  // if readDEMData() is a success it returns TRUE
	if (readDEMData())
		indexCalPow();      // Run indexCal() function        
	else
		success = 0;     // if it fails, reset success to 0 so failure is known
}



// IndexCalculator constructor  - Exponential Decay - directory is initiliazed to the passed directory,
// success is initilized to 1 - an indication of a successful run - it is reinitialized to 0 if readDEMData fails
IndexCalculator::IndexCalculator(std::string directory, std::string simulationMode, std::string routingAlgorithm)
	: directory(directory), simulationMode(simulationMode), routingAlgorithm(routingAlgorithm), pi(3.14159265359), success(1)
{
	init();
	clear();

	// ?
	NAC = 30;
	// Resizing the following vectors to contain 30 elements
	y.resize(NAC);    // 
	AC.resize(NAC);   // Proportion of catchment area associated with ST(IA)
	ST.resize(NAC);   // Ln(a/tanB) or (a/tanB)^n value in summary histogram

	if (readDEMData())
		indexCalExp();
	else
		success = 0;

}


// The clear() clear all the following vectors of their contents
void IndexCalculator::clear()
{

	elevation.clear();
	ATB.clear();
	Area.clear();
	y.clear();
	AC.clear();
	ST.clear();
}



//--------------< check to see if the current folder exists >------------------
// Used to check the bounds of the DEM, slopeorganizer,  flowDirorganizer, flowAccumorganizer, temporganizer, TWIorganizer and possibly others

bool IndexCalculator::checkfolder(int row, int col)
{
	if (row<0 || row>(rows - 1) || col<0 || col>(cols - 1))
		return false;
	else
		return true;
}


//--------------< returns the elevation of the current folder >------------------
// Used to check the bounds of the DEM, slopeorganizer,  flowDirorganizer, flowAccumorganizer, temporganizer, TWIorganizer and possibly others

double IndexCalculator::getElevation(int row, int col)
{
	return elevation[row][col];
}


bool IndexCalculator::fileExists(const std::string& filename)
{
	struct stat buf;
	if (stat(filename.c_str(), &buf) != -1)
	{
		return true;
	}
	return false;
}

// the readDEMData function reads in the DEMDATA
bool IndexCalculator::readDEMData()
{

	double tempElv;                 // Variable for temporarily storing the read in elevation data
	int size = 0;                    // Initializing the size storing variable to 0
	const int MAX = 80;             // 
	char buffer[MAX];               //

	std::string location = directory + "dem.dat";
	bool datExists = fileExists(location);
	if (!datExists)
	{
		location = directory + "dem.txt";
	}
	else if (!fileExists(location))
	{
		std::cout << "Cannot find DEM file" << std::endl;
		LogMessage::log("Cannot find DEM file. Should be named dem.dat or dem.txt in directory: " + directory);
		return 0;
	}

	ifstream infile(location);    // note: every line should followed by a "enter" immiediately.
	if (!infile.good())
	{
		std::cout << "Cannot find DEM file" << std::endl;
		LogMessage::log("Cannot find DEM file. Should be named dem.dat or dem.txt in directory: "+directory);
		return 0;

	}

	// Outputting to the user the current step/process
	std::cout << "Reading DEM file\n" << std::endl;

	infile >> buffer;
	_catName = buffer;


	if (_catName == "ncols" || _catName == "NCOLS")  // new version with no city name and xllcorner/yllcorner info
	{
		std::cout << "DEM stats:" << std::endl;
		//col number, nrows, row number, xll, xll, yll,yll, cell aka folder size, DX number, NODATA, NODATA
		infile >> cols >> buffer >> rows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> DX >> buffer >> NODATA;
		std::cout << "Rows: " << rows << "   Cols: " << cols << "   cell size: " << DX << "   NODATA: " << NODATA << std::endl;
	}
	else  // this is the trimmed down older format with a project/city name and no xllcorner/yllcorner information
	{
		std::cout << "DEM stats:" << std::endl;
		infile.clear();
		infile.seekg(0, ios::beg);
		infile.getline(buffer, MAX);
		_catName = buffer;
		infile >> buffer >> cols >> buffer >> rows >> buffer >> DX >> buffer >> NODATA;
		std::cout << "Rows: " << rows << "   Cols: " << cols << "   cell size: " << DX << "   NODATA: " << NODATA << std::endl;
	}
	/* End of modification */

	elevation.reserve(rows);
	slopeorganizer.reserve(rows);
	aspectorganizer.reserve(rows);
	flowDirorganizer.reserve(rows);
	flowAccumorganizer.reserve(rows);
	temporganizer.reserve(rows);
	infloworganizer.reserve(rows);
	TWIorganizer.reserve(rows);


	for (int k = 0; k<rows; k++)
	{
		elevation.push_back(*new vector<double>(cols));
		slopeorganizer.push_back(*new vector<double>(cols));
		aspectorganizer.push_back(*new vector<double>(cols));
		flowDirorganizer.push_back(*new vector<double>(cols));
		flowAccumorganizer.push_back(*new vector<double>(cols));
		temporganizer.push_back(*new vector<double>(cols));
		infloworganizer.push_back(*new vector<double>(cols));
		TWIorganizer.push_back(*new vector<double>(cols));

	}

	double temp = 0.0;

	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (!infile.good())
			{
				infile.clear();
				break;
			}

			infile >> tempElv;
			elevation[i][j] = tempElv;
			elevation1D.push_back(tempElv);
			// Initializing new organizers to the NODATA value no matter what
			// overwritten with real values during calculations

			slopeorganizer[i][j] = NODATA;
			aspectorganizer[i][j] = NODATA;
			flowDirorganizer[i][j] = NODATA;
			temporganizer[i][j] = NODATA;
			infloworganizer[i][j] = NODATA;
			TWIorganizer[i][j] = NODATA;

			// THis one must be filled in with values
			if (elevation[i][j] == NODATA)
				flowAccumorganizer[i][j] = NODATA;
			else
				flowAccumorganizer[i][j] = 1;
		}
	}

	infile.close();
	std::cout << "DEM file read." << std::endl << std::endl;
	// Return 1 to represent success
	return 1;
}


// These three functions are called for the Power Decay Topographic Index calculation
void IndexCalculator::indexCalPow()
{
	TI_Cal();
	if (simulationMode.find("Stat") != std::string::npos)
		indexHist();
	transform();
	createDistorganizerFiles();

}


// These two functions are called for the Exponential Decay Topographic Index calculation
void IndexCalculator::indexCalExp()
{
	TI_Cal();
	if (simulationMode.find("Stat") != std::string::npos)
		indexHist();
	createDistorganizerFiles();
}


// The Topographic Index calculation controlling function
// Calculates slope 
void IndexCalculator::TI_Cal()
{

	cout << "Starting the Topographic Index calculation." << endl;
	LogMessage::log("Starting the Topographic Index calculation.");
	// all index functions call this function - so I should replace this function
	// this function will call three other functions now:
	// 1. a slope calculating function (Horn, 1981)
	// 2. a functich determines flow directions (maybe parent/chile relationships?)
	// 3. a function that calculates upslope contributing area per unit contour width based on the flow direction
	// 4. The final step which is the simple (upslope contrinbuting area/tan(slope)) calculation


	std::cout << "Computing the slope of each folder." << endl;
	LogMessage::log("Computing the slope of each folder.");
	calcSlope();

	std::cout << "Computing the flow direction of each folder." << endl;
	LogMessage::log("Computing the flow direction of each folder. Routing Algorithm: "+routingAlgorithm);
	if (routingAlgorithm == "DInfinity")
	{
		calcFlowDirection_DInf();
	}
	else if (routingAlgorithm == "D8")
	{
		calcFlowDirection_D8();
	}
	else if (routingAlgorithm == "MFD")
	{
		calcFlowDirection_MFD();
	}

	std::cout << "Computing the specfic contributing area of each folder." << endl;
	LogMessage::log("Computing the specfic contributing area of each folder.");
	if (routingAlgorithm == "DInfinity")
		calcSCA_Dinf();
	else if (routingAlgorithm == "D8")
		calcSCA_D8();
	else if (routingAlgorithm == "MFD")
		calcSCA_MFD();

	std::cout << "Computing the topographic index value of each folder." << endl;
	LogMessage::log("Computing the topographic index value of each folder.");
	calcTWI();


}



//---------------< calculate folder slopes >----------------------//

void IndexCalculator::calcSlope()
{
	// Using the 3rd order difference slope algorithm and aspect algorithm published by Horn (1981)
	/*DEM folder layout
	|6|7|0|
	|5|8|1|
	|4|3|2|
	*/

	double z;
	double organizer[8];
	double fx, fy;
	double slopefolder;
	double progress = 0;

	//The DEM cells start at 0 in top-right and increase clockwise, finishing with 7 in the center top
	//Dy[8] and Dx[8] impose this sequence in folders about a central folder, similar to WhiteBox GIS
	//(x=1,y=-1) is top-right, (x=1,y=1) is bottom-right, etc
	int Dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	int Dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	
	double radToDeg = 180 / pi;
	
	for (int row = 0; row<rows; row++)
	{
		for (int col = 0; col<cols; col++)
		{

			z = getElevation(row, col);

			if (z != NODATA)
			{
				for (int k = 0; k<8; k++)
				{
					// if the folder exists, grab it's elevation, otherwise set it to NODATA, 
					// Dy and Dx are resolution of grid
					if (checkfolder(row + Dy[k], col + Dx[k]))
						organizer[k] = getElevation(row + Dy[k], col + Dx[k]);
					else
						organizer[k] = NODATA;

					if (organizer[k] == NODATA)
						//Set to the central elevation 
						organizer[k] = z;      

				}

				// Calculating the folder's slope based on Horn (1981)
				//The DEM cells start at 0 in top-right and increase clockwise, finishing with 7 in the center top
				//fx = (organizer[2] - organizer[4] + 2 * (organizer[1] - organizer[5]) + organizer[0] - organizer[6]) / (8 * DX);
				//fy = (organizer[6] - organizer[4] + 2 * (organizer[7] - organizer[3]) + organizer[0] - organizer[2]) / (8 * DX);
				//Horn Eqn (1981) for fx and fy re-organized below to lump addition and subtraction terms
				fx = (organizer[2] + 2 * organizer[1] + organizer[0] - organizer[4] - 2 * organizer[5]  - organizer[6]) / (8 * DX);
				fy = (organizer[6] + 2 * organizer[7] + organizer[0] - organizer[4] - 2 * organizer[3] - organizer[2]) / (8 * DX);

				// Compute slope based on Horn (1981)
				slopefolder = (atan(sqrt(fx * fx + fy * fy)) * radToDeg);

				// Replace fx or fy = 0 with 0.0000001 to ensure aspect calculation is not dividing by zero.
				if (fx == 0)
					fx = 0.0000001;
				if (fy == 0)
					fy = 0.0000001;

				double t = slopefolder / radToDeg;

				// Compute aspect based on Horn (1981) and WhiteBox, w/ Pi = 3.1415926 and Pi/2 = 1.5707963 to properly rotate aspect
				aspectorganizer[row][col] = 3.1415926 - atan(fy / fx) + 1.5707963 * fx / abs(fx);


				// checking a setting a minimum slope if slope = 0 or approx = 0 set to 0.001
				if (slopefolder <= 0.001)
					slopeorganizer[row][col] = 0.001;
				else
					slopeorganizer[row][col] = slopefolder;

			}
			else
			{
				slopeorganizer[row][col] = NODATA;

			}
		}
	}
}



//---------------< calculate folder upslope contributing areas using the DInf algorithm  >----------------------//

void IndexCalculator::calcFlowDirection_DInf()
{

	double DX2 = DX * sqrt(2.0);      // distance between diagnonal folder centers
	double e0;                        // Current folder's elevation 
	double maxSlope;                  // Current folder's max slope
	double flowDir = 0;               // Current folder's flow direction

	double pi = 3.14159265358979323846;
	double atanof1 = atan(1.0);

	double af;                        //       
	double ac;                        //
	double e1, r, s1, s2, s, e2;      //

									  // arrarys for properly cycling through (switching between) the 8 neighboring folders
									  // follows the pattern laid out in Tarboton (1997)
	int acVals[8] = { 0, 1, 1, 2, 2, 3, 3, 4 };
	int afVals[8] = { 1, -1, 1, -1, 1, -1, 1, -1 };

	int e1Col[8] = { 1, 0, 0, -1, -1, 0, 0, 1 };
	int e1Row[8] = { 0, -1, -1, 0, 0, 1, 1, 0 };

	int e2Col[8] = { 1, 1, -1, -1, -1, -1, 1, 1 };
	int e2Row[8] = { -1, -1, -1, -1, 1, 1, 1, 1 };

	for (int row = 0; row<rows; row++)
	{
		for (int col = 0; col<cols; col++)
		{

			e0 = elevation[row][col];    // reading in the initial elevation

			if (e0 != NODATA)
			{
				maxSlope = -9999999999;
				flowDir = 360;
				for (int k = 0; k < 8; k++)
				{
					ac = acVals[k];
					af = afVals[k];

					// Checking to see if the neighboring folders exist
					if (checkfolder(row + e1Row[k], col + e1Col[k]))
						e1 = elevation[row + e1Row[k]][col + e1Col[k]];
					else
						e1 = NODATA;

					if (checkfolder(row + e2Row[k], col + e2Col[k]))
						e2 = elevation[row + e2Row[k]][col + e2Col[k]];
					else
						e2 = NODATA;

					// Beginning of the slope calculation loop
					if (e1 != NODATA && e2 != NODATA)
					{
						if (e0 > e1 && e0 > e2)
						{
							s1 = (e0 - e1) / DX;
							if (s1 == 0) {
								s1 = 0.00001;
							}
							s2 = (e1 - e2) / DX;
							r = atan(s2 / s1);
							s = sqrt(s1 * s1 + s2 * s2);
							if (s1 < 0 && s2 < 0) {
								s = -1 * s;
							}
							if (s1 < 0 && s2 == 0) {
								s = -1 * s;
							}
							if (s1 == 0 && s2 < 0) {
								s = -1 * s;
							}
							if (s1 == 0.00001 && s2 < 0) {
								s = -1 * s;
							}
							if (r < 0 || r > atanof1)
							{
								if (r < 0) {
									r = 0;
									s = s1;
								}
								else {
									r = atanof1;
									s = (e0 - e2) / DX2;
								}
							}
							if (s >= maxSlope && s != 0.00001)
							{
								maxSlope = s;
								flowDir = af * r + ac * (pi / 2);
							}
						}
						else if (e0 > e1 || e0 > e2) {
							if (e0 > e1) {
								r = 0;
								s = (e0 - e1) / DX;
							}
							else {
								r = atanof1;
								s = (e0 - e2) / DX2;
							}
							if (s >= maxSlope && s != 0.00001)
							{
								maxSlope = s;
								flowDir = af * r + ac * (pi / 2);
							}
						}
					}
				}
				if (maxSlope <= 0) {
					flowDirorganizer[row][col] = -1;
				}
				else {
					// This section of code re-orients the angles to match North = 0/360 degrees
					flowDir = floor((flowDir * (180 / pi)) * 10) / 10;
					flowDir = 360 - flowDir + 90;
					if (flowDir > 360) {
						flowDir = flowDir - 360;
					}
					flowDirorganizer[row][col] = flowDir;
				}
			}
			else {
				flowDirorganizer[row][col] = NODATA;
			}


		}
	}
}

//---------------< calculate folder upslope contributing areas using the D8 algorithm  >----------------------//

void IndexCalculator::calcFlowDirection_D8()
{

	double e0, e1;                        // Current folder's elevation 
	double slope;					  // Temp max slope
	double maxSlope;                  // Current folder's max slope
	double flowDir = 0;               // Current folder's flow direction
	int current_row, current_col;

	// this three vectors combine to match the output of ESRI's FD output
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };

	// this two vectors combine to match the output of WhiteBox's FD output
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };


	double DX2 = DX * sqrt(2.0);      // distance between diagnonal folder centers
	double dist[8] = { DX2, DX, DX2, DX, DX2, DX, DX2, DX };


	for (int row = 0; row<rows; row++)
	{
		for (int col = 0; col<cols; col++)
		{

			e0 = elevation[row][col];    // reading in the initial elevation, using to see if NODATA

			if (e0 != NODATA)
			{
				maxSlope = -9999999999;
				flowDir = -9999999999;
				for (int k = 0; k < 8; k++)
				{
					current_row = row + dy[k];
					current_col = col + dx[k];
					// Checking to see if the neighboring folders exist
					if (checkfolder(row + dy[k], col + dx[k]))
						e1 = elevation[row + dy[k]][col + dx[k]];
					else
						e1 = NODATA;

					// Beginning of the slope calculation loop
					if (e1 != NODATA)
					{
						slope = (e0 - e1) / dist[k];
						if (slope > maxSlope) {
							maxSlope = slope;
							flowDir = pow(2, k);
						}
					}
				}
				if (maxSlope <= 0)
					flowDirorganizer[row][col] = -1;
				else
					flowDirorganizer[row][col] = flowDir;
			}
			else {
				flowDirorganizer[row][col] = NODATA;
			}


		}
	}
}

//---------------< calculate folder upslope contributing areas using the D8 algorithm  >----------------------//

void IndexCalculator::calcFlowDirection_MFD()
{

	double e0, e1;                        // Current folder's elevation 
	double slope;					  // Temp max slope
	double maxSlope;                  // Current folder's max slope
	double flowDir = 0;               // Current folder's flow direction
	int current_row, current_col;

	// this three vectors combine to match the output of ESRI's FD output
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };

	// this two vectors combine to match the output of WB's FD output
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };


	double DX2 = DX * sqrt(2.0);      // distance between diagnonal folder centers
	double dist[8] = { DX2, DX, DX2, DX, DX2, DX, DX2, DX };


	for (int row = 0; row<rows; row++)
	{
		for (int col = 0; col<cols; col++)
		{

			e0 = elevation[row][col];    // reading in the initial elevation, using to see if NODATA

			if (e0 != NODATA)
			{
				flowDir = 0;
				for (int k = 0; k < 8; k++)
				{
					current_row = row + dy[k];
					current_col = col + dx[k];
					// Checking to see if the neighboring folders exist
					if (checkfolder(row + dy[k], col + dx[k]))
						e1 = elevation[row + dy[k]][col + dx[k]];
					else
						e1 = NODATA;

					// Beginning of the slope calculation loop
					if (e1 != NODATA && e1 < e0)
					{
						flowDir += pow(2, k);
					}
				}
				
				flowDirorganizer[row][col] = flowDir;
			}
			else {
				flowDirorganizer[row][col] = NODATA;
			}


		}
	}
}






//---------------< calculate out Specific Contributing Area (SCA) based on the Dinf flow pointer organizer>----------------------//
void IndexCalculator::calcSCA_Dinf()
{

	// Original rotation - Matches with the setting of North as 0/360 degrees
	int dCol[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dRow[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double organizerRes = 1;
	double startFD[8] = { 180, 225, 270, 315, 0, 45, 90, 135 };
	double endFD[8] = { 270, 315, 360, 45, 90, 135, 180, 225 };

	double flowDir, flowAccum;
	int row, col, newRow, newCol;
	int i;
	int loopNum = 0;

	// Setting the maximum flow path length
	// 1000 is what John Linsdsay (University of Guelph) used in Whitebox GAT
	// Has to do with concerns of stack overflow due to ram/processor(?) limitations
	// this is incremented once for each folder along a pathway that is investigated by DinfAccum()
	maxDepth = 10000;

	// Used to track whether a flow path was followed
	// when all connected folders ahve been explored, this allows the do-while loop to exit
	bool somethingDone;


	// Start of loop to keep track of the total inflowing neighbors to the current folder
	for (row = 0; row < rows; row++) {
		for (col = 0; col < cols; col++) {
			flowDir = flowDirorganizer[row][col];
			if (flowDir != NODATA) {
				i = 0;
				for (int k = 0; k < 8; k++)
				{
					newCol = col + dCol[k];
					newRow = row + dRow[k];

					if (checkfolder(newRow, newCol))
						flowDir = flowDirorganizer[newRow][newCol];
					else
						continue;

					if (flowDir >= 0 && flowDir <= 360) {
						if (k != 3) {
							if (flowDir > startFD[k] && flowDir < endFD[k]) {
								i++;
							}
						}
						else {
							if (flowDir > startFD[k] || flowDir < endFD[k]) {
								i++;
							}
						}
					}
				}

				temporganizer[row][col] = i;
				infloworganizer[row][col] = i;

			}
			else {
				flowAccumorganizer[row][col] = NODATA;
			}
		}
	}

	// Start of loop to call the recursive algorithm for summing accumulation along a connected flow path
	do {
		loopNum++;
		somethingDone = false;
		for (row = 0; row < rows; row++) {
			for (col = 0; col < cols; col++)
			{
				if (getElevation(row, col) != NODATA)
				{
					if (temporganizer[row][col] == 0)
					{   //there are no 
						//remaining inflowing neighbours, send its current 
						//flow accum val downslope
						currentDepth = 0;
						somethingDone = true;
						DinfAccum(row, col);
					}
				}
				else
					continue;
			}
		}
	} while (somethingDone);

	// for loops for turning the area into specific contriibuting area
	// up til now area has been bassed on as fractions of an area unit
	// here it is multiplied by the organizer resolution to get specific catchment area
	//  (unit area * organizer res^2) = total catchment area
	//  total catchment area / organizer res = unit area *  organizer res = specific catchment area
	for (row = 0; row < rows; row++) {
		for (col = 0; col < cols; col++) {
			flowAccum = flowAccumorganizer[row][col];
			if (flowAccum != NODATA) {
				flowAccumorganizer[row][col] = flowAccum * DX;
			}
		}
	}

	// End of Dinf Accum main funciton

}

void IndexCalculator::calcSCA_D8()
{

	// this three vectors combine to match the output of WB's FD output
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double inflowDirVals[8] = { 16, 32, 64, 128, 1, 2, 4, 8 };

	// this three vectors combine to match the output of ESRI's FD output
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	//double inflowDirVals[8] = {32, 64, 128, 1, 2, 4, 8, 16 };

	double flowDir, flowAccum;
	int row, col, newRow, newCol, temporganizerVal, i, x, y;
	int z = 0;
	int loopNum = 0;


	// Used to track whether a flow path was followed
	// when all connected folders ahve been explored, this allows the do-while loop to exit
	bool somethingDone;


	for (int row = 0; row<rows; row++)
	{
		for (int col = 0; col<cols; col++)
		{
			flowDir = flowDirorganizer[row][col];
			if (flowDir != NODATA) {
				z = 0;
				for (int k = 0; k < 8; k++)
				{
					flowDir = flowDirorganizer[row + dy[k]][col + dx[k]];
					if (flowDir == inflowDirVals[k]) {
						z++;
					}
				}
				temporganizer[row][col] = z;
				infloworganizer[row][col] = z;
				//cout << row << " " << col << endl;
			}
			else {
				temporganizer[row][col] = NODATA;
				infloworganizer[row][col] = NODATA;
				flowAccumorganizer[row][col] = NODATA;
			}
		}
	}

	// Start of loop to call the recursive algorithm for summing accumulation along a connected flow path
	for (row = 0; row < rows; row++) {
		for (col = 0; col < cols; col++)
		{
			temporganizerVal = temporganizer[row][col];
			if (temporganizerVal == 0) {
				temporganizer[row][col] = -1;
				somethingDone = false;
				newCol = col;
				newRow = row;
				do {
					flowAccum = flowAccumorganizer[newRow][newCol];
					flowDir = flowDirorganizer[newRow][newCol];
					if (flowDir >= 0) {
						// find which  folder recieves flow direction and pass it down stream
						i = log2(flowDir);
						newCol += dx[i];
						newRow += dy[i];
						//update the output organizers
						flowAccumorganizer[newRow][newCol] += flowAccum;
						temporganizer[newRow][newCol]--;
						if (temporganizer[newRow][newCol] == 0) {
							temporganizer[newRow][newCol] = -1;
							somethingDone = true;
						}
						else {
							somethingDone = false;
						}
					}
					else {
						somethingDone = false;
					}
				} while (somethingDone);
			}
		}
	}

	for (row = 0; row < rows; row++) {
		for (col = 0; col < cols; col++)
		{
			if (flowAccumorganizer[row][col] != NODATA)
				flowAccumorganizer[row][col] *= DX;
		}
	}


}

void IndexCalculator::calcSCA_MFD()
{

	// this three vectors combine to match the output of WB's FD output
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double inflowDirVals[8] = { 16, 32, 64, 128, 1, 2, 4, 8 };

	// this three vectors combine to match the output of ESRI's FD output
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	//double inflowDirVals[8] = {32, 64, 128, 1, 2, 4, 8, 16 };

	double flowDir, flowAccum, e0, e1;
	int row, col, newRow, newCol, temporganizerVal, i, x, y;
	int numNeighbors = 0;
	int loopNum = 0;

	maxDepth = 10000;


	// Used to track whether a flow path was followed
	// when all connected folders ahve been explored, this allows the do-while loop to exit
	bool somethingDone;


	for (int row = 0; row<rows; row++)
	{
		for (int col = 0; col<cols; col++)
		{
			e0 = elevation[row][col];    // reading in the initial elevation, using to see if NODATA
			if (e0 != NODATA){
				numNeighbors = 0;
				for (int k = 0; k < 8; k++)
				{
					e1 = elevation[row + dy[k]][col + dx[k]];
					// Checking to see if the neighboring folders exist
					if (e1 != NODATA && (e1 > e0)){
						numNeighbors++;
					}
				}
				temporganizer[row][col] = numNeighbors;
				infloworganizer[row][col] = numNeighbors;
				//cout << row << " " << col << endl;
			}
			else {
				temporganizer[row][col] = NODATA;
				infloworganizer[row][col] = NODATA;
			}
		}
	}
	
	// Start of loop to call the recursive algorithm for summing accumulation along a connected flow path
	do {
		loopNum++;
		somethingDone = false;
		for (row = 0; row < rows; row++) {
			for (col = 0; col < cols; col++)
			{
				if (getElevation(row, col) != NODATA)
				{
					if (temporganizer[row][col] == 0)
					{   //there are no 
						//remaining inflowing neighbours, send its current 
						//flow accum val downslope
						currentDepth = 0;
						MFDAccum(row, col);
						somethingDone = true;
					}
				}
				else
					continue;
			}
		}
	} while (somethingDone);

}




void IndexCalculator::DinfAccum(int row, int col)
{

	double flowAccumVal = flowAccumorganizer[row][col];
	double flowDir = flowDirorganizer[row][col];
	double tempAccum = 0;
	double proportion1 = 0;
	double proportion2 = 0;
	int a1 = 0;
	int b1 = 0;
	int a2 = 0;
	int b2 = 0;

	currentDepth++;
	if (currentDepth > maxDepth) {
		return;
	}

	temporganizer[row][col] = -1; // this ensures that you don't process this folder a second time.

	if (flowDir >= 0) {
		// find which two folders receive flow and the proportion to each
		if (flowDir >= 0 && flowDir < 45) {
			proportion1 = (45 - flowDir) / 45;
			a1 = col;
			b1 = row - 1;
			proportion2 = flowDir / 45;
			a2 = col + 1;
			b2 = row - 1;
		}
		else if (flowDir >= 45 && flowDir < 90) {
			proportion1 = (90 - flowDir) / 45;
			a1 = col + 1;
			b1 = row - 1;
			proportion2 = (flowDir - 45) / 45;
			a2 = col + 1;
			b2 = row;
		}
		else if (flowDir >= 90 && flowDir < 135) {
			proportion1 = (135 - flowDir) / 45;
			a1 = col + 1;
			b1 = row;
			proportion2 = (flowDir - 90) / 45;
			a2 = col + 1;
			b2 = row + 1;
		}
		else if (flowDir >= 135 && flowDir < 180) {
			proportion1 = (180 - flowDir) / 45;
			a1 = col + 1;
			b1 = row + 1;
			proportion2 = (flowDir - 135) / 45;
			a2 = col;
			b2 = row + 1;
		}
		else if (flowDir >= 180 && flowDir < 225) {
			proportion1 = (225 - flowDir) / 45;
			a1 = col;
			b1 = row + 1;
			proportion2 = (flowDir - 180) / 45;
			a2 = col - 1;
			b2 = row + 1;
		}
		else if (flowDir >= 225 && flowDir < 270) {
			proportion1 = (270 - flowDir) / 45;
			a1 = col - 1;
			b1 = row + 1;
			proportion2 = (flowDir - 225) / 45;
			a2 = col - 1;
			b2 = row;
		}
		else if (flowDir >= 270 && flowDir < 315) {
			proportion1 = (315 - flowDir) / 45;
			a1 = col - 1;
			b1 = row;
			proportion2 = (flowDir - 270) / 45;
			a2 = col - 1;
			b2 = row - 1;
		}
		else if (flowDir >= 315 && flowDir <= 360) {
			proportion1 = (360 - flowDir) / 45;
			a1 = col - 1;
			b1 = row - 1;
			proportion2 = (flowDir - 315) / 45;
			a2 = col;
			b2 = row - 1;
		}

		// giving one of the receiving downslope neighbors the proper flow accumulation
		if (proportion1 > 0 && flowAccumorganizer[b1][a1] != NODATA)
		{
			tempAccum = flowAccumorganizer[b1][a1];
			flowAccumorganizer[b1][a1] = (tempAccum + (flowAccumVal * proportion1));
			temporganizer[b1][a1]--;
			//temporganizer[b1][a1] = (temporganizer[b1][a1] - 1);


			if (temporganizer[b1][a1] == 0) {
				DinfAccum(b1, a1);
			}
		}

		// giving the other of the receiving downslope neighbors the proper flow accumulation
		if (proportion2 > 0 && flowAccumorganizer[b2][a2] != NODATA)
		{
			tempAccum = flowAccumorganizer[b2][a2];
			flowAccumorganizer[b2][a2] = (tempAccum + (flowAccumVal * proportion2));
			temporganizer[b2][a2]--;
			//temporganizer[b2][a2] = (temporganizer[b2][a2] - 1);
			if (temporganizer[b2][a2] == 0) {
				DinfAccum(b2, a2);
			}
		}
	}
	currentDepth--;


}


void IndexCalculator::MFDAccum(int row, int col)
{

	// this three vectors combine to match the output of WB's FD output
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double inflowDirVals[8] = { 16, 32, 64, 128, 1, 2, 4, 8 };

	// this three vectors combine to match the output of ESRI's FD output
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	//double inflowDirVals[8] = {32, 64, 128, 1, 2, 4, 8, 16 };


	double flowAccumVal = flowAccumorganizer[row][col];
	double flowDir = flowDirorganizer[row][col];
	double tempAccum = 0;
	double temporganizerVal = 0;
	double proportion = 0;
	double totalRelief = 0;
	double power = 1.0;
	int newRow, newCol = 0;
	double e0 = 0;
	double e1 = 0;


	currentDepth++;
	if (currentDepth > maxDepth) {
		return;
	}

	temporganizer[row][col] = -1; // this ensures that you don't process this folder a second time.

	e0 = elevation[row][col];

	for (int k = 0; k < 8; k++)
	{
		newRow = row + dy[k];
		newCol = col + dx[k];
		e1 = elevation[newRow][newCol];
		// Checking to see if the neighboring folders exist
		if ((e0 > e1) && (e1 != NODATA)) {
			totalRelief += pow((e0 - e1), power);

		}

	}

	for (int k = 0; k < 8; k++)
	{
		newRow = row + dy[k];
		newCol = col + dx[k];
		e1 = elevation[newRow][newCol];
		// Checking to see if the neighboring folders exist
		if ((e0 > e1) && (e1 != NODATA)) 
		{
			// WHATS GONING ON HERE!!!!!!!!
			proportion = pow((e0 - e1), power) / totalRelief;
			tempAccum = flowAccumorganizer[newRow][newCol];
			double new_FA = (tempAccum + (flowAccumVal * proportion));
			flowAccumorganizer[newRow][newCol] = new_FA;
			temporganizerVal = temporganizer[newRow][newCol];
			temporganizer[newRow][newCol]--;
			if (temporganizer[newRow][newCol] == 0)
			{
				MFDAccum(newRow, newCol);
			}
		}
		
	}

	currentDepth--;


}



void IndexCalculator::calcTWI()
{

	double degToRad = pi / 180;


	for (int row = 0; row < rows; row++) {
		for (int col = 0; col < cols; col++) {
			//cout << row<<" - "<<col;
			if (flowAccumorganizer[row][col] != NODATA)
				TWIorganizer[row][col] = log(flowAccumorganizer[row][col] / (tan(degToRad * slopeorganizer[row][col])));
			else
				TWIorganizer[row][col] = NODATA;
		}
	}
}



void IndexCalculator::indexHist()
{
	int index = 0;
	double TWImax = 0.0;
	double TWImin = 10000;
	double total = 0.0;

	cout << "Generating the Topographic Index histogram." << endl;
	LogMessage::log("Computing the topographic index value of each folder.");

	for (int i = 0; i<NAC; i++)
	{
		y[i] = 0.0;
		AC[i] = 0.0;
		ST[i] = 0.0;
	}

	// Calculating the ATBmin and ATBmax
	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (TWIorganizer[i][j] != NODATA)
			{
				if (TWIorganizer[i][j] > TWImax)
					TWImax = TWIorganizer[i][j];
				if (TWIorganizer[i][j] < TWImin)
					TWImin = TWIorganizer[i][j];
			}
		}
	}

	double dTWI = (TWImax - TWImin) / (NAC - 1);



	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			//cout << "Row: "<<i<<" Col: "<<j<<endl;

			if (TWIorganizer[i][j] != NODATA)
			{
				// index = ((current index value (Alpha/Tan Beta)  - min index value) / min index value) + 1
				// As an example, if we get (((150 - 7) / 9) + 1) = 15.88888 + 1 = 16.8888, which we then turn in 16.  
				// We are then saying, to the 16th bin, add another member
				// If the min is 7, and the max is 270, we get an dTWIorganizer of 9, with bin locations at:
				// 16, 25, 34, 43, 52, 61, 70, 79, 88, 97, 106, 115, 124, 133, 142, 151, 160, 169, 178, 187, 196, 205, 214, 223, 232, 241, 250, 259, 268, 277
				// An index value of 150 puts it in the 16th bin, which has a value of 151
				// An index value of 8 (right above the min) puts it in the 1st bin with a value of 16.
				// An index value of 276 (right below the max) puts it in the last bin, with a value of 30
				index = static_cast<int>(((TWIorganizer[i][j] - TWImin) / dTWI) + 1);
				if (index > NAC - 1)
					index = NAC - 1;
				y[index] += 1;
				total += 1;
			}
		}
	}

	ST[0] = TWImax;

	for (int i = 1; i<NAC; i++)
	{
		AC[i] = y[NAC - i] / total;
		ST[i] = TWImax - (i)*dTWI;
	}

}




void IndexCalculator::transform()
{
	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{

			if (TWIorganizer[i][j] == NODATA)
				TWIorganizer[i][j] = NODATA;
			else
				TWIorganizer[i][j] = pow(exp(TWIorganizer[i][j]), 1 / N);
			if (TWIorganizer[i][j] <0)
				TWIorganizer[i][j] = 0.01;

		}
	}

	for (int i = 0; i<NAC; i++)
		ST[i] = pow(exp(ST[i]), 1 / N);

}




void IndexCalculator::createDistorganizerFiles()
{

	std::cout << "Outputing the topography files." << endl;
	LogMessage::log("Outputing the topography files.");

	// Outputting the TIorganizer.dat
	//ofstream outfile(directory + "TIorganizer.dat");
	ofstream outfile(directory + "TIorganizer.dat");
	if (!outfile.good())
	{
		LogMessage::log("Could not create"+directory + "TIorganizer.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "TIorganizer.dat");
	}
	//cte outfile << _catName << endl;
	outfile << "ncols" << std::setw(10) << cols << endl;
	outfile << "nrows" << std::setw(10) << rows << endl;
	outfile << "xllcorner" << std::setw(20) << xllcorner << endl;
	outfile << "yllcorner" << std::setw(20) << yllcorner << endl;
	outfile << "cellsize" << std::setw(10) << DX << endl;
	outfile << "NODATA_value" << std::setw(10) << NODATA << endl;

	int rem = cols % 8;

	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile << " " << std::setw(10) << TWIorganizer[i][j];
			else
				outfile << " " << std::setw(10) << NODATA;
		}
		outfile << std::endl;
	}
	// End of outputting the TIorganizer.dat



	// outputting the flowdirection organizer	
	//ofstream outfile1(directory + "FDorganizer.dat");
	ofstream outfile1(directory + "FDorganizer.dat");
	if (!outfile1.good())
	{
		LogMessage::log("Could not create" + directory + "FDorganizer.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "FDorganizer.dat");
	}
	outfile1 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile1 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile1 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile1 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile1 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile1 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;

	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile1 << flowDirorganizer[i][j] << " ";
			else
				outfile1 << NODATA << " ";
		}
		outfile1 << std::endl;
	}

	outfile1.close();
	// End of outputting the FDorganizer.dat

	// Outputting the slope organizer in radians
	double radToDeg = 180 / pi;
	ofstream outfile2(directory + "Slope.dat");
	if (!outfile2.good())
	{
		LogMessage::log("Could not create" + directory + "Slope.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "Slope.dat");
	}
	outfile2 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile2 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile2 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile2 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile2 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile2 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;


	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
			{
				outfile2 << slopeorganizer[i][j] / radToDeg << " ";
			}
			else
				outfile2 << NODATA << " ";
		}
		outfile2 << std::endl;
	}

	outfile2.close();
	// End of outputting the slope organizer

	// Outputting the aspect organizer 
	ofstream outfile2p5(directory + "Aspect.dat");
	if (!outfile2p5.good())
	{
		LogMessage::log("Could not create" + directory + "Aspect.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "Aspect.dat");
	}
	outfile2p5 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile2p5 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile2p5 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile2p5 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile2p5 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile2p5 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;


	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile2p5 << aspectorganizer[i][j] << " ";
			else
				outfile2p5 << NODATA << " ";
		}
		outfile2p5 << std::endl;
	}

	outfile2p5.close();
	// End of outputting the slope organizer
	// Outputting the flow accumulation organizer	
	//ofstream outfile3(directory + "FlowAccum.dat");
	ofstream outfile3(directory + "FlowAccum.dat");

	if (!outfile3.good())
	{
		LogMessage::log("Could not create" + directory + "FlowAccum.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "FlowAccum.dat");
	}
	outfile3 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile3 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile3 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile3 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile3 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile3 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;

	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile3 << flowAccumorganizer[i][j] << " ";
			else
				outfile3 << NODATA << " ";
		}
		outfile3 << std::endl;
	}

	outfile3.close();
	// End of outputting the flow accumulation organizer


}



void IndexCalculator::createIndexFile(bool powdecay)
{
	std::string name = "";
	if (powdecay)
	{
		name = "powdecayTI.dat";
	}
	else
	{
		name = "expdecayTI.dat";
	}
	ofstream outfile(directory + name);
	if (!outfile.good())
	{
		LogMessage::log("Could not create" + directory + name);
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + name);
	}

	outfile << ((_catName == "ncols" || _catName == "NCOLS") ? "%ofProjectArea TIValueBin" : _catName) << endl;
	outfile << NAC << " " << fArea << endl;

	for (int i = 0; i<NAC; i++)
		outfile << "  " << std::setw(10) << AC[i] << "  " << std::setw(10) << ST[i] << endl;

	outfile.close();


}