#ifndef INDEX_H
#define INDEX_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
//  index.h -- Calculate topgraphic index                               //
//                                                                      //
//                                                                      //
//                                                                      //
//  Language:     Visual C++, ver 6.0                                   //
//  Platform:     Pentium III,IV, Windows 2000, Windows XP              //
//  Application:  iTree Hydro, Summer 2003, 2015//
//  Author:       Jun Wang                                              //
//                Thomas P. Taggart, traggatmot@gmail.com//
//                                                                      //
//////////////////////////////////////////////////////////////////////////


#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <math.h>
#include <sstream>
#include "..\LogMessage.h"

using namespace std;                    

class IndexCalculator
{
public:
	IndexCalculator(std::string directory, double N, std::string simulationMode, std::string routingAlgorithm);        // power decay	
	IndexCalculator(std::string directory, std::string simulationMode, std::string routingAlgorithm);                  // exponential decay	
	void createDistorganizerFiles();
	void createIndexFile(bool powerdecay);
	bool fileExists(const std::string& filename);
	bool getParseSucess() {return success;}
	        
	void getElevation(vector<vector<double>> &elevation){elevation = elevation;}
	void getElevation1D(vector<double> &elevation){elevation = elevation1D;}
	int getcols() { return cols; }
	int getrows() { return rows; }
	double getNODATA() { return NODATA; }
	double getxllcorner() { return xllcorner; }
	double getyllcorner() { return yllcorner; }

private:

	vector<double> elevation1D;
	vector<vector<double>> elevation;	// elevation of each folder, elevation(i,j)
	vector<vector<double>> slopeorganizer;	// slope of each organizer folder
	vector<vector<double>> flowDirorganizer;	// Dinf flow direction organizer
	vector<vector<double>> flowAccumorganizer;// Dinf flow accumulation organizer
	vector<vector<double>> temporganizer;	// temporary organizer for the flow accumulation function
	vector<vector<double>> infloworganizer;	// permanent organizer for the checking the flow accumulation function
	vector<vector<double>> TWIorganizer;// final topographic wetness index  
	vector<vector<double>> aspectorganizer;

	vector<vector<double>> ATB;         // index (ln(a/tanB) or (a/tanB)^n) array, ATB(i,j)
	vector<vector<double>> Area;        // Upslope area draining through a point, Area(i,j)
	vector<double> y;                   // 
	vector<double> AC;                  // Proportion of catchment area associated with ST(IA)
	vector<double> ST;                  // Ln(a/tanB) or (a/tanB)^n value in summary histogram
	int rows, cols, NATB, NAC;          // NATB:Total number of organizer elements within catchment area
	std::string simulationMode;
	std::string routingAlgorithm;
	double xllcorner, yllcorner;        // Lower left lat and long values
	int simMethod;                      // simulation method (0;1): 1: need to load impervious coverage data
	string _catName;                    // Number of ln(a/tanB) or (a/tanB)^n histogram ordinates
	double DX, fArea, NODATA;	// DX, organizer spacing in elevation array, fArea: fractional area of the subcatchment
	double totalFImpArea;               // total fractional impervious area of the simulated subcatchment
    std::string directory;
	double N;
	int currentDepth;
	int maxDepth;
	const double pi;


	bool readDEMData();                  //     Added by Sanyam
	void init() { rows = 0; cols = 0;  NATB = 0; NAC =0; DX = 0.0; fArea = 1.0; totalFImpArea = 0.0;}
	//void flowDirection(int i, int j, double DX1, double DX2, double ROUTE[], int& NROUT, double& SUM);
	//void flowSlope(int i, int j, double DX1, double DX2);
	//void aveIndex(int i, int j,double ROUTE[], double SUM);
	bool checkfolder(int row, int col);    // function that checks if a folder exists, returns the NODATA valus if it doesn't, a '1' value if it does
	double getElevation(int row, int col);
	void TI_Cal();
	void calcSlope();
	void calcFlowDirection_DInf();
	void calcFlowDirection_MFD();
	void calcFlowDirection_D8();
	void calcSCA_Dinf();
	void calcSCA_D8();
	void calcSCA_MFD();
	void DinfAccum(int row, int col);  // Recursive Dinf flow accumulation algorithm
	void MFDAccum(int row, int col);  // Recursive MFD flow accumulation algorithm
	void calcTWI();
	void indexHist();
	void indexCalPow();
	void indexCalExp();
	void transform();
	void clear();
	bool success;




};

#endif

