#ifndef INPUTPARAMS_H
#define INPUTPARAMS_H
//////////////////////////////////////////////////////////////////////////////////////
//																		            //
// Inputs.h - Parses input files and stores input data                              //
//////////////////////////////////////////////////////////////////////////////////////
/*


Module Operations:
==================
Input File(s) 
1. Read - UnifiedHydroConfig.xml



Maintenance History:
====================

*/

#include <string>
#include <map>
#include <fstream>
#include <sstream>
#include "..\XMLReader\XMLDocument.h"
#include "..\LogMessage.h"
#include "InputTopographyData.h"
#include <unordered_set>

class Inputs
{
public:
	Inputs(std::string &directory);

	//UnifiedHydroConfig.xml parameters
	std::map<std::string, std::string> SimulationStringParams;
	std::map<std::string, double> SimulationNumericalParams;
	std::map<std::string, double> LandCoverParams;
	//std::map<std::string, double> VegetationParams;
	//std::map<std::string, double> SoilParams;
	std::map<std::string, double> LocationParams;

	std::map<std::string, double> RefParams;
	std::map<std::string, std::string> RefStringParams;

	std::vector<std::vector<std::map<std::string, double>>> DataDrawers;
	std::vector<std::vector<std::map<std::string, std::string>>> StringDataDrawers;

	std::map<std::string, double> TemperatureCalculationParams;
	std::map<std::string, std::string> TemperatureExecutionParams;

	std::map<std::string, double> CoolRiverVariables;

	//

	//weather.dat parameters
	//std::string weatherXMLVal;
	std::vector<int> CalendarDates;

	std::vector<std::string> TimeSeries;
	std::vector<double> WindSp_mps;		   //input m/s
	std::vector<double> Precip_mpdt;	   //input m/timestep (typically m/h)
	std::vector<double> RadiationNet_Wpm2; //input W/m2
	std::vector<double> Tair;			   //input degrees F
	std::vector<double> Tdew;			   //input degrees F
	std::vector<double> SnowDepth;		   //input m/timestep (typically m/h)
	std::vector<double> Rain;
	std::vector<double> Snow;
	double TotalRain_m;
	double TotalPrecip_m;
	double TotalSnow_m;
	//

	//Evaporation.dat parameters
	std::string evapXMLVal;
	std::vector<double> PE;
	std::vector<double> PETree;
	std::vector<double> PET;
	std::vector<double> PESnow;
	std::vector<double> PETreeSnow;
	std::vector<int> EvapCalendarDates;
	std::vector<std::string> EvapTimeSeries;
	//

	//SolarRadiation.dat parameters
	vector<double> DirectShortWaveRadiation;  //incoming direct short wave radiation  W/m^2, which is not corrected by any angle, so it may have values for 24hour
	vector<double> DiffuseShortWaveRadiation; //incoming diffuse short wave radiation  W/m^2
	vector<double> DownLongWaveRadiation;	  //incoming long wave radiation   W/m^2
	vector<double> UpLongWaveRadiation;		  //up long wave radiation
	vector<double> Latitude;				  //this value is actually same for all time step, may change it to a double value instead of vector of double
	vector<double> HourAngle;				  // in rad
	vector<double> Decline;					  // in rad
	vector<double> Zenith;
	vector<double> CosAzimuth;
	vector<double> SinAzimuth;
	std::vector<int> SolarRadCalendarDates;
	std::vector<std::string> SolarRadTimeSeries;
	//

	//distributed cover: landcover.txt, imperviouscover.txt, treecover.txt
	std::vector<double> NLCDLandCoverData;
	std::vector<double> ImperCoverdata;
	std::vector<double> TreeCoverdata;
	std::vector<double> TotalCover;
	std::vector<double> SoilCover;
	std::vector<double> ImperviousNOTree;
	std::vector<double> ShortVegCover;
	std::vector<double> ConnectedImpervious;
	std::vector<double> FracEgreenTree;
	std::vector<double> FracEgreenShortVeg;
	std::vector<double> TreeOnPervious;
	std::vector<double> TreeOnImpervious;

	//temp input/output processing
	void processTemperatureExecutionParams();
	std::unordered_set<int> LocationIdsToPrintTempOutput;
	std::vector<int> StartDatesForRowColTempOutput;
	std::vector<int> EndDatesForRowColTempOutput;
	std::vector<int> StartDatesForTimeBasedTempOutput;
	std::vector<int> EndDatesForTimeBasedTempOutput;
	std::vector<int> StartingBlockIdsToPrintTempOutput;
	std::vector<int> EndingBlockIdsToPrintTempOutput;
	std::vector<int> EachfoldersBlockGroup;

	//From InputTopographyData.h/.cpp
	void ProcessTopographyData();
	std::string demXMLVal;
	double AveSMD;
	double IndexAreaSize;
	std::vector<double> IndexArea;
	double AverageIndex;
	std::vector<double> Index;
	std::vector<double> DEM;
	std::vector<double> Slope;
	std::vector<double> Aspect;
	//repo
	std::map<std::string, double> RepoDict;
	std::map<std::string, std::vector<double>> RepoVecDict;

	//LAI calcs
	void CalculateLAISeries();
	void CalcTreeLAI(int JulianDay);
	void CalcShortVegLAI(int JulianDay);
	int ToJulianDate(int date);
	std::map<std::string, double> LaiVariables;
	std::vector<double> TreeLAI;
	std::vector<double> ShortVegLAI;
	//

	double MinAllowableWindSpeed;

	std::vector<double> PAQi;
	std::vector<double> BQi;
	std::vector<double> QQi;
	std::vector<double> DCIAQi;
	std::vector<double> BQi_NR;
	std::vector<double> PAQi_NR;
	std::vector<double> DCIAQi_NR;

	// For CoolRiver RiverTimeSeries parameters
	std::vector<int> DatesRiverTime;
	std::vector<std::string> RiverTimeSeries;

	std::vector<double> Sediment_T_C;
	std::vector<double> Cloudiness_F;
	std::vector<double> Humidity_Per;

	std::vector<vector<double>> SW_Q_cms;
	std::vector<vector<double>> SW_T_C;

	std::map<int, vector<double>> SW_Q_cms1;
	std::map<int, vector<double>> SW_T_C1;

	std::map<int, vector<double>> GW_Q_cms1;
	std::map<int, vector<double>> GW_T_C1;

	std::vector<vector<double>> GW_Q_cms;
	std::vector<vector<double>> GW_T_C;
	std::vector<int> number;
	std::vector<double> dischargeDist;
	std::vector<double> dischargeValue1;
	int get_Q_count() { return (totalDischargeCount - 1); }

	// For CoolRiver RiverStationSeries parameters

	std::vector<int> StationNumber;
	std::vector<int> StationDistanceBedData;
	std::vector<double> DepthMesurments;

	std::vector<std::string> TypeOfRiver;
	std::vector<double> HorizontalBedConnectivity;

	std::vector<double> BedParticleSize;
	std::vector<double> Embeddedness;

	std::vector<int> StationDistanceMorphology;
	std::vector<double> AreaR_m2;
	std::vector<double> WidthR_m;

	std::vector<double> DepthR_m;
	std::vector<double> SlopeR_f;

	std::vector<int> RowRiver;
	std::vector<int> ColRiver;
	std::vector<double> LongitudeR_frac;
	std::vector<double> LatitudeR_frac;
	std::vector<double> ZRiver_m;

	std::vector<int> StationDistanceShading;
	std::vector<double> EastBankHR_m;
	std::vector<double> EastTreeHR_m;
	std::vector<double> EastBuildingHR_m;
	std::vector<double> EastBankDistR_m;
	std::vector<double> EastCanDistR_m;
	std::vector<double> EastBuildingDistR_m;
	std::vector<double> EastBufferWR_m;

	std::vector<double> WestBBankHR_m;
	std::vector<double> WestBankHR_m;
	std::vector<double> WestTreeHR_m;
	std::vector<double> WestBuildingHR_m;
	std::vector<double> WestBankDistR_m;
	std::vector<double> WestCanDistR_m;
	std::vector<double> WestBuildingDistR_m;
	std::vector<double> WestBufferWR_m;

	std::vector<int> StationDistanceShadingPercent;
	std::vector<double> ElevationR_m;
	std::vector<double> StreamAzimuthR_deg;
	std::vector<double> ShadeFactorR_f;
	std::vector<double> ViewToSkyR_f;
	int getcols() { return nDemDataCols; }
	int getrows() { return nDemDataRows; }
	double getNODATA() { return NODATA; }
	double getxllcorner() { return nDemDataxllcorner; }
	double getyllcorner() { return nDemDatayllcorner; }

	void setcols(int x) { nDemDataCols = x; }
	void setrows(int x) { nDemDataRows = x; }
	void setxllcorner(double x) { nDemDataxllcorner = x; }
	void setyllcorner(double x) { nDemDatayllcorner = x; }

	//HecRas.dat file
	std::vector<double> downStreamDist;
	std::vector<double> totalQ;
	std::vector<double> chElev;
	std::vector<double> waterSurElev;
	std::vector<double> velChnl;
	std::vector<double> xArea;
	std::vector<double> topWidth;
	std::vector<double> waterSurSlope;
	std::vector<double> wettedPerimeter;

	//Time.dat file
	std::vector<double> TimeMin;
	int totalCount;
	int getCount() { return (totalCount); }

private:
	std::string directory;
	int nDemDataCols;
	int nDemDataRows;
	double nDemDataxllcorner;
	double nDemDatayllcorner;

	//Evaporation.dat file
	bool readEvaporationFile(double startDate, double endDate);
	//

	//Weather.dat file
	bool readWeatherFile(double startDate, double endDate);
	bool readSolarRadiationFile(double startDate, double endDate);
	void solarRadiationCalZenAzim();
	//

	int rdn(int y, int m, int d);

	int Subtract(int dateStart, int dateEnd);

	//UnifiedHydroConfigFile.xml
	void readUnifiedHydroConfigFile();
	void readOldConfigParamFiles();

	//

	//distributed cover: landcover.txt, imperviouscover.txt, treecover.txt
	bool readNLCDLandCoverFile(std::string &directory, double simulationNoDataVal);
	bool readTreeCoverFile(std::string &directory, double simulationNoDataVal);
	bool readImperviousFile(std::string &directory, double simulationNoDataVal);
	void calcNLCDLandCoverData(int NumberOfLocations, double simulationNoDataVal);

	bool readRiverTimeSeriesFile(double startDate, double endDate);
	bool readRiverGWTimeSeriesFile(double startDate, double endDate);
	bool readRiverSWTimeSeriesFile(double startDate, double endDate);
	bool readRiverStationSeriesFile(std::string &directory, double simulationNoDataVal);
	bool readHecRasFile(double startDate, double endDate);
	bool readTimeFile(double startDate, double endDate);
	int ncols;
	int nrows;
	double xllcorner;
	double yllcorner;
	double foldersize;
	double NODATA;
	int totalDischargeCount;
	bool readBlockGroupOutputFileMap(std::string &directory);
};

#endif