#ifndef POLLUTION_H
#define POLLUTION_H
///////////////////////////////////////////////////////////////
//  Pollution.h   -  Calculating various pollution data      //
//  ver 1.0                                                  //
//                                                           //
//  Language:     Visual C++, ver 6.0                        //
//  Application:  Pollution part for Hydro                   //
//  Author:       Rahul Kumbhar, Syracuse Univ.              //
//                (617)416-3561, rahul.d.kumbhar@gmail.com   //
///////////////////////////////////////////////////////////////


#include <iostream>
#include <fstream>
#include <vector>


class Pollution
{
private:
	double TssmedDensity;
	double TssmeanDensity;
	double BODmedDensity;
	double BODmeanDensity;
	double CODmedDensity;
	double CODmeanDensity;
	double TPmedDensity;
	double TPmeanDensity;
	double SolPmedDensity;
	double SolPmeanDensity;
	double TKNmedDensity;
	double TKNmeanDensity;
	double NO2_3medDensity;
	double NO2_3meanDensity;
	double CumedDensity;
	double CumeanDensity;
	double PbmedDensity;
	double PbmeanDensity;
	double ZnmedDensity;
	double ZnmeanDensity;

	std::vector<double> Tssmedmass_kg;
	std::vector<double> Tssmeanmass_kg;
	std::vector<double> BODmedmass_kg;
	std::vector<double> BODmeanmass_kg;
	std::vector<double> CODmedmass_kg;
	std::vector<double> CODmeanmass_kg;
	std::vector<double> TPmedmass_kg;
	std::vector<double> TPmeanmass_kg;
	std::vector<double> SolPmedmass_kg;
	std::vector<double> SolPmeanmass_kg;
	std::vector<double> TKNmedmass_kg;
	std::vector<double> TKNmeanmass_kg;
	std::vector<double> NO2_3medmass_kg;
	std::vector<double> NO2_3meanmass_kg;
	std::vector<double> Cumedmass_kg;
	std::vector<double> Cumeanmass_kg;
	std::vector<double> Pbmedmass_kg;
	std::vector<double> Pbmeanmass_kg;
	std::vector<double> Znmedmass_kg;
	std::vector<double> Znmeanmass_kg;

	std::vector<double> TreePervious;
	std::vector<double> TreeImpervious;
	double timesteps;
	double catchmentarea;
	std::string OutputDirLocation;

	
public:

	Pollution() {};
	bool PollutionDataFileExists;
	Pollution(double timesteps, double catchmentarea, std::string directory);
		 
	void calPollutionData(std::vector<double> &Per, std::vector<double> &ImpPer);
	void calMass(std::vector<double> &mass, double volume, double med_mean);
	void writePollutiondata(std::vector<double> &date, std::vector<double> &time, std::string OutputDir);


	std::vector<double> getTssmedmass();
	std::vector<double> getTssmeanmass();
	std::vector<double> getBODmedmass();
	std::vector<double> getBODmeanmass();
	std::vector<double> getCODmedmass();
	std::vector<double> getCODmeanmass();
	std::vector<double> getTPmedmass();
	std::vector<double> getTPmeanmass();
	std::vector<double> getSolPmedmass();
	std::vector<double> getSolPmeanmass();
	std::vector<double> getTKNmedmass();
	std::vector<double> getTKNmeanmass();
	std::vector<double> getNO2_3medmass();
	std::vector<double> getNO2_3meanmass();
	std::vector<double> getCumedmass();
	std::vector<double> getCumeanmass();
	std::vector<double> getPbmedmass();
	std::vector<double> getPbmeanmass();
	std::vector<double> getZnmedmass();
	std::vector<double> getZnmeanmass();
	

};


	inline std::vector<double> Pollution::getTssmedmass(){return Tssmedmass_kg;}
	inline std::vector<double> Pollution::getTssmeanmass(){return Tssmeanmass_kg;}
	inline std::vector<double> Pollution::getBODmedmass(){return BODmedmass_kg;}
	inline std::vector<double> Pollution::getBODmeanmass(){return BODmeanmass_kg;}
	inline std::vector<double> Pollution::getCODmedmass(){return CODmedmass_kg;}
	inline std::vector<double> Pollution::getCODmeanmass(){return CODmeanmass_kg;}
	inline std::vector<double> Pollution::getTPmedmass(){return TPmedmass_kg;}
	inline std::vector<double> Pollution::getTPmeanmass(){return TPmeanmass_kg;}
	inline std::vector<double> Pollution::getSolPmedmass(){return SolPmedmass_kg;}
	inline std::vector<double> Pollution::getSolPmeanmass(){return SolPmeanmass_kg;}
	inline std::vector<double> Pollution::getTKNmedmass(){return TKNmedmass_kg;}
	inline std::vector<double> Pollution::getTKNmeanmass(){return TKNmeanmass_kg;}
	inline std::vector<double> Pollution::getNO2_3medmass(){return NO2_3medmass_kg;}
	inline std::vector<double> Pollution::getNO2_3meanmass(){return NO2_3meanmass_kg;}
	inline std::vector<double> Pollution::getCumedmass(){return Cumedmass_kg;}
	inline std::vector<double> Pollution::getCumeanmass(){return Cumeanmass_kg;}
	inline std::vector<double> Pollution::getPbmedmass(){return Pbmedmass_kg;}
	inline std::vector<double> Pollution::getPbmeanmass(){return Pbmeanmass_kg;}
	inline std::vector<double> Pollution::getZnmedmass(){return Znmedmass_kg;}
	inline std::vector<double> Pollution::getZnmeanmass(){return Znmeanmass_kg;}


#endif