#include "SpatialOutputWriter.h"


void SpatialOutputWriter::writeOutputFile(std::string indirectory, Inputs *input, DataOrganizer *organizer, DataFolder *folder)
{
	std::ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "Output.dat");
	if (!outfile)
	{
		LogMessage::log("Could not create: " + input->SimulationStringParams["OutputDirectory"] + "Output.dat");
		return;
	}

	double TotalTreeCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	double ShortVegCover_frac = folder->ParamDict["ShortVegetationCover_frac"];
	double TotalCatchmentArea_m2 = input->SimulationNumericalParams["CatchmentArea_m2"];

	double RainInterceptByTree_frac = (input->TotalRain_m - input->RepoDict["totalTreeThruFlow_m"]) / input->TotalRain_m;
	double RainInterceptByShortVeg_frac = (input->TotalRain_m - input->RepoDict["totalShortVegThruFlow_m"]) / input->TotalRain_m;
	double RainOverTreeIntercepted_frac = (input->TotalRain_m - input->RepoDict["totalTreeThruFlow_m"]) * TotalTreeCover_frac / input->TotalRain_m;
	double RainOverShortVegIntercepted_frac = (input->TotalRain_m - input->RepoDict["totalShortVegThruFlow_m"]) * ShortVegCover_frac / input->TotalRain_m;
	double RainInterceptByTree_m3 = (input->TotalRain_m - input->RepoDict["totalTreeThruFlow_m"]) * TotalCatchmentArea_m2 * TotalTreeCover_frac;
	double RainInterceptByShortVeg_m3 = (input->TotalRain_m - input->RepoDict["totalShortVegThruFlow_m"]) * TotalCatchmentArea_m2 * ShortVegCover_frac;

	outfile << " Tree LAI:  " << folder->ParamDict["TreeLAI"] << "  Tree  %: " << TotalTreeCover_frac * 100.0 << " % " << endl
		<< " Shrub LAI: " << folder->ParamDict["ShortVegLAI"] << "  Shrub %: " << ShortVegCover_frac * 100.0 << " % " << endl << endl;

	outfile << " Total precipitation                                      :   " << input->TotalPrecip_m * 1000 << "mm " << endl;
	outfile << " Total rainfall                                           :   " << input->TotalRain_m * 1000 << "mm " << endl;
	outfile << " Total snow water equivalent                              :   " << input->TotalSnow_m * 1000 << "mm " << endl;
	outfile << " Total tree intercepted rain                              :   " << input->RepoDict["totalTreeInterceptionRain_m"] * 1000 << "mm " << endl;
	outfile << " Total short vegetation intercepted rain                  :   " << input->RepoDict["totalShortVegInterceptionRain_m"] * 1000 << "mm " << endl;
	outfile << " Total tree intercepted snow water equivalent             :   " << input->RepoDict["totalTreeInterceptionSnowWE_m"] * 1000 << "mm " << endl;
	outfile << " Total short vegetation intercepted snow water equivalent :   " << input->RepoDict["totalShortVegInterceptionSnowWE_m"] * 1000 << "mm " << endl << endl;



	// Addition of Timestep change code by Pallavi Iyengar
	if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration") {
		// Addition of Timestep change code by Pallavi Iyengar
		calibrate calQ(indirectory, input->SimulationNumericalParams["StartDay"], input->SimulationNumericalParams["EndDay"]);
		std::vector<double> qV = input->QQi;
		calQ.objFunRunoff(qV);
		// End of Timestep change code by Pallavi Iyengar
		std::vector<double> QHourly = input->QQi;
		std::vector<double> QDaily = hourlyToDaily(QHourly);
		std::vector<double> QWeekly = hourlyToWeekly(QHourly);

		// Addition of Monthly Timestep change code by Isira Samarasekera
		std::vector<double> QMonthly = hourlyToMonthly(QHourly);
		// End of Monthly Timestep change code by Isira Samarasekera

		std::vector<double>& QobsHourly = calQ.getQobs();
		std::vector<double>& QobsDaily = calQ.getDailyQobs();
		std::vector<double>& QobsWeekly = calQ.getWeeklyQobs();

		// Addition of Monthly Timestep change code by Isira Samarasekera
		std::vector<double>& QobsMonthly = calQ.getMonthlyQobs();
		// End of Monthly Timestep change code by Isira Samarasekera

		calibrate calQHourly(QHourly, QobsHourly);
		double fCRF1 = calQHourly.getF1();
		double fCRF2 = calQHourly.getF2();
		double fCRF3 = calQHourly.getF3();
		double fVE = calQHourly.getVE();
		double fSE = calQHourly.getE();

		calibrate calQDaily(QDaily, QobsDaily);
		double dailyfCRF1 = calQDaily.getF1();
		double dailyfCRF2 = calQDaily.getF2();
		double dailyfCRF3 = calQDaily.getF3();
		double dailyfVE = calQDaily.getVE();
		double dailyfSE = calQDaily.getE();

		calibrate calQWeekly(QWeekly, QobsWeekly);
		double weeklyfCRF1 = calQWeekly.getF1();
		double weeklyfCRF2 = calQWeekly.getF2();
		double weeklyfCRF3 = calQWeekly.getF3();
		double weeklyfVE = calQWeekly.getVE();
		double weeklyfSE = calQWeekly.getE();

		// Addition of Monthly Timestep change code by Isira Samarasekera
		calibrate calQMonthly(QMonthly, QobsMonthly);
		double monthlyfCRF1 = calQMonthly.getF1();
		double monthlyfCRF2 = calQMonthly.getF2();
		double monthlyfCRF3 = calQMonthly.getF3();
		double monthlyfVE = calQMonthly.getVE();
		double monthlyfSE = calQMonthly.getE();
		// End of Monthly Timestep change code by Isira Samarasekera

		outfile << "Hourly fitness:  " << " VE = " << fVE << "   " << " CRF1 = " << fCRF1 << "   " << " CRF2 = " << fCRF2 << "   " << " CRF3 = " << fCRF3 << endl;
		outfile << "Daily fitness:  " << " VE = " << dailyfVE << "   " << " CRF1 = " << dailyfCRF1 << "   " << " CRF2 = " << dailyfCRF2 << "   " << " CRF3 = " << dailyfCRF3 << endl;
		outfile << "Weekly fitness:  " << " VE = " << weeklyfVE << "   " << " CRF1 = " << weeklyfCRF1 << "   " << " CRF2 = " << weeklyfCRF2 << "   " << " CRF3 = " << weeklyfCRF3 << endl;
		outfile << "Monthly fitness:  " << " VE = " << monthlyfVE << "   " << " CRF1 = " << monthlyfCRF1 << "   " << " CRF2 = " << monthlyfCRF2 << "   " << " CRF3 = " << monthlyfCRF3 << endl << endl;
		// End of Monthly Timestep change code by Isira Samarasekera
		// End of Timestep change code by Pallavi Iyengar
	}
	else
	{
		outfile << " Observed Discharge is not available. The goodness of fit is %ofProjectArea TIValueBin." << endl;
	}

	//Following 4 are required to write to Cumulative o/p file
	/*
	double TotalFlow = totalQQ *1000;
	double TotalOLQ= totalOLQ*1000;
	double TotalBaseFlow = totalSubsurfaceRunoff_mm*1000;
	double TotalUrImpQ = totalUrImpQ*1000;

	*/
	// Addition of Timestep change code by Pallavi Iyengar
	vector<int> dateVec = input->CalendarDates;
	vector<string> timeVec = input->TimeSeries;
	vector <double> QQ, BQi, PervQi, UrImpQQ, precip;
	QQ = input->QQi;
	BQi = input->BQi;
	PervQi = input->PAQi;
	UrImpQQ = input->DCIAQi;
	precip = input->Precip_mpdt;

	
	double aTotalQQ = 0.0;
	double aTotalBQ = 0.0;
	double aTotalPervQ = 0.0;
	double aTotalUrImpQ = 0.0;
	int numTruefolders = 0;
	for (int j = 0; j < input->SimulationNumericalParams["TotalTimeSteps"]; ++j)
	{
		QQ.push_back(0.0);
		BQi.push_back(0.0);
		PervQi.push_back(0.0);
		UrImpQQ.push_back(0.0);
		for (int i = 0; i < organizer->DataDrawers.size(); ++i)
		{
			//std::cout << i << std::endl;
			for (int k = 0; k < organizer->DataDrawers[i].size(); ++k)
			{

				if (organizer->DataDrawers[i][k]->ParamStringDict["Type"]  =="NODATA")
				{
					continue;
				}
				else
				{
					DataFolder *folder = organizer->DataDrawers[i][k];
					QQ[j] += input->QQi[j];
					BQi[j] += input->BQi[j];
					PervQi[j] += input->PAQi[j];
					UrImpQQ[j] += input->DCIAQi[j];
					++numTruefolders;
				}
			}
		}
		QQ[j] /= numTruefolders;
		BQi[j] /= numTruefolders;
		PervQi[j] /= numTruefolders;
		UrImpQQ[j] /= numTruefolders;
		aTotalQQ += QQ[j];
		aTotalBQ += BQi[j];
		aTotalPervQ += PervQi[j];
		aTotalUrImpQ += UrImpQQ[j];
	}
	
	outfile << "Average Total runoff (TotalQ :mm)       :	" << aTotalQQ * 1000 << "		Total pervious area surface flow (TotalOLQ :mm) :    " << aTotalPervQ * 1000 << endl
		<< " Total Baseflow(BQ:mm)           :   " << aTotalBQ * 1000 << "		Total Impervious impervious flow (TotalDCIAQ:mm)     :    " << aTotalUrImpQ * 1000 << endl << endl;
	outfile << "  BQ/TotalQ: " << aTotalBQ / aTotalQQ << "   TotalUrImpQ/TotalQ: " << aTotalUrImpQ / aTotalQQ << endl << endl;
	outfile << " The unit of the following time series output is : m/DT , DT: simulation time interval, such as 1 hour" << endl << endl;
	outfile << "  " << std::setw(10) << "yyyymmdd"
		<< "  " << std::setw(10) << "hh:mm:ss"
		<< "  " << std::setw(10) << "Precip_mpdt "
		<< "  " << std::setw(13) << "AveTQ "
		<< "  " << std::setw(13) << "AveBQ "
		<< "  " << std::setw(13) << "AvePQ "
		<< "  " << std::setw(13) << "AveDCIAQ "
		<< endl << endl;//check this with previous Ver

	

	if (input->SimulationStringParams["OutputTimeStep"] == "Hourly")
	{

		for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
		{
			outfile << "  " << std::setw(10) << dateVec.at(i)
				<< "  " << std::setw(10) << timeVec.at(i)
				<< "  " << std::setw(10) << precip.at(i)
				<< "  " << std::setw(13) << QQ.at(i)
				<< "  " << std::setw(13) << BQi.at(i)
				<< "  " << std::setw(13) << PervQi.at(i)
				<< "  " << std::setw(13) << UrImpQQ.at(i)
				<< endl;
		}
		outfile.close();
	}

	if (input->SimulationStringParams["OutputTimeStep"] == "Daily")
	{
		vector<int>dailydate = getDailyDate(dateVec);
		vector<string> dailyHHMMSS = getDailyHHMMSS(timeVec, dateVec);
		std::vector<double> dailyPrecip = hourlyToDaily(precip), dailyQQ = hourlyToDaily(QQ),
			dailyBQQ = hourlyToDaily(BQi),
			dailyPervQi = hourlyToDaily(PervQi),
			dailyUrImpQQ = hourlyToDaily(UrImpQQ);
		int dailySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / 24);
		for (int i = 0; i < dailySize; i++)
		{
			outfile << "  " << std::setw(10) << dailydate.at(i)
				<< "  " << std::setw(10) << dailyHHMMSS.at(i)
				<< "  " << std::setw(10) << dailyPrecip.at(i)
				<< "  " << std::setw(13) << dailyQQ.at(i)
				<< "  " << std::setw(13) << dailyBQQ.at(i)
				<< "  " << std::setw(13) << dailyPervQi.at(i)
				<< "  " << std::setw(13) << dailyUrImpQQ.at(i)
				<< endl;
		}
		outfile.close();

	}

	if (input->SimulationStringParams["OutputTimeStep"] == "Weekly")
	{
		vector<int>weeklydate = getWeeklyDate(dateVec);
		vector<string> weeklyHHMMSS = getWeeklyHHMMSS(timeVec, dateVec);
		std::vector<double> weeklyQQ = hourlyToWeekly(QQ), weeklyBQQ = hourlyToWeekly(BQi),
			weeklyPervQi = hourlyToWeekly(PervQi),
			weeklyUrImpQQ = hourlyToWeekly(UrImpQQ),
			weeklyprecip = hourlyToWeekly(precip);
		int weeklySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / (24 * 7));
		for (int i = 0; i < weeklySize; i++)
		{
			outfile << "  " << std::setw(10) << weeklydate.at(i)
				<< "  " << std::setw(10) << weeklyHHMMSS.at(i)
				<< "  " << std::setw(10) << weeklyprecip.at(i)
				<< "  " << std::setw(13) << weeklyQQ.at(i)
				<< "  " << std::setw(13) << weeklyBQQ.at(i)
				<< "  " << std::setw(13) << weeklyPervQi.at(i)//does it correspong to pervious
				<< "  " << std::setw(13) << weeklyprecip.at(i)
				<< endl;
		}
		outfile.close();
	}

	// End of Timestep change code by Pallavi Iyengar

	// Addition of Monthly Timestep change code by Isira Samarasekera
	if (input->SimulationStringParams["OutputTimeStep"] == "Monthly")
	{
		vector<int> monthlyDate = getMonthlyDate(dateVec);
		vector<string> monthlyHHMMSS = getMonthlyHHMMSS(timeVec, dateVec);
		std::vector<double> monthlyQQ = hourlyToMonthly(QQ), monthlyBQQ = hourlyToMonthly(BQi),
			monthlyPervQi = hourlyToMonthly(PervQi),
			monthlyUrImpQQ = hourlyToMonthly(UrImpQQ),
			monthlyprecip = hourlyToMonthly(precip);
		int monthlySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / (24 * 30));
		for (int i = 0; i < monthlySize; i++)
		{
			outfile << "  " << std::setw(10) << monthlyDate.at(i)
				<< "  " << std::setw(10) << monthlyHHMMSS.at(i)
				<< "  " << std::setw(10) << monthlyprecip.at(i)
				<< "  " << std::setw(13) << monthlyQQ.at(i)
				<< "  " << std::setw(13) << monthlyBQQ.at(i)
				<< "  " << std::setw(13) << monthlyPervQi.at(i)
				<< "  " << std::setw(13) << monthlyUrImpQQ.at(i)
				<< endl;
		}
		outfile.close();
	}
	// End of Timestep change code by Isira Samarasekera
	outfile.close();
	LogMessage::log("Created: " + input->SimulationStringParams["OutputDirectory"] + "Output.dat");

}

vector<int> SpatialOutputWriter::getDailyDate(const std::vector<int> date)
{
	int size = date.size() / (24);
	std::vector<int> dailyDate(size);

	for (int i = 0; i<size; i++)
	{
		dailyDate[i] = date[i * 24];
	}
	return dailyDate;
}

vector<int> SpatialOutputWriter::getWeeklyDate(const std::vector<int> date)
{
	int size = date.size() / (24 * 7);
	std::vector<int> weeklyDate(size);

	for (int i = 0; i<size; i++)
	{
		weeklyDate[i] = date[i * 24 * 7];
	}
	return weeklyDate;
}

vector<int> SpatialOutputWriter::getMonthlyDate(const std::vector<int> date)
{
	int size = date.size() / (24 * 30);
	std::vector<int> monthlyDate(size);

	for (int i = 0; i<size; i++)
	{
		monthlyDate[i] = date[i * 24 * 30];
	}
	return monthlyDate;
}

vector<string> SpatialOutputWriter::getDailyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{
	int size = date.size() / 24;
	std::vector<string> dailyHHMMSS(size);
	for (int i = 0; i<size; i++)
	{
		dailyHHMMSS[i] = HHMMSS[i * 24];
	}
	return dailyHHMMSS;
}

vector<string> SpatialOutputWriter::getWeeklyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{
	int size = date.size() / (24 * 7);
	std::vector<string> weeklyHrMMSS(size);
	for (int i = 0; i<size; i++)
	{
		weeklyHrMMSS[i] = HHMMSS[i * 24 * 7];
	}
	return weeklyHrMMSS;
}

vector<string> SpatialOutputWriter::getMonthlyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{
	int size = date.size() / (24 * 30);
	std::vector<string> monthlyHrMMSS(size);
	for (int i = 0; i<size; i++)
	{
		monthlyHrMMSS[i] = HHMMSS[i * 24 * 30];
	}
	return monthlyHrMMSS;
}

std::vector<double> SpatialOutputWriter::convertToTimeStep(const std::vector<double>& val, const std::string timeStep) {
	std::vector<double> tmpVal = val;
	if (timeStep == "Daily" || timeStep == "daily")
	{
		tmpVal = hourlyToDaily(tmpVal);
	}
	if (timeStep == "Weekly" || timeStep == "weekly")
	{
		tmpVal = hourlyToWeekly(tmpVal);
	}
	if (timeStep == "Monthly" || timeStep == "monthly")
	{
		tmpVal = hourlyToMonthly(tmpVal);
	}
	return tmpVal;
}

vector<double> SpatialOutputWriter::hourlyToDaily(const vector<double>& input)
{
	int dailySize = input.size() / 24;
	vector<double> dailyValue(dailySize);

	for (int i = 0; i<dailySize; i++)
	{

		dailyValue[i] = hourlyToDailyVal(input, i);
	}

	return dailyValue;
}

double SpatialOutputWriter::hourlyToDailyVal(const vector<double>& input, const int i)
{
	double dailySum = 0.0;
	for (int j = 0; j<24; j++)
	{
		dailySum = dailySum + input[i * 24 + j];
	}
	return dailySum;
}

vector<double> SpatialOutputWriter::hourlyToWeekly(const vector<double>& input)
{
	int weeklySize = input.size() / (24 * 7);
	vector<double> weeklyValue(weeklySize);

	for (int i = 0; i<weeklySize; i++)
	{
		weeklyValue[i] = hourlyToWeeklyVal(input, i);
	}

	return weeklyValue;
}


double SpatialOutputWriter::hourlyToWeeklyVal(const vector<double>& input, const int i)
{
	double weeklySum = 0.0;
	for (int j = 0; j < (24 * 7); j++)
	{
		weeklySum = weeklySum + input[i * 24 * 7 + j];
	}

	return weeklySum;
}

vector<double> SpatialOutputWriter::hourlyToMonthly(const vector<double>& input)
{
	int monthlySize = input.size() / (24 * 30);
	vector<double> monthlyValue(monthlySize);
	for (int i = 0; i<monthlySize; i++)
	{
		monthlyValue[i] = hourlyToMonthlyVal(input, i);
	}
	return monthlyValue;
}

double SpatialOutputWriter::hourlyToMonthlyVal(const vector<double>& input, const int i)
{
	double monthlySum = 0.0;
	for (int j = 0; j < (24 * 30); j++)
	{
		monthlySum = monthlySum + input[i * 24 * 30 + j];
	}

	return monthlySum;
}