#include "StatisticalOutputWriter.h"
#include <iomanip>
#include <iostream>



void StatisticalOutputWriter::writeLAISeries(Inputs *input)
{

	ofstream outfileLAI(input->SimulationStringParams["OutputDirectory"] + "VegetationLAISeries.dat");
	if (!outfileLAI.good())
	{
		LogMessage::log("Could not create: "+input->SimulationStringParams["OutputDirectory"] + "VegetationLAISeries.dat");
		return;
	}
	outfileLAI<<"yyyymmdd   "<<"Hr:Min:Sec "<<"TreeLAI "<<"ShortVegLAI  "<<endl;
	for(int i=0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
	{
		  outfileLAI<<input->CalendarDates[i]<<"   "<<input->TimeSeries[i]<<"    "<<input->TreeLAI[i]<<"     "<<input->ShortVegLAI[i]<<"    "<<endl;
	}
	outfileLAI.close();
	LogMessage::log("Created: " + input->SimulationStringParams["OutputDirectory"] + "VegetationLAISeries.dat");

}

void StatisticalOutputWriter::writeCumulativeFile(Inputs *input, DataFolder* BulkAreafolder)
{
	std::string path;
	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") { //RC,11/30/2018-in future rename to Output_BulkArea if isolated to BulkArea, then have Output.dat for whole prj area
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput.dat"; //For now, naming Output.dat so when no GI is run we get output for prj, which GUI & tests expect
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainGarden") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_RainGarden.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "BioRetention") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_BioRetention.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "GreenRoof") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_GreenRoof.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "PermeablePavement") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_PermeablePavement.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainBarrel") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_RainBarrel.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "TreePit") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_TreePit.dat";
	}
	//added by Li
	else if (BulkAreafolder->ParamStringDict["Type"] == "Swale") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_Swale.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RoofDiscon") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_RoofDiscon.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "InfilTrench") {
		path = input->SimulationStringParams["OutputDirectory"] + "CumOutput_InfilTrench.dat";
	}

	std::ifstream infile(path);
	bool exists = infile.good();

	std::ofstream OutFile(path, std::ios::app);
	if (!OutFile.good())
	{
		LogMessage::log("Could not create: " + path);
		return;
	}
	
	double ImpPercent = BulkAreafolder->ParamDict["TotalImperviousCover_frac"] *100; // TotalImperviousCover_frac == ImperviousCover_frac + ImperviousCoverUnderTreeCanopy_frac
	double PerPercent = BulkAreafolder->ParamDict["TotalPerviousCover_frac"] * 100;
	double DCIA_frac = BulkAreafolder->ParamDict["DCIA_frac"];
    double NoTreeImperviousCover_frac = BulkAreafolder->ParamDict["ImperviousCover_frac"];
	double TreePerviousCover_frac = BulkAreafolder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	double TreeImperviousCover_frac = BulkAreafolder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
    double BareSoilCover_frac = BulkAreafolder->ParamDict["SoilCover_frac"];
	double ShortVegCover_frac = BulkAreafolder->ParamDict["ShortVegetationCover_frac"];
	double TotalFlow_mm = input->RepoDict["runTotalFlow_m"]*1000;
	double TotalPerviousAreaRunoff_mm = input->RepoDict["runTotalPerviousAreaRunoff_m"]*1000;
	double TotalSubsurfaceFlow_mm = input->RepoDict["runTotalSubsurfaceRunoff_m"]*1000;
	double TotalImpAreaRunoff_mm = input->RepoDict["runTotalDCImpAreaRunoff_m"]*1000;
	
	if (!exists)
	{
		OutFile << "  " << std::setw(5) << "TotalImperviousArea_percent"
			<< "  " << std::setw(10) << "Pervious-Percentage "
			<< "  " << std::setw(10) << "TotalImpAreaDirectConnect_frac"
			<< "  " << std::setw(10) << "NoTreeImpAreaDirectConnect_frac "
			<< "  " << std::setw(10) << "TreePerviousCover_frac"
			<< "  " << std::setw(10) << "TreeImperviousCover_frac"
			<< "  " << std::setw(10) << "SoilCover_frac"
			<< "  " << std::setw(10) << "ShortVegCover_frac"
			<< "  " << std::setw(10) << "TotalFlow"
			<< "  " << std::setw(10) << "TotalPerviousRunoff_mm"
			<< "  " << std::setw(10) << "TotalBaseFlow_mm"
			<< "  " << std::setw(10) << "TotalDCIARunoff_mm" << std::endl << std::endl;
	}
	OutFile << "  " << std::setw(15) << ImpPercent
		<< "  " << std::setw(20) << (TreePerviousCover_frac+ TreeImperviousCover_frac)*100
		<< "  " << std::setw(15) << DCIA_frac
		<< "  " << std::setw(15) << NoTreeImperviousCover_frac
		<< "  " << std::setw(20) << TreePerviousCover_frac
		<< "  " << std::setw(15) << TreeImperviousCover_frac
		<< "  " << std::setw(15) << BareSoilCover_frac
		<< "  " << std::setw(15) << ShortVegCover_frac
		<< "  " << std::setw(15) << TotalFlow_mm
		<< "  " << std::setw(15) << TotalPerviousAreaRunoff_mm
		<< "  " << std::setw(15) << TotalSubsurfaceFlow_mm
		<< "  " << std::setw(15) << TotalImpAreaRunoff_mm << std::endl;

	OutFile.close();
	LogMessage::log("Created: " + path);
}

void StatisticalOutputWriter::writeWaterBalanceFile(Inputs *input, DataFolder* BulkAreafolder,DataFolder* folder)
{
	std::string path;
	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") { //RC,11/30/2018-in future rename to Output_BulkArea if isolated to BulkArea, then have Output.dat for whole prj area
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance.dat"; //For now, naming Output.dat so when no GI is run we get output for prj, which GUI & tests expects
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainGarden") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RainGarden.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "BioRetention") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_BioRetention.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "GreenRoof") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_GreenRoof.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "PermeablePavement") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_PermeablePavement.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainBarrel") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RainBarrel.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "TreePit") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_TreePit.dat";
	}
	//Added by Li
	else if (BulkAreafolder->ParamStringDict["Type"] == "RoofDiscon") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_RoofDiscon.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "Swale") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_Swale.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "InfilTrench") {
		path = input->SimulationStringParams["OutputDirectory"] + "WaterBalance_InfilTrench.dat";
	}

	std::ofstream outfileWB(path);
	if (!outfileWB)
	{
		LogMessage::log("Could not create: " + path);
		return;
	}

	int ia = input->RepoDict["iA"];

	double TotalImperviousCover_percent = BulkAreafolder->ParamDict["TotalImperviousCover_frac"] * 100;
	double TotalPerviousCover_percent = BulkAreafolder->ParamDict["TotalPerviousCover_frac"] * 100;
	double TotalImpDirectConnect_frac = BulkAreafolder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"];
	double NoTreeImpAreaDirectConnect_frac = BulkAreafolder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"];
	double TreePerviousCover_frac = BulkAreafolder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	double TreeImperviousCover_frac = BulkAreafolder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	double BareSoilCover_frac = BulkAreafolder->ParamDict["SoilCover_frac"];
	double ShortVegCover_frac = BulkAreafolder->ParamDict["ShortVegetationCover_frac"];
	double totalFlow_mm = input->RepoDict["runTotalFlow_m"] * 1000;
	double totalPerAreaRunoff_mm = input->RepoDict["runTotalPerviousAreaRunoff_m"] * 1000;
	double totalSubsurfaceRunoff_mm = input->RepoDict["runTotalSubsurfaceRunoff_m"] * 1000;
	double totalImpAreaRunoff_mm = input->RepoDict["runTotalDCImpAreaRunoff_m"] * 1000;

	int totalTimeSteps = input->SimulationNumericalParams["TotalTimeSteps"] - 1;


	//cte - should add _mm and _m to totals, which conisder variables present on or in BulkArea folders 
	double totalRain_mm = input->TotalRain_m * 1000;
	double totalSnow_mm = input->TotalSnow_m * 1000;
	double totalPrecip_mm = input->TotalPrecip_m * 1000;
	double totalTreeInterceptionRain_mm = input->RepoDict["totalTreeInterceptionRain_m"] *1000;
	double totalTreeInterceptionSnowWE_mm = input->RepoDict["totalTreeInterceptionSnowWE_m"] * 1000;
	double totalTreeInterception_mm = totalTreeInterceptionRain_mm + totalTreeInterceptionSnowWE_mm;
	double totalShortVegInterceptionRain_mm = input->RepoDict["totalShortVegInterceptionRain_m"] * 1000;
	double totalShortVegInterceptionSnowWE_mm = input->RepoDict["totalShortVegInterceptionSnowWE_m"] * 1000;
	double totalShortVegInt_mm = totalShortVegInterceptionRain_mm + totalShortVegInterceptionSnowWE_mm;
	double totalTreeEvap_mm = input->RepoDict["totalTreeEvap_m"] * 1000;
	double totalShortVegEvap_mm = input->RepoDict["totalShortVegEvap_m"] * 1000;
	double totalTreeThruFlow_mm = input->RepoDict["totalTreeThruFlow_m"] * 1000;
	double totalShortVegThruFlow_mm = input->RepoDict["totalShortVegThruFlow_m"] * 1000;
	double snowOpen_LastTS_mm = input->RepoVecDict["totalSnowOpen_TS_m"][totalTimeSteps] * 1000;
	double snowUnderTree_LastTS_mm = input->RepoVecDict["totalSnowUnderTree_TS_m"][totalTimeSteps] * 1000;
	double snowUnderShortVeg_LastTS_mm = input->RepoVecDict["totalSnowUnderShortVeg_TS_m"][totalTimeSteps] * 1000;
	double totalSnowEvapUnderTree_mm = input->RepoDict["totalSnowEvapUnderTree_m"] * 1000;
	double totalSnowEvapUnderShortVeg_mm = input->RepoDict["totalSnowEvapUnderShortVeg_m"] * 1000;
	double totalSnowEvapOpen_mm = input->RepoDict["totalSnowEvapOpen_m"]*1000;
	double totalWaterOnPerviousArea_mm = input->RepoDict["totalWaterOnPerviousArea_m"] * 1000;
	double totalWaterOnImperviousArea_mm = input->RepoDict["totalWaterOnImperviousArea_m"] * 1000;
	double totalImpDepEvap_mm = input->RepoDict["totalImpDepEvap_m"] * 1000;
	double totalImpDepStor_LastTS_mm = input->RepoVecDict["totalImpDepStor_TS_m"][totalTimeSteps] * 1000;
	double totalImpFlowToSoil_mm = input->RepoDict["totalImpFlowToSoil_m"] * 1000;
	double totalPerDepEvap_mm = input->RepoDict["totalPerDepEvap_m"] * 1000;
	double totalPerDepStor_LastTS_mm = input->RepoVecDict["totalPerDepStore_TS_m"][totalTimeSteps] * 1000;
	double totalPerFlowToInfil_mm = input->RepoDict["totalPerFlowToInfil_m"] * 1000;
	double totalWaterToSoil_mm = input->RepoDict["totalWaterToSoil_m"] * 1000;
	double totalDCIAFlowToOutlet_mm = input->RepoDict["totalDCIAFlowToOutlet_m"] * 1000;
	//cte - 20200928 These outputs modified by CASE student Ramya R.B. but do not seem properly implemented based on my notes, so need to be checked.
	//double totalInfiltration_mm //cte- accumulate totalInfiltration_m sum using of folder->VarVecDict["infilWeighted"][i] * folder->ParamDict["TotalPerviousCover_frac"] as RepoDict in StoreTimeStepTotalsCalc.cpp
	//double totalEvapTrans_mm // cte- accumulate totalEvapTrans_m sum using folder->VarVecDict["rET"][totalTimeSteps] * folder->ParamDict["TotalPerviousCover_frac"] as RepoDict in StoreTimeStepTotalsCalc.cpp;
	double aveSMD_1stTS = folder->VarDict["aveSMD"]; //cte- write first time step of folder->VarDict["aveSMD"] inputTopographyData.getAveSoilMoistureDeficit(AverageIndex, RefParams["T0_m2ph"], RefParams["m"], RefParams["Q0_mph"], RefParams["n"]);
	double unsatZoneStor_1stTS = folder->VarVecDict["unsatZoneStor_TI"][ia]; //cte - need to save 1stTS of folder->VarVecDict["rootZoneStorDef_TI"].push_back(initRootZoneDef); need to use summation like for(int ia=0; ia< input->IndexAreaSize; ia++)   unsatZoneStor_1stTS_sum += folder->VarVecDict["unsatZoneStor_TI"][ia] * input->IndexArea[ia];
	double initRootZoneDef = ((folder->ParamDict["SD0_percent"] / 100) * folder->ParamDict["MSD_m"]);//cte - need to save 1stTS of folder->VarVecDict["rootZoneStorDef_TI"].push_back(initRootZoneDef);
	double rootZoneStor_1stTS = folder->VarVecDict["rootZoneStorDef_TI"][0];//cte - need to save LastTS of sum of folder->VarVecDict["rootZoneStorDef_TI"][ia] weighted by area of watershed in each ia
	double aveSMD_LastTS = folder->VarDict["aveSMD"] - folder->VarDict["unsatZoneQ"] + folder->VarDict["baseFlow"]; //cte - need to save LastTS of folder->VarDict["aveSMD"] = folder->VarDict["aveSMD"] - folder->VarDict["unsatZoneQ"] + folder->VarDict["baseFlow"];
	double unsatZoneStor_LastTS = folder->VarVecDict["unsatZoneStor_TI"][ia];//cte - need to save LastTS of sum of folder->VarVecDict["rootZoneStorDef_TI"][ia] weighted by area of watershed in each ia
	double rootZoneStor_LastTS = folder->VarVecDict["rootZoneStorDef_TI"][ia];

	outfileWB  << "TotalImperviousArea_percent"
		<< "," << "TotalPerviousArea_percent"
		<< "," << "TotalImpAreaDirectConnect_frac"
		<< "," << "NoTreeImpAreaDirectConnect_frac"
		<< "," << "TreePerviousCover_frac"
		<< "," << "TreeImperviousCover_frac"
		<< "," << "ShortVegCover_frac"
		<< "," << "BareSoilCover_frac"
		<< "," << "Total_Precip_(Rain+SWE)_mm"
		<< "," << "Total_TreeIntercept_(Rain+SWE)_mm"
		<< "," << "Total_ShortVegIntercept_(Rain+SWE)_mm"
		<< "," << "Total_TreeEvap_(Rain+SWE)_mm"
		<< "," << "Total_ShortVegEvap_(Rain+SWE)_mm"
		<< "," << "Total_TreeThruFlow_(Rain+SWE)_mm"
		<< "," << "Total_ShortVegThruFlow_(Rain+SWE)_mm"
		<< "," << "SnowWEOpen_LastTS_mm"
		<< "," << "SnowWEUnderTree_LastTS_mm"
		<< "," << "SnowWEUnderShortVeg_LastTS_mm"
		<< "," << "Total_SnowSub_UnderTree_mm"
		<< "," << "Total_SnowSub_UnderShortVeg_mm"
		<< "," << "Total_SnowSub_OpenArea_mm"
		<< "," << "Total_WaterOnPerArea_mm"
		<< "," << "Total_WaterOnImpArea_mm"
		<< "," << "Total_ImperDepEvap_mm"
		<< "," << "ImpDepStor_LastTS_mm"
		<< "," << "Total_ImperFlowToSoil_mm"
		<< "," << "Total_WaterToPerDep+mm"
		<< "," << "Total_PerDepEvap_mm"
		<< "," << "PerDepStor_LastTS_mm"
		<< "," << "Total_FlowToInfil_mm"
		<< "," << "Total_Infiltration_mm"
		<< "," << "Total_RootZoneET_mm"
		<< "," << "AveSMD_1stTS_mm"
		<< "," << "UnsatZoneStor_1stTS_mm"
		<< "," << "RootZoneStor_1stTS_mm"
		<< "," << "AveSMD_LastTS_mm"
		<< "," << "UnsatZoneStor_LastTS_mm"
		<< "," << "RootZoneStor_LastTS_mm"
		<< "," << "m_ScalingParam"
		<< "," << "DWSMI"
		<< "," << "TotalFlow_mm"
		<< "," << "TotalPerAreaRunoff_mm"
		<< "," << "TotalSubsurfaceFlow_mm"
		<< "," << "TotalImpAreaRunoff_mm" 
		<< std::endl << std::endl;

	outfileWB << TotalImperviousCover_percent
		<< "," << TotalPerviousCover_percent
		<< "," << TotalImpDirectConnect_frac
		<< "," << NoTreeImpAreaDirectConnect_frac
		<< "," << TreePerviousCover_frac
		<< "," << TreeImperviousCover_frac
		<< "," << ShortVegCover_frac
		<< "," << BareSoilCover_frac
		<< "," << totalPrecip_mm
		<< "," << totalTreeInterception_mm * (TreePerviousCover_frac + TreeImperviousCover_frac)
		<< "," << totalShortVegInt_mm * ShortVegCover_frac
		<< "," << totalTreeEvap_mm * (TreePerviousCover_frac + TreeImperviousCover_frac)
		<< "," << totalShortVegEvap_mm * ShortVegCover_frac
		<< "," << totalTreeThruFlow_mm * (TreePerviousCover_frac + TreeImperviousCover_frac)
		<< "," << totalShortVegThruFlow_mm * ShortVegCover_frac
		<< "," << snowOpen_LastTS_mm * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac))
		<< "," << snowUnderTree_LastTS_mm * (TreePerviousCover_frac + TreeImperviousCover_frac)
		<< "," << snowUnderShortVeg_LastTS_mm * ShortVegCover_frac
		<< "," << totalSnowEvapUnderTree_mm * (TreePerviousCover_frac + TreeImperviousCover_frac)
		<< "," << totalSnowEvapUnderShortVeg_mm * ShortVegCover_frac
		<< "," << totalSnowEvapOpen_mm * (1 - (TreePerviousCover_frac + TreeImperviousCover_frac))
		<< "," << totalWaterOnPerviousArea_mm * (TotalPerviousCover_percent / 100)
		<< "," << totalWaterOnImperviousArea_mm * (TotalImperviousCover_percent / 100)
		<< "," << totalImpDepEvap_mm * (TotalImperviousCover_percent / 100)
		<< "," << totalImpDepStor_LastTS_mm * (TotalImperviousCover_percent / 100)
		<< "," << totalImpFlowToSoil_mm * (TotalImperviousCover_percent / 100)
		<< "," << totalWaterToSoil_mm * (TotalPerviousCover_percent / 100)
		<< "," << totalPerDepEvap_mm * (TotalPerviousCover_percent / 100)
		<< "," << totalPerDepStor_LastTS_mm * (TotalPerviousCover_percent / 100)
		<< "," << totalPerFlowToInfil_mm * (TotalPerviousCover_percent / 100)
		<< "," << "0" // totalInfiltration //cte - accumulate timeseries of folder->VarVecDict["infilWeighted"][i] * folder->ParamDict["TotalPerviousCover_frac"] in StoreTimeStepTotalsCalc.cpp
		<< "," << "0" // totalEvapTrans * 1000 //totalEvapTrans * 1000 //cte - need to accumulate timeseries of folder->VarVecDict["rET"][i] * folder->ParamDict["TotalPerviousCover_frac"] in StoreTimeStepTotalsCalc.cpp
		<< "," << aveSMD_1stTS * 1000 //cte - aveSMD_1stTS * 1000 //cte - need to save 1stTS of AveSMD = inputTopographyData.getAveSoilMoistureDeficit(AverageIndex, RefParams["T0_m2ph"], RefParams["m"], RefParams["Q0_mph"], RefParams["n"]);
		<< "," << unsatZoneStor_1stTS * 1000 //cte - need to save 1stTS of sum of folder->VarVecDict["unsatZoneStor_TI"][ia] weighted by area of watershed in each ia
		<< "," << rootZoneStor_1stTS * 1000 //cte - need to save 1stTS of folder->VarVecDict["rootZoneStorDef_TI"].push_back(initRootZoneDef);
		<< "," << aveSMD_LastTS * 1000 //cte - need to save LastTS of folder->VarDict["aveSMD"] = folder->VarDict["aveSMD"] - folder->VarDict["unsatZoneQ"] + folder->VarDict["baseFlow"];
		<< "," << unsatZoneStor_LastTS * 1000 //cte - need to save LastTS of sum of folder->VarVecDict["unsatZoneStor_TI"][ia] weighted by area of watershed in each ia
		<< "," << rootZoneStor_LastTS * 1000 //cte - need to save LastTS of sum of folder->VarVecDict["rootZoneStorDef_TI"][ia] weighted by area of watershed in each ia
		<< "," << "0" //M //cte - get value, but does not need to be written out for water balance
		<< "," << "0" //DWSMI //cte - get value, but does not need to be written out for water balance
		<< "," << totalFlow_mm //sum of terms below, adjusted for fraction of area
		<< "," << totalPerAreaRunoff_mm //PAQ multiplied by fPervious_total in FlowSumCalc.cpp
		<< "," << totalSubsurfaceRunoff_mm //BQ multiplied by fPervious_total in FlowSumCalc.cpp
		<< "," << totalImpAreaRunoff_mm //impRunoffDCIA multiplied by folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] in ImperviousDepressionStorageCalc.cpp
		<< std::endl;

	outfileWB.close();
	LogMessage::log("Created: " + path);
}

void StatisticalOutputWriter::writeCaliQobs(Inputs *input, std::string indirectory){
	ofstream outfileCaliQobs(input->SimulationStringParams["OutputDirectory"] + "CaliQobs.dat");
	if (!outfileCaliQobs.good())
	{
		LogMessage::log("Could not create: " + input->SimulationStringParams["OutputDirectory"] + "CaliQobs.dat");
		return;
	}

	calibrate _qobs(indirectory, input->SimulationNumericalParams["StartDay"], input->SimulationNumericalParams["EndDay"]);
	std::vector<double>& caliQobs = _qobs.getQobs();
	caliQobs = convertToTimeStep(caliQobs, input->SimulationStringParams["CalibrationTimeStep"]);//chk

	outfileCaliQobs << caliQobs.size() << endl;
	for (unsigned int i = 0; i<caliQobs.size(); i++)
		outfileCaliQobs << setiosflags(ios::fixed)
		<< setiosflags(ios::showpoint)
		<< setprecision(9)
		<< caliQobs[i] << endl;

	outfileCaliQobs.close();
	LogMessage::log("Created: " + input->SimulationStringParams["OutputDirectory"] + "CaliQobs.dat");
}
//

void StatisticalOutputWriter::writeCaliTotalQQ(Inputs *input){
	ofstream outfile(input->SimulationStringParams["OutputDirectory"] + "CaliTotalQQ.dat");
	if (!outfile.good())
	{
		LogMessage::log("Could not create: "+ input->SimulationStringParams["OutputDirectory"] + "CaliTotalQQ.dat");
		return;
	}

	std::vector<double> caliTotalQQ = input->QQi;
	caliTotalQQ = convertToTimeStep(caliTotalQQ, input->SimulationStringParams["CalibrationTimeStep"]);

	outfile << caliTotalQQ.size() << endl;
	for (unsigned int i = 0; i<caliTotalQQ.size(); i++)
		outfile << setiosflags(ios::fixed)
		<< setiosflags(ios::showpoint)
		<< setprecision(9)
		<< caliTotalQQ[i] << endl;

	outfile.close();
	LogMessage::log("Created: " + input->SimulationStringParams["OutputDirectory"] + "CaliTotalQQ.dat");
}

std::vector<double> StatisticalOutputWriter::convertToTimeStep(const std::vector<double>& val, const std::string timeStep){
	std::vector<double> tmpVal = val;
	if (timeStep == "Daily" || timeStep == "daily")
	{
		tmpVal = hourlyToDaily(tmpVal);
	}
	if (timeStep == "Weekly" || timeStep == "weekly")
	{
		tmpVal = hourlyToWeekly(tmpVal);
	}
	if (timeStep == "Monthly" || timeStep == "monthly")
	{
		tmpVal = hourlyToMonthly(tmpVal);
	}
	return tmpVal;
}

vector<double> StatisticalOutputWriter::hourlyToDaily(const vector<double>& input)
{
	int dailySize = input.size() / 24;
	vector<double> dailyValue(dailySize);

	for (int i = 0; i<dailySize; i++)
	{
		
		dailyValue[i] = hourlyToDailyVal(input, i);
	}

	return dailyValue;
}

double StatisticalOutputWriter::hourlyToDailyVal(const vector<double>& input, const int i)
{
		double dailySum = 0.0;
		for (int j = 0; j<24; j++)
		{
			dailySum = dailySum + input[i * 24 + j];
		}
		return dailySum;
}

vector<double> StatisticalOutputWriter::hourlyToWeekly(const vector<double>& input)
{
	int weeklySize = input.size() / (24 * 7);
	vector<double> weeklyValue(weeklySize);

	for (int i = 0; i<weeklySize; i++)
	{
		weeklyValue[i] = hourlyToWeeklyVal(input, i);
	}

	return weeklyValue;
}


double StatisticalOutputWriter::hourlyToWeeklyVal(const vector<double>& input, const int i)
{
	double weeklySum = 0.0;
	for (int j = 0; j < (24 * 7); j++)
	{
		weeklySum = weeklySum + input[i * 24 * 7 + j];
	}

	return weeklySum;
}

vector<double> StatisticalOutputWriter::hourlyToMonthly(const vector<double>& input)
{
	int monthlySize = input.size() / (24 * 30);
	vector<double> monthlyValue(monthlySize);
	for (int i = 0; i<monthlySize; i++)
	{
		monthlyValue[i] = hourlyToMonthlyVal(input, i);
	}
	return monthlyValue;
}

double StatisticalOutputWriter::hourlyToMonthlyVal(const vector<double>& input, const int i)
{
	double monthlySum = 0.0; 
	for (int j = 0; j < (24 * 30); j++)
	{
		monthlySum = monthlySum + input[i * 24 * 30 + j];
	}

	return monthlySum;
}

void StatisticalOutputWriter::writePollutionFile(std::string indirectory, Inputs *input)
{
	Pollution poll(input->SimulationNumericalParams["TotalTimeSteps"], input->SimulationNumericalParams["CatchmentArea_m2"], indirectory);
	if (!poll.PollutionDataFileExists)
	{
		LogMessage::log("Pollutiondata.dat can not be found/read. Should be in dir: "+indirectory);
		return;
	}
	std::vector<double> oQi, uQi;
	oQi = input->PAQi;
	uQi = input->DCIAQi;
	poll.calPollutionData(oQi, uQi);
	poll.writePollutiondata(oQi, uQi, input->SimulationStringParams["OutputDirectory"]);
	LogMessage::log("Created " + input->SimulationStringParams["OutputDirectory"] + "Pollution.dat");
}

void StatisticalOutputWriter::writeOutputFile(std::string indirectory,Inputs *input, DataFolder* BulkAreafolder)
{	
	std::string path;
	if (BulkAreafolder->ParamStringDict["Type"] == "BulkArea") { //RC,11/30/2018-in future rename to Output_BulkArea if isolated to BulkArea, then have Output.dat for whole prj area
		path = input->SimulationStringParams["OutputDirectory"] + "Output.dat"; //For now, naming Output.dat so when no GI is run we get output for prj, which GUI & tests expect
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainGarden") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_RainGarden.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "BioRetention") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_BioRetention.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "GreenRoof") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_GreenRoof.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "PermeablePavement") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_PermeablePavement.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "RainBarrel") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_RainBarrel.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "TreePit") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_TreePit.dat";
	}
	//added by Li
	else if (BulkAreafolder->ParamStringDict["Type"] == "RoofDiscon") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_RoofDiscon.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "Swale") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_Swale.dat";
	}
	else if (BulkAreafolder->ParamStringDict["Type"] == "InfilTrench") {
		path = input->SimulationStringParams["OutputDirectory"] + "Output_InfilTrench.dat";
	}

	std::ofstream outfile(path);
	if (!outfile)
	{
		LogMessage::log("Could not create " + path);
		return;
	}

	double TotalTreeCover_frac = BulkAreafolder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + BulkAreafolder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	double ShortVegCover_frac = BulkAreafolder->ParamDict["ShortVegetationCover_frac"];
	double TotalCatchmentArea_m2 = input->SimulationNumericalParams["CatchmentArea_m2"];

	double RainInterceptByTree_frac		=	(input->TotalRain_m - input->RepoDict["totalTreeThruFlow_m"])/input->TotalRain_m;
	double RainInterceptByShortVeg_frac	=	(input->TotalRain_m  - input->RepoDict["totalShortVegThruFlow_m"] ) / input->TotalRain_m;
	double RainOverTreeIntercepted_frac		=	(input->TotalRain_m  - input->RepoDict["totalTreeThruFlow_m"] ) * TotalTreeCover_frac/input->TotalRain_m;
	double RainOverShortVegIntercepted_frac	=	(input->TotalRain_m  - input->RepoDict["totalShortVegThruFlow_m"]) * ShortVegCover_frac /input->TotalRain_m ;
   	double RainInterceptByTree_m3		=	(input->TotalRain_m  - input->RepoDict["totalTreeThruFlow_m"] ) * TotalCatchmentArea_m2 * TotalTreeCover_frac;
	double RainInterceptByShortVeg_m3	=	(input->TotalRain_m  - input->RepoDict["totalShortVegThruFlow_m"]) * TotalCatchmentArea_m2 * ShortVegCover_frac ;

	outfile <<" Tree LAI:  "<< BulkAreafolder->ParamDict["TreeLAI"]<<"  Tree  %: " <<TotalTreeCover_frac*100.0<<" % "<<endl
		    <<" Shrub LAI: "<< BulkAreafolder->ParamDict["ShortVegLAI"] <<"  Shrub %: " <<ShortVegCover_frac*100.0 <<" % "<<endl<<endl;

	outfile <<" Total precipitation                                      :   " <<input->TotalPrecip_m*1000<<"mm " <<endl;
    outfile <<" Total rainfall                                           :   " <<input->TotalRain_m*1000<<"mm " <<endl;
    outfile <<" Total snow water equivalent                              :   " <<input->TotalSnow_m*1000<<"mm " <<endl;
    outfile <<" Total tree intercepted rain                              :   " <<input->RepoDict["totalTreeInterceptionRain_m"]*1000<<"mm " <<endl;
	outfile <<" Total short vegetation intercepted rain                  :   " <<input->RepoDict["totalShortVegInterceptionRain_m"]*1000<<"mm " <<endl;
	outfile <<" Total tree intercepted snow water equivalent             :   " <<input->RepoDict["totalTreeInterceptionSnowWE_m"]*1000<<"mm " <<endl;
	outfile <<" Total short vegetation intercepted snow water equivalent :   " <<input->RepoDict["totalShortVegInterceptionSnowWE_m"]*1000<<"mm " <<endl<<endl;

	std::vector<double> QH;
	std::vector<double> QD;
	std::vector<double> QW;
	std::vector<double> QM;

	// Addition of Timestep change code by Pallavi Iyengar
	if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration"){
		// Addition of Timestep change code by Pallavi Iyengar
		calibrate calQ(indirectory, input->SimulationNumericalParams["StartDay"], input->SimulationNumericalParams["EndDay"]);
		std::vector<double> qV = input->QQi;
		calQ.objFunRunoff(qV);
		// End of Timestep change code by Pallavi Iyengar
		std::vector<double> QHourly = input->QQi;
		std::vector<double> QDaily = hourlyToDaily(QHourly);
		std::vector<double> QWeekly = hourlyToWeekly(QHourly); 

		// Addition of Monthly Timestep change code by Isira Samarasekera
		std::vector<double> QMonthly = hourlyToMonthly(QHourly);
		// End of Monthly Timestep change code by Isira Samarasekera

		std::vector<double>& QobsHourly = calQ.getQobs();
		std::vector<double>& QobsDaily = calQ.getDailyQobs();
		std::vector<double>& QobsWeekly = calQ.getWeeklyQobs();

		// Addition of Monthly Timestep change code by Isira Samarasekera
		std::vector<double>& QobsMonthly = calQ.getMonthlyQobs();
		// End of Monthly Timestep change code by Isira Samarasekera
		QH = QobsHourly;
		QD = QobsDaily;
		QW = QobsWeekly;
		QM = QobsMonthly;

		calibrate calQHourly(QHourly, QobsHourly);
		double fCRF1 = calQHourly.getF1();
		double fCRF2 = calQHourly.getF2();
		double fCRF3 = calQHourly.getF3();
		double fVE = calQHourly.getVE();
		double fSE = calQHourly.getE();

		calibrate calQDaily(QDaily, QobsDaily);
		double dailyfCRF1 = calQDaily.getF1();
		double dailyfCRF2 = calQDaily.getF2();
		double dailyfCRF3 = calQDaily.getF3();
		double dailyfVE = calQDaily.getVE();
		double dailyfSE = calQDaily.getE();

		calibrate calQWeekly(QWeekly, QobsWeekly);
		double weeklyfCRF1 = calQWeekly.getF1();
		double weeklyfCRF2 = calQWeekly.getF2();
		double weeklyfCRF3 = calQWeekly.getF3();
		double weeklyfVE = calQWeekly.getVE();
		double weeklyfSE = calQWeekly.getE();

		// Addition of Monthly Timestep change code by Isira Samarasekera
		calibrate calQMonthly(QMonthly, QobsMonthly);
		double monthlyfCRF1 = calQMonthly.getF1();
		double monthlyfCRF2 = calQMonthly.getF2();
		double monthlyfCRF3 = calQMonthly.getF3();
		double monthlyfVE = calQMonthly.getVE();
		double monthlyfSE = calQMonthly.getE();
		// End of Monthly Timestep change code by Isira Samarasekera

		outfile << "Hourly fitness:  " << " VE = " << fVE << "   " << " CRF1 = " << fCRF1 << "   " << " CRF2 = " << fCRF2 << "   " << " CRF3 = " << fCRF3  <<endl;
		outfile << "Daily fitness:  " << " VE = " << dailyfVE << "   " << " CRF1 = " << dailyfCRF1 << "   " << " CRF2 = " << dailyfCRF2 << "   " << " CRF3 = " << dailyfCRF3 << endl;
		outfile << "Weekly fitness:  " << " VE = " << weeklyfVE << "   " << " CRF1 = " << weeklyfCRF1 << "   " << " CRF2 = " <<weeklyfCRF2 << "   " << " CRF3 = " <<weeklyfCRF3 << endl;
		outfile << "Monthly fitness:  " << " VE = " << monthlyfVE << "   " << " CRF1 = " << monthlyfCRF1 << "   " << " CRF2 = " << monthlyfCRF2 << "   " << " CRF3 = " << monthlyfCRF3 << endl << endl;
		// End of Monthly Timestep change code by Isira Samarasekera
		// End of Timestep change code by Pallavi Iyengar
	}
	else
	{
		outfile << " Observed Discharge is not available. The goodness of fit is %ofProjectArea TIValueBin." << endl;
	}

	//Following 4 are required to write to Cumulative o/p file
/*	
     double TotalQ_mm = totalQQ *1000;
	double TotalPerviousAreaQ_mm= totalOLQ*1000;
	double TotalBaseFlowQ_mm = totalSubsurfaceRunoff_mm*1000;
	double TotalDCIAQ_mm = totalUrImpQ*1000;

*/
// Addition of Timestep change code by Pallavi Iyengar
	vector<int> dateVec = input->CalendarDates;
	vector<string> timeVec = input->TimeSeries;
	vector <double> QQ, BQi, PervQi, UrImpQQ, precip;
	QQ = input->QQi;
	BQi = input->BQi;
	PervQi = input->PAQi;
	UrImpQQ = input->DCIAQi;
	precip = input->Precip_mpdt;


	
		outfile << " Total runoff (TotalQ :mm)       :	" << input->RepoDict["runTotalFlow_m"] * 1000 << "		Total pervious area surface flow (TotalPerviousAreaQ_mm :mm) :    " << input->RepoDict["runTotalPerviousAreaRunoff_m"] * 1000 << endl
			<< " Total Baseflow(BQ:mm)           :   " << input->RepoDict["runTotalSubsurfaceRunoff_m"] * 1000 << "		Total Impervious impervious flow (TotalDCIAQ:mm)     :    " << input->RepoDict["runTotalDCImpAreaRunoff_m"] * 1000 << endl << endl;
		outfile << "  BQ/TotalQ: " << input->RepoDict["runTotalSubsurfaceRunoff_m"] / input->RepoDict["runTotalFlow_m"] << "   TotalDCIAQ_mm/TotalQ: " << input->RepoDict["runTotalDCImpAreaRunoff_m"] / input->RepoDict["runTotalFlow_m"] << endl << endl;
		outfile << " The unit of the following time series output is : m/DT , DT: simulation time interval, such as 1 hour" << endl << endl;

		outfile << "  " << std::setw(10) << "yyyymmdd"
			<< "  " << std::setw(10) << "hh:mm:ss"
			<< "  " << std::setw(10) << "Precip "
			<< "  " << std::setw(13) << "TotalQ "
			<< "  " << std::setw(13) << "BaseQ "
			<< "  " << std::setw(13) << "PerviousQ "
			<< "  " << std::setw(13) << "DCIAQ ";
			if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
			{
				outfile << "  " << std::setw(13) << "Qobs ";
			}
			outfile<< endl << endl;//check this with previous Ver



	if (input->SimulationStringParams["OutputTimeStep"] == "Hourly")
	{
	
		for (int i = 0; i<input->SimulationNumericalParams["TotalTimeSteps"]; i++)
		{
			outfile << "  " << std::setw(10) << dateVec.at(i)
				<< "  " << std::setw(10) << timeVec.at(i)
				<< "  " << std::setw(10) << precip.at(i)
				<< "  " << std::setw(13) << QQ.at(i)
				<< "  " << std::setw(13) << BQi.at(i)
				<< "  " << std::setw(13) << PervQi.at(i)
				<< "  " << std::setw(13) << UrImpQQ.at(i);
				if(input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QH[i];
				}
				outfile<< endl;
		}
		outfile.close();
	}

	if (input->SimulationStringParams["OutputTimeStep"] == "Daily")
	{
		vector<int>dailydate = getDailyDate(dateVec);
		vector<string> dailyHHMMSS = getDailyHHMMSS(timeVec, dateVec);
		std::vector<double> dailyPrecip = hourlyToDaily(precip), dailyQQ = hourlyToDaily(QQ),
			dailyBQQ = hourlyToDaily(BQi),	
			dailyPervQi = hourlyToDaily(PervQi),
			dailyUrImpQQ = hourlyToDaily(UrImpQQ);
		int dailySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / 24);
		for (int i = 0; i < dailySize; i++)
		{
			outfile << "  " << std::setw(10) << dailydate.at(i)
				<< "  " << std::setw(10) << dailyHHMMSS.at(i)
				<< "  " << std::setw(10) << dailyPrecip.at(i)
				<< "  " << std::setw(13) << dailyQQ.at(i)
				<< "  " << std::setw(13) << dailyBQQ.at(i)
				<< "  " << std::setw(13) << dailyPervQi.at(i)
				<< "  " << std::setw(13) << dailyUrImpQQ.at(i);
				if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QD[i];
				}
			outfile << endl;
		}
		outfile.close();

	}

	if (input->SimulationStringParams["OutputTimeStep"] == "Weekly")
	{
		vector<int>weeklydate = getWeeklyDate(dateVec);
		vector<string> weeklyHHMMSS = getWeeklyHHMMSS(timeVec, dateVec);
		std::vector<double> weeklyQQ = hourlyToWeekly(QQ), weeklyBQQ = hourlyToWeekly(BQi),
			weeklyPervQi = hourlyToWeekly(PervQi),
			weeklyUrImpQQ = hourlyToWeekly(UrImpQQ),
			weeklyprecip = hourlyToWeekly(precip);
			int weeklySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / (24 * 7));
			for (int i = 0; i < weeklySize; i++)
			{
				outfile << "  " << std::setw(10) << weeklydate.at(i)
					<< "  " << std::setw(10) << weeklyHHMMSS.at(i)
					<< "  " << std::setw(10) << weeklyprecip.at(i)
					<< "  " << std::setw(13) << weeklyQQ.at(i)
					<< "  " << std::setw(13) << weeklyBQQ.at(i)
					<< "  " << std::setw(13) << weeklyPervQi.at(i)//does it correspong to pervious
					<< "  " << std::setw(13) << weeklyprecip.at(i);
				if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QW[i];
				}
			outfile << endl;
		}
			outfile.close();
	}
	
	// End of Timestep change code by Pallavi Iyengar

	// Addition of Monthly Timestep change code by Isira Samarasekera
	if (input->SimulationStringParams["OutputTimeStep"] == "Monthly")
	{
		vector<int> monthlyDate = getMonthlyDate(dateVec);
		vector<string> monthlyHHMMSS = getMonthlyHHMMSS(timeVec, dateVec);
		std::vector<double> monthlyQQ = hourlyToMonthly(QQ), monthlyBQQ = hourlyToMonthly(BQi),
			monthlyPervQi = hourlyToMonthly(PervQi),
			monthlyUrImpQQ = hourlyToMonthly(UrImpQQ),
			monthlyprecip = hourlyToMonthly(precip);
		int monthlySize = (int)(input->SimulationNumericalParams["TotalTimeSteps"] / (24 * 30));
		for (int i = 0; i < monthlySize; i++)
		{
			outfile << "  " << std::setw(10) << monthlyDate.at(i)
				<< "  " << std::setw(10) << monthlyHHMMSS.at(i)
				<< "  " << std::setw(10) << monthlyprecip.at(i)
				<< "  " << std::setw(13) << monthlyQQ.at(i)
				<< "  " << std::setw(13) << monthlyBQQ.at(i)
				<< "  " << std::setw(13) << monthlyPervQi.at(i)
				<< "  " << std::setw(13) << monthlyUrImpQQ.at(i);
				if (input->SimulationStringParams["CalibrationTimeStep"] != "NoCalibration")
				{
					outfile << "  " << std::setw(13) << QM[i];
				}
			outfile << endl;
		}
		outfile.close();
	}
	// End of Timestep change code by Isira Samarasekera
	outfile.close();
	LogMessage::log("Created " + path);

}

vector<int> StatisticalOutputWriter::getDailyDate(const std::vector<int> date)
{
	int size = date.size() / (24);
	std::vector<int> dailyDate(size);

	for (int i = 0; i<size; i++)
	{
		dailyDate[i] = date[i * 24];
	}
	return dailyDate;
}

vector<int> StatisticalOutputWriter::getWeeklyDate(const std::vector<int> date)
{
	int size = date.size() / (24 * 7);
	std::vector<int> weeklyDate(size);

	for (int i = 0; i<size; i++)
	{
		weeklyDate[i] = date[i * 24 * 7];
	}
	return weeklyDate;
}

vector<int> StatisticalOutputWriter::getMonthlyDate(const std::vector<int> date)
{
	int size = date.size() / (24 * 30);
	std::vector<int> monthlyDate(size);

	for (int i = 0; i<size; i++)
	{
		monthlyDate[i] = date[i * 24 * 30];
	}
	return monthlyDate;
}

vector<string> StatisticalOutputWriter::getDailyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{
	int size = date.size() / 24;
	std::vector<string> dailyHHMMSS(size);
	for (int i = 0; i<size; i++)
	{
		dailyHHMMSS[i] = HHMMSS[i * 24];
	}
	return dailyHHMMSS;
}

vector<string> StatisticalOutputWriter::getWeeklyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{ 
	int size = date.size() / (24 * 7);
	std::vector<string> weeklyHrMMSS(size);
	for (int i = 0; i<size; i++)
	{
		weeklyHrMMSS[i] = HHMMSS[i * 24 * 7];
	}
	return weeklyHrMMSS;
}

vector<string> StatisticalOutputWriter::getMonthlyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date)
{
	int size = date.size() / (24 * 30);
	std::vector<string> monthlyHrMMSS(size);
	for (int i = 0; i<size; i++)
	{
		monthlyHrMMSS[i] = HHMMSS[i * 24 * 30];
	}
	return monthlyHrMMSS;
}

