	#ifndef OUTPUT_H
#define OUTPUT_H
//////////////////////////////////////////////////////////////////////////////////////
//																		            //
// StatisticalOutputWriter.h - Writes ouptut files for running V6 in semi-distributed mode // 
//							format identical to V5						            //											   
// ver 1.0                                                                          //
// Language:    Visual C++ 2010                                                     //
// Platform:    MacBookPro5,4  Win7										            //
// Application: HydroPrime/iHydro Version6, Summer 2012                             //
// Programmer:  				            //																	   	
//////////////////////////////////////////////////////////////////////////////////////
/*
  Origin:
  ==================
  Program: Version 5 
  
  Source Code File(s): watershed.h
 
             
   Author(s): Jun Wang, jwang06@mailbox.syr.edu, Original author V4/V4.5
              Yang Yang, yyang31@syr.edu, V5
			  
						*incorporated into new architecture
  Creation Date: Summer 2000

  Publication(s): PHDThesis

  Module Operations: 
  ==================
  ...

  Public Interface:
  =================
     StatisticalOutputWriter(Inputs *input, Repo *repo, std::string indirectory, std::string input->SimulationStringParams["OutputDirectory"]);
	 void writeOutput();

  Build Process:
  ==============
  Required files
	-StatisticalOutputWriter.cpp, InputsRespository.h, ResultsRepository.h, IOutputWriter.h
				
  Build commands 
    - devenv hydroprime.sln
   
  Maintenance History:
  ====================

 */

#include <string>
#include <vector>
#include "Pollution.h"
#include "calibrate.h"
#include "../Inputs/Inputs.h"
#include "../DataFolder.h"
#include <fstream>

using namespace std;

class StatisticalOutputWriter
{
	public:
     	static void writeOutputFile(std::string indirectory, Inputs *input, DataFolder* BulkAreafolder);
		static void writePollutionFile(std::string indirectory, Inputs *input);
		static void writeCumulativeFile(Inputs *input, DataFolder* BulkAreafolder);
		static void writeWaterBalanceFile(Inputs *input, DataFolder* BulkAreafolder, DataFolder* folder);
		static void writeLAISeries(Inputs *input);
		static void writeCaliQobs(Inputs *input, std::string indirectory);
		static void writeCaliTotalQQ(Inputs *input);

		static vector<double> hourlyToDaily(const vector<double>& input);
		static double hourlyToDailyVal(const vector<double>& input, const int i);
		static vector<double> hourlyToWeekly(const vector<double>& input);
		static double hourlyToWeeklyVal(const vector<double>& input, const int i);
		static vector<double> hourlyToMonthly(const vector<double>& input);
		static double hourlyToMonthlyVal(const vector<double>& input, const int i);
		static std::vector<double> convertToTimeStep(const std::vector<double>& val, const std::string timeStep);
		static vector<int> getDailyDate(const std::vector<int> date);	
		static vector<int> getWeeklyDate(const std::vector<int> date);
		static vector<int>  getMonthlyDate(const std::vector<int> date);
		static vector<string> getDailyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date);
		static vector<string> getWeeklyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date);
		static vector<string> getMonthlyHHMMSS(const std::vector<string> HHMMSS, const std::vector<int> date);
};



#endif