////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  calibrate.cpp --  model calibration for total runoff                    //
//                                                                            //
//                                                                            //
//                                                                            //
//  Language:     Visual C++, ver 6.0                                         //
//  Platform:     Pentium II 400, Window 98, Win NT 4.0                       //
//  Application:  PHDThesis, Summer 2000                                      //
//  Author:       Jun Wang                                                    //
//                jwang06@mailbox.syr.edu                                     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

#include "calibrate.h"
#include <math.h>
#include <iostream>
#include <fstream>
#include <string>
#include <iostream>
#include <sstream>


calibrate::calibrate(std::string directory)
{
	CRF1 = 0.0;
	CRF2 = 0.0;
	CRF3 = 0.0;
	SE = 0.0;
	VE=0.0;

	double tempQ =0.0;
	
	std::ifstream readQobs(directory + "Qobs.dat");
	if(!readQobs)
	{
		std::cout<<"Qobs.dat could not be opened!"<<std::endl;
		return;
	}
	
	

	int size = 0;
	double tempQobs;

	 
	readQobs>>size;
	for(int i = 0; i < size; ++i)
	{
		readQobs>>tempQobs;
		_Qobs.push_back(tempQobs);
	}
	
	readQobs.close();

	// Addition of Timestep changes by Pallavi Iyengar
	calcDailyQobs();
	calcWeeklyQobs();
	// End of Timestep changes by Pallavi Iyengar

	//  Addition of Monthly Timestep change code by Isira Samarasekera
	calcMonthlyQobs();
	// End of Monthly Timestep changes by Isira Samarasekera
}

calibrate::calibrate(std::string directory, double startDate, double endDate)
{

	CRF1 = 0.0;
	CRF2 = 0.0;
	CRF3 = 0.0;
	SE = 0.0;
	VE=0.0;

	double tempQ =0.0;
	int size =0;
	std::ifstream readQobs(directory + "Qobs.dat");


	if(!readQobs)
	{
		std::cout<<"Qobs.dat could not be opened!"<<std::endl;
		return;
	}
	
	

	double date = -1.0;
	double tempQobs;
	std::string temp; 

	// getting the header line and the blank line out of the way
	getline(readQobs, temp);
	//getline(readQobs, temp);									// this second getline handles the extra spaces between the header and the start of data,
																// right now it has been commented out because the GUI isn't producing files with the extra space
	// reading in a CSV
	while (readQobs.good())
	{
		getline(readQobs, temp, ',');
		std::istringstream ss(temp);
		ss >> date;
		if (date <= endDate)
		{
			if (date >= startDate)
			{
				getline(readQobs, temp, ',');
				getline(readQobs, temp, '\n');
				std::istringstream qobs(temp);
				qobs >> tempQobs;
				_Qobs.push_back(tempQobs);
			}
		}
	}

	/*
	readQobs>>startline>>startline>>startline;
	while (readQobs>> date && date <= endDate)
	{
		    
		    if (date >= startDate)
			{
				  readQobs>>startline>>tempQobs;
				  _Qobs.push_back(tempQobs);
				 
			}
			else
				readQobs>>startline>>tempQobs;
    
	}*/
	
	readQobs.close();

	// Addition of Timestep changes by Pallavi Iyengar
	calcDailyQobs();
	calcWeeklyQobs();
	// End of Timestep changes by Pallavi Iyengar

	//  Addition of Monthly Timestep change code by Isira Samarasekera
	calcMonthlyQobs();
	// End of Monthly Timestep changes by Isira Samarasekera
}



// ------------------   object function -----------------------------------//

void calibrate::objFunRunoff(std::vector<double>& Qsim) {
	
	double SUMQobs = 0.0;
	double SSQ = 0.0;

	double VARQ1 = 0.0;
    double VARQ2 = 0.0;
    double VARQ3 = 0.0;

	double aveQobs =0.0;
	double sqrtAveQobs =0.0;
    double sqrtSUMQobs =0.0;
	int size = 1;
	double sF1 = 0.0;
	double sF2 = 0.0;
	double sF3 = 0.0;
	
	CRF1 =0.0;
	CRF2 = 0.0;
	CRF3 =0.0;
	SE = 0.0;
	VE=0.0;

	int k =0;
	int j=0;

	size = (Qsim.size() > _Qobs.size()) ?  _Qobs.size() : Qsim.size();
	//size = _Qobs.size()-1;

	for(int i=0; i < size; i++)    //calculate sum of observed flow
	{
		SUMQobs += _Qobs[i];
		sqrtSUMQobs += sqrt(_Qobs[i]);
	}

	aveQobs = SUMQobs/size;         //calculate average observed flow
    sqrtAveQobs = sqrtSUMQobs/size;
	for(int i=0; i<size; i++)
	{

		sF1 += (Qsim[i] - _Qobs[i])*(Qsim[i] - _Qobs[i]);
		sF2 += fabs(_Qobs[i] - Qsim[i]);
        sF3 += (sqrt(Qsim[i]) - sqrt(_Qobs[i]))*(sqrt(Qsim[i]) - sqrt(_Qobs[i]));
		
		VARQ1 += (_Qobs[i] - aveQobs)*(_Qobs[i] - aveQobs);
	    VARQ2 += fabs(_Qobs[i] - aveQobs);	
		VARQ3 += (sqrt(_Qobs[i]) - sqrtAveQobs)*(sqrt(_Qobs[i]) - sqrtAveQobs);

	}


	SE = sF1/size ;              // mean squared error
	CRF1 = 1 - sF1/VARQ1;        // 1-SUM(Qobsi-Qsimi)^2/SUM(Qobsi-aveQobs)^2
	CRF2 = 1 - sF2/VARQ2;        // 1-SUM|Qobsi-Qsimi|/SUM|Qobsi-aveQobs|
    CRF3 = 1 - sF3/VARQ3;        // 1-SUM(sqrt(Qobsi)-sqrt(Qsim))^2/SUM(sqrt(Qobsi)-sqrt(aveQobs))^2
	VE=1-1-sF2/SUMQobs;

/*	 CString strF1, strF2, strE;
	 strF1.Format(_T("%f"),strF1) ;
	::AfxMessageBox (strF1, MB_OK, 0 );
*/
}

//-

//------------------- index operator for observed runoff  ---------------------

double calibrate::operator [](size_t i) const{
	
	static double safe;
	if(i<0 || _Qobs.size()<=i) 
	{
		std::cout << "index out of bounds in calibrate Qobs[], i: "<<std::endl<<std::endl;
		return safe;
	}
    return _Qobs[i];
}	

// Addition of Timestep changes by Pallavi Iyengar
void calibrate::calcDailyQobs(){
	int dailySize = _Qobs.size() / 24;
	dailyQobs.resize(dailySize);
	for (int i = 0; i<dailySize; i++)
	{
		double dailySum = 0.0;
		for (int j = 0; j<24; j++)
		{
			dailySum = dailySum + _Qobs[i * 24 + j];
		}
		dailyQobs[i] = dailySum;
	}
}
// End of Timestep changes by Pallavi Iyengar

// Addition of Timestep changes by Pallavi Iyengar
void calibrate::calcWeeklyQobs(){
	int weeklySize = _Qobs.size() / (24 * 7);
	weeklyQobs.resize(weeklySize);
	for (int i = 0; i<weeklySize; i++)
	{
		double weeklySum = 0.0;
		for (int j = 0; j<(24 * 7); j++)
		{
			weeklySum = weeklySum + _Qobs[i * 24 * 7 + j];
		}
		weeklyQobs[i] = weeklySum;
	}
}
// End of Timestep changes by Pallavi Iyengar

// Addition of Monthly Timestep change by Isira Samarasekera
void calibrate::calcMonthlyQobs(){
	int monthlySize = _Qobs.size() / (24 * 30);
	monthlyQobs.resize(monthlySize);
	for (int i = 0; i< monthlySize; i++)
	{
		double monthlySum = 0.0;
		for (int j = 0; j<(24 * 30); j++)
		{
			monthlySum = monthlySum + _Qobs[i * 24 * 30 + j];
		}
		monthlyQobs[i] = monthlySum;
	}
}
// End of Monthly Timestep changes by Isira Samarasekera

calibrate::calibrate(std::vector<double>& Qsim, std::vector<double>& Qobs){
_Qobs.clear();
CRF1 = 0.0;
CRF2 = 0.0;
CRF3 = 0.0;
SE = 0.0;
VE = 0.0;

double tempQ = 0.0;
int size = 0;

double SUMQobs = 0.0;
double SSQ = 0.0;

double VARQ1 = 0.0;
double VARQ2 = 0.0;
double VARQ3 = 0.0;

double aveQobs = 0.0;
double sqrtAveQobs = 0.0;
double sqrtSUMQobs = 0.0;

double sF1 = 0.0;
double sF2 = 0.0;
double sF3 = 0.0;

int k = 0;
int j = 0;


size = Qsim.size();

for (int i = 0; i<size; i++)    //calculate sum of observed flow
{
	SUMQobs += Qobs[i];
	sqrtSUMQobs += sqrt(Qobs[i]);
}

aveQobs = SUMQobs / size;         //calculate average observed flow
sqrtAveQobs = sqrtSUMQobs / size;
for (int i = 0; i<size; i++)
{

	sF1 += (Qsim[i] - Qobs[i])*(Qsim[i] - Qobs[i]);
	sF2 += fabs(Qobs[i] - Qsim[i]);
	sF3 += (sqrt(Qsim[i]) - sqrt(Qobs[i]))*(sqrt(Qsim[i]) - sqrt(Qobs[i]));

	VARQ1 += (Qobs[i] - aveQobs)*(Qobs[i] - aveQobs);
	VARQ2 += fabs(Qobs[i] - aveQobs);
	VARQ3 += (sqrt(Qobs[i]) - sqrtAveQobs)*(sqrt(Qobs[i]) - sqrtAveQobs);

}


SE = sF1 / size;              // mean squared error
CRF1 = 1 - sF1 / VARQ1;        // CRF1 = NSE = 1 - (SUM((Qobsi-Qsimi)^2))/SUM(Qobsi-aveQobs)^2
CRF2 = 1 - sF2 / VARQ2;        // 1-SUM|Qobsi-Qsimi|/SUM|Qobsi-aveQobs|
CRF3 = 1 - sF3 / VARQ3;        // 1-SUM(sqrt(Qobsi)-sqrt(Qsim))^2/SUM(sqrt(Qobsi)-sqrt(aveQobs))^2
VE = 1 - sF2 / SUMQobs;        // 1 - SUM(abs(Qsim - Qobs)) / SUM (Qobs)
}