#include "PerviousDepressionStorageCalc.h"

//te no comments on PerviousDepressionStorageCalc.cpp

void PerviousDepressionStorageCalc::calculatePerDepEvapTemperature(Inputs *input, DataFolder *folder, int timeStep)
{
    #define lamda 2257000.0
	folder->VarDict["perDepEvap"] = folder->VarDict["WaterLE"] / lamda;
}

void PerviousDepressionStorageCalc::calculatePerDepEvapStatistical(Inputs *input, DataFolder *folder, int timeStep)
{
	if (folder->VarDict["perDepStor"] > 0.0) {
		folder->VarDict["perDepEvap"] = (folder->VarDict["perDepStor"] / folder->VarDict["perDepStorMax"]) * input->PE[timeStep];
	}
	else
		folder->VarDict["perDepEvap"] = 0.0;
}

void PerviousDepressionStorageCalc::calculate(Inputs *input, DataFolder *folder, int timeStep, std::map<std::string, void(*)(Inputs *input, DataFolder *folder, int timeStep)> &functors)
{

	// perviousStorageMax = param.dat pervious storage maximum value (in mm) * 0.001 -> to convert to meters
	folder->VarDict["perDepStorMax"] = 0.001 * folder->ParamDict["PerviousDepressionStorage_mm"];

	// perDepStorPrev = impDepStor  (Shifts the current pervious depression storage to the temp variable)
	double perDepStorPrev =  folder->VarDict["perDepStor"]; 
	double perPrecip;
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		perPrecip = folder->VarDict["waterToPerDep"];
	}
	else {
		perPrecip = folder->VarDict["waterToPerDep_pre"];
	}
	// increment in pervious Storage
	double incrPerDepStor = 0.0;  

	
	// Calculate the current pervious depression evaporation (tempEper) = (current pervious storage / pervious storage max) * potential evaporation
	// If the SnowWE (snow-water-equivalent) is <= 0
	if(folder->VarDict["snowOpen"] <= 0.0){
		// Set current pervious depression evaporation to (perDepStor/perDepStorMax) * Potential Evaporation
		functors["calculatePerDepEvap"](input, folder, timeStep);
	}else{
		// Otherwise set CurrentEimp (0) - the water won't evaporate if there is snow on the ground.
		folder->VarDict["perDepEvap"] = 0.0;
	}
	
	// If there is no previous storage, there will be no evaporation
	if (perDepStorPrev <= 0.0) {
		folder->VarDict["perDepEvap"] = 0.0;
	}

	// Modify the (current)  pervious depression storage = pervious pression storage + increase in pervious depression storage
	folder->VarDict["perDepStor"] = folder->VarDict["perDepStor"] + perPrecip - folder->VarDict["perDepEvap"];	// update interception storage

	
	// if the evap uses up more than all the stored water and incoming water
	if (folder->VarDict["perDepStor"] < 0.0)
	{
		folder->VarDict["perDepEvap"] = perDepStorPrev + perPrecip;					// per dep evap = incoming plus stored water
		folder->VarDict["perDepStor"] = 0.0;
	}



	// If the current pervious storage < inputStorageTime (Pervious Storage Max)
	if (folder->VarDict["perDepStor"] < folder->VarDict["perDepStorMax"])
	{
		// There is no flow to infiltrate - QToInf at this timestep is 0
		folder->VarDict["runoffToInfil"] = 0.0;
	}
	else  if(folder->VarDict["perDepStor"] >= folder->VarDict["perDepStorMax"])
	{
		// If the previous dep storage < pervious dep storage Max   (meaning the amount that put it over the max has occurred in this timestep)
		if (perDepStorPrev < folder->VarDict["perDepStorMax"])
		{
			// If the current pervios storage >= inputStorageTime (Pervious Storage Max)
			// the flow to infiltrate (QToInf) = incrPerDepStor - (pervious storage max - pervious dep store)
			folder->VarDict["runoffToInfil"] = perPrecip - (folder->VarDict["perDepStorMax"] - perDepStorPrev) - folder->VarDict["perDepEvap"];
		}
		else
		{
			// This Else statement covers the situation of perDepStorPrev being equal to the storage max
			folder->VarDict["runoffToInfil"] = perPrecip - folder->VarDict["perDepEvap"];
			if (folder->ParamStringDict["Type"] != "BulkArea") {
				folder->VarDict["runoffToInfil"] = perPrecip + folder->VarDict["surfaceDepth"]
					- folder->VarDict["perDepEvap"];	// update interception storage
			}
		}
	    // Either of the two preceding cases leaves the current Impervious Storage equal to the storage max 
		folder->VarDict["perDepStor"] = folder->VarDict["perDepStorMax"];
	}


	// Making sure that pervious dep runoff to infiltration  isn't < 0
	if(folder->VarDict["runoffToInfil"] < 0.0)
		folder->VarDict["runoffToInfil"] = 0.0;

	// Making sure that the amount currently stored in pervious depression storage isn't < 0
	if (folder->VarDict["perDepStor"] < 0.0)
		folder->VarDict["perDepStor"] = 0.0;


	storeValues(input, folder, timeStep);


	// Reza, for the GI unit Evap.rate variable based on the SWMM approach
	folder->VarDict["totalEvap"] = folder->VarDict["treeEvapW"] + folder->VarDict["treeEvapSWE"]
		+ folder->VarDict["treeEvap"] + folder->VarDict["perDepEvap"] + folder->VarDict["impEvap_m"];


}


void PerviousDepressionStorageCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["totalperDepEvap"] += folder->VarDict["perDepEvap"];
	folder->VarDict["totalrunoffToInfil"] += folder->VarDict["runoffToInfil"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		folder->VarVecDict["perDepEvap"].emplace_back(folder->VarDict["perDepEvap"]);
		folder->VarVecDict["perDepStor"].emplace_back(folder->VarDict["perDepStor"]);
		folder->VarVecDict["runoffToInfil"].emplace_back(folder->VarDict["runoffToInfil"]);
	}
}

