#ifndef PERVDEPRES_H
#define PERVDEPRES_H

//////////////////////////////////////////////////////////////////////////////////
//																		        //
// PerviousDepressionStorage.h - Calculates pervious depression storage         //																		   
// ver 1.0                                                                      //
// Language:    Visual C++ 2010                                                 //
// Platform:    MacBookPro5,4  Win7										        //
// Application: HydroPrime/iHydro Version6, Summer 2012                         //
// Programmer:  				        //																	   	
//////////////////////////////////////////////////////////////////////////////////
/*
  Origin:
  ==================
  Program: iHydro Version 5 & iHydro Version 4.5
  
  Source Code File(s): VegInterception.cpp
  Functions:           V5 - (line 111) ShortVegintercept()
                       V4.5 - (line 280) intercpt()
  
  Author(s): Jun Wang, jwang06@mailbox.syr.edu, Original author V4/V4.5
             Yang Yang, yyang31@syr.edu, V5
			  	
						*Incorporated calculations into new architecture
  Creation Date: Summer 2000

  Publication(s): Jun Wang PHDThesis

  Module Operations: 
  ==================
  Calculates pervious area through flow after tree and shortVeg interception just used for soil input 

  Public Interface:
  =================
  PerviousAreaDepressionStorageCalc(Inputs *input, Repo *repo);
  void calculate(Inputs *input, DataFolder *folder, int timeStep);

  Build Process:
  ==============
  Required files
	-ICalc.h, InputsRepository.h, ResultsRepository.h, PerviousDepressionStorage.cpp 
				
  Build commands 
    - devenv hydroprime.sln
   
  Maintenance History:
  ====================

 */
#include "../Inputs/Inputs.h"
#include "../DataFolder.h"
#include "../GI/GI_Outlets.h"

class PerviousDepressionStorageCalc
{
	public:		
		static void calculatePerDepEvapTemperature(Inputs * input, DataFolder *folder, int timeStep);
		static void calculatePerDepEvapStatistical(Inputs * input, DataFolder *folder, int timeStep);
		static void calculate(Inputs *input, DataFolder *folder, int timeStep, std::map<std::string, void(*)(Inputs *input, DataFolder *folder, int timeStep)> &functors);
		static void storeValues(Inputs *input, DataFolder *folder, int timeStep);

};



#endif