#ifndef RZONEEVAP_H
#define RZONEEVAP_H
//////////////////////////////////////////////////////////////////////////////////
//																		        //
// RootZoneET.h - Calculates root zone ET       //																		   
// ver 1.0                                                                      //
// Language:    Visual C++ 2010                                                 //
// Platform:    MacBookPro5,4  Win7										        //
// Application: HydroPrime/iHydro Version6, Summer 2012                         //
// Programmer:  				        //																	   	
//////////////////////////////////////////////////////////////////////////////////
/*
  Origin:
  ==================
  Program: iHydro Version 5 & iHydro Version 4.5
  
  Source Code File(s): rrootzone.cpp
  Functions:           V5 - (line 71) evap()
                       V4.5 - (line 83) evap()
  
  Author(s): Jun Wang, jwang06@mailbox.syr.edu, Original author V4/V4.5
             Yang Yang, yyang31@syr.edu, V5
			  	
						*Incorporated calculations into new architecture
  Creation Date: Summer 2000

  Publication(s): Jun Wang PHDThesis

  Module Operations: 
  ==================
  Calculation for zoot zone ET. 

  Public Interface:
  =================
  RootZoneETFlowCalc(Inputs *input, Repo *repo);
  void calculate(Inputs *input, DataFolder *folder, int timeStep);

  Build Process:
  ==============
  Required files
	-ICalc.h, InputsRepository.h, ResultsRepository.h, RootZoneET.cpp 
				
  Build commands 
    - devenv hydroprime.sln
   
  Maintenance History:
  ====================

 */




#include "../Inputs/Inputs.h"
#include "../DataFolder.h"


class RootZoneETCalc
{
	public:	
		static void calculateRootzoneETStatistical(Inputs * input, DataFolder *folder, int timeStep);
		static void calculateRootzoneETTemperature(Inputs * input, DataFolder *folder, int timeStep);
		static void calculate(Inputs *input, DataFolder *folder, int timeStep, std::map<std::string, void(*)(Inputs *input, DataFolder *folder, int timeStep)> &functors);
		static void storeValues(Inputs *input, DataFolder *folder, int timeStep);

};



#endif