#include "SaturatedBaseFlowPowDecay.h"


void SaturatedBaseFlowPowDecayCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	double satZoneQ = 0.0;
	double nN = folder->ParamDict["n"];
	double T0 = folder->ParamDict["T0_m2ph"]; // from ParamDict since it is structure specific, not universal SoilParam
	double mM = folder->ParamDict["m"];

	// Initializing baseflow to 0.0
	folder->VarDict["baseFlow"] = 0.0;
	// Calculating the transmissivity - applying the correct power decay, based on the average topographic index? 
	satZoneQ = folder->ParamDict["BF_flag"] * T0 * pow(input->AverageIndex, -nN);


	// if the average soil moisture deficit is greater than the watershed scaling parameter, then make it just below the watershed scaling param 
	if(folder->VarDict["aveSMD"] > mM)
		folder->VarDict["aveSMD"] = (mM - 0.000001);

	// baseflow is (decayed transmissivity * (1 - average soil moisture deficit / M) ^ N
	folder->VarDict["baseFlow"] = (satZoneQ * pow((1 - (folder->VarDict["aveSMD"]/mM)), nN));
	// storing this folder's baseflow value in the timestep
	// doing this in the FlowSumCalc
	//folder->VarDict["baseFlow"]_vec.emplace_back(folder->VarDict["baseFlow"]);    

}