#include "SnowMeltUnderShortVegCalc.h"


void SnowMeltUnderShortVegCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	meltsnowUnderShortVeg(input, folder, timeStep);
	storeValues(input, folder, timeStep);
}


void SnowMeltUnderShortVegCalc::meltsnowUnderShortVeg(Inputs *input, DataFolder *folder, int timeStep)
{

	double temp = folder->VarDict["snowUnderShortVeg"] + folder->VarDict["shortVegThruFlowSWE"];
	folder->VarDict["snowUnderShortVeg"] = temp;


	double P = folder->VarDict["shortVegThruFlowW"];                                 
	double Tair = input->Tair[timeStep];  
	double RadiationNet_Wpm2 = input->RadiationNet_Wpm2[timeStep];								
	double Tdew = input->Tdew[timeStep];                             
	double UV = input->WindSp_mps[timeStep];                  
	double Kp = 0.8;
	double Za = 2.0;
	double Zb = 2.0;



	double Mp = 0.0129 * P * (Tair - 32);
	double Ms = 0.0000317;
	double Ml = 0.0000307 * (Tair - 32);
	double Mh = 0.0000044 * Kp* pow(Za*Zb, -(1.0 / 6.0)) * (Tair - 32) * UV;
	double Me = 0.0000124 * pow(Za*Zb, -(1.0 / 6.0)) * (Tdew - 32) * UV;
	double Mg = 0.000021;

	folder->VarDict["snowMeltUnderShortVeg"] = Ms + Ml + Mp + Mg + 0.3 * (Mh + Me);   //0.3 can be changed to other value in range 0.3 to 1.0

	if (folder->VarDict["snowMeltUnderShortVeg"] < 0.0)
		folder->VarDict["snowMeltUnderShortVeg"] = 0.0;

	if (folder->VarDict["snowMeltUnderShortVeg"] > folder->VarDict["snowUnderShortVeg"])
		folder->VarDict["snowMeltUnderShortVeg"] = folder->VarDict["snowUnderShortVeg"];

	folder->VarDict["snowUnderShortVeg"] = folder->VarDict["snowUnderShortVeg"] - folder->VarDict["snowMeltUnderShortVeg"];

	//double SnowEP=SnowSublimation(E,i);
	double SnowEP = input->PESnow[timeStep];

	if (SnowEP > folder->VarDict["snowUnderShortVeg"])
		SnowEP = folder->VarDict["snowUnderShortVeg"];

	folder->VarDict["snowUnderShortVeg"] = folder->VarDict["snowUnderShortVeg"] - SnowEP;

	folder->VarDict["waterUnderShortVeg"] = folder->VarDict["snowMeltUnderShortVeg"] + P;

	// updating the snow evap under tree
	folder->VarDict["snowEvapUnderShortVeg"] = SnowEP;
}


void SnowMeltUnderShortVegCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["totalSnowEvapUnderShortVeg_mm"] += folder->VarDict["snowEvapUnderShortVeg"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		folder->VarVecDict["snowEvapUnderShortVeg"].emplace_back(folder->VarDict["snowEvapUnderShortVeg"]);
		folder->VarVecDict["snowMeltUnderShortVeg"].emplace_back(folder->VarDict["snowMeltUnderShortVeg"]);
		folder->VarVecDict["snowUnderShortVeg"].emplace_back(folder->VarDict["snowUnderShortVeg"]);
	}
}