#include "BlockGroupHandler.h"
#include <algorithm>

void BlockGroupHandler::AddToBlockGroup(int DEMLocationIndex, Inputs *input, DataFolder *DEMLocationFolder)
{
	bool folderIsInABlockGroupRange = PrintBlockGroupCheck(DEMLocationIndex, input);
	if (input->EachfoldersBlockGroup[DEMLocationIndex] == input->LocationParams["NODATA"])
		return;
	if (input->TemperatureCalculationParams["PrintAllBlockGroups"] == 1 || folderIsInABlockGroupRange)
	{
		AddDEMLocationFolderValuesToBlockGroupFolder(DEMLocationFolder, input->EachfoldersBlockGroup[DEMLocationIndex]);
	}
}

bool BlockGroupHandler::PrintBlockGroupCheck(int organizerLocationIndex, Inputs *input)
{
	for (int iter2 = 0; iter2 <input->StartingBlockIdsToPrintTempOutput.size(); ++iter2)
	{
		if (input->EachfoldersBlockGroup[organizerLocationIndex] >= input->StartingBlockIdsToPrintTempOutput[iter2] && input->EachfoldersBlockGroup[organizerLocationIndex] <= input->EndingBlockIdsToPrintTempOutput[iter2])
		{
			return true;
		}
	}
	return false;
}

void BlockGroupHandler::TimeStepProcessing(Inputs *input, int timeStep)
{
	int hr = timeStep % 24;
	std::string curDate1 = std::to_string(input->CalendarDates[timeStep] * 100 + hr);
	std::string curDate2 = std::to_string(input->CalendarDates[timeStep]);
	for (auto blockGroupIter : BlockGroupIDToItsFolder)
	{
		int blockGroupNum = blockGroupIter.first;
		if(input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"]=="true"|| input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"] == "True")
			TemperatureOutputWriter::writeToBlockGroupHourlyFile(input->SimulationStringParams["OutputDirectory"], curDate1, BlockGroupIDToItsFolder[blockGroupNum]);
		DailyCal(blockGroupNum, input->WindSp_mps[timeStep]);
		Resefolder(blockGroupNum);
		if (hr == 23)
		{
			TemperatureOutputWriter::writeToBlockGroupDailyFile(input->SimulationStringParams["OutputDirectory"], curDate2, BlockGroupIDToItsFolder[blockGroupNum], timeStep);
			ResefolderDailyValues(blockGroupNum);
		}
	}
}

BlockGroupHandler::BlockGroupHandler(Inputs *input)
{
	//TemperatureCalculationParams["RowColOutputAllTimeSteps"] && block group >-9999
	for (int i = 0; i < input->EachfoldersBlockGroup.size(); ++i)
	{
		bool createBlockGroupfolder = (input->TemperatureCalculationParams["PrintAllBlockGroups"] && (input->EachfoldersBlockGroup[i] != input->LocationParams["NODATA"]));
		if (!createBlockGroupfolder)
			createBlockGroupfolder = PrintBlockGroupCheck(i, input);
		if (!createBlockGroupfolder)
			continue;
		if (BlockGroupIDToItsFolder[input->EachfoldersBlockGroup[i]] == NULL)
		{
			DataFolder *bgFolder = new DataFolder;
		    //#undef max
			//size_t limit = std::numeric_limits<double>::max();
			double limit = 10000;
			bgFolder->VarDict["WindChillMin"] = limit;
			bgFolder->VarDict["AirTMin"] = limit;
			BlockGroupIDToItsFolder[input->EachfoldersBlockGroup[i]] = bgFolder;
			BlockGroupIDToItsFolder[input->EachfoldersBlockGroup[i]]->ParamDict["blockGroupIndex"] = input->EachfoldersBlockGroup[i];
		}
		BlockGroupIDToItsFolder[input->EachfoldersBlockGroup[i]]->ParamDict["folderCount"] += 1;
	}
}

BlockGroupHandler::~BlockGroupHandler()
{
	for (auto b : BlockGroupIDToItsFolder)
	{
		delete b.second;
	}

}

void BlockGroupHandler::AddDEMLocationFolderValuesToBlockGroupFolder(DataFolder *folder, int blockGroupID)
{
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ImpNR"] += folder->VarDict["ImpNR"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ImperviousH"] += folder->VarDict["ImperviousH"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ImperviousLE"] += folder->VarDict["ImperviousLE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeNR"] += folder->VarDict["TreeNR"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeH"] += folder->VarDict["TreeH"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeLE"] += folder->VarDict["TreeLE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeLEE"] += folder->VarDict["TreeLEE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeLET"] += folder->VarDict["TreeLET"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegNR"] += folder->VarDict["ShortVegNR"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegH"] += folder->VarDict["ShortVegH"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegLE"] += folder->VarDict["ShortVegLE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegLEE"] += folder->VarDict["ShortVegLEE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegLET"] += folder->VarDict["ShortVegLET"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["SoilNR"] += folder->VarDict["SoilNR"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["SoilH"] += folder->VarDict["SoilH"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["SoilLE"] += folder->VarDict["SoilLE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WaterNR"] += folder->VarDict["WaterNR"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WaterH"] += folder->VarDict["WaterH"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WaterLE"] += folder->VarDict["WaterLE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["H"] += folder->VarDict["H"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["LE"] += folder->VarDict["LE"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["Tb"] += folder->VarDict["Tb"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["Td"] += folder->VarDict["Td"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["Eb"] += folder->VarDict["Eb"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirT"] += folder->VarDict["AirT"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirE"] += folder->VarDict["AirE"];
}

void BlockGroupHandler::ResefolderDailyValues(int blockGroupID)
{
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTMax"] = 0;
	//#undef max
	//size_t limit = std::numeric_limits<double>::max();
	double limit = 10000;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTMin"] = limit;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTAvg"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TdAvg"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndexAvg"] = 0; //NOAA NWS Method described below
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndexMax"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndex2Avg"] = 0; //Lin et al. Method described below
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndex2Max"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WindChillMin"] = limit;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WindChillAvg"] = 0;
}


void BlockGroupHandler::Resefolder(int blockGroupID)
{
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ImpNR"] =0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ImperviousH"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ImperviousLE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeNR"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeH"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeLE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeLEE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TreeLET"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegNR"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegH"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegLE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegLEE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["ShortVegLET"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["SoilNR"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["SoilH"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["SoilLE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WaterNR"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WaterH"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WaterLE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["H"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["LE"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["Tb"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["Td"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["Eb"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirT"] = 0;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirE"] = 0;
}


void BlockGroupHandler::DailyCal(int blockGroupID, double windSpeed)
{

	double AirTC = BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirT"] / BlockGroupIDToItsFolder[blockGroupID]->ParamDict["folderCount"] - 273.15;
	double TdC = BlockGroupIDToItsFolder[blockGroupID]->VarDict["Td"] / BlockGroupIDToItsFolder[blockGroupID]->ParamDict["folderCount"] - 273.15;
	double e_AirT = 0.6108 * exp((17.27 * AirTC) / (237.3 + AirTC));
	double e_Tdew = 0.6108 * exp((17.27 * TdC) / (237.3 + TdC));
	double RH = e_Tdew / e_AirT * 100.0;
	double AirTF = AirTC * 9.0/5.0 + 32;

	//HeatIndex terms are from Rothfusz, Lans P. National Weather Service Technical Attachment SR 90-23 (1990), https://www.wpc.ncep.noaa.gov/html/heatindex_equation.shtml
	double heatindex_F = 0.5 * (AirTF + 61.0 + ((AirTF - 68.0) * 1.2) + (RH * 0.094));
	heatindex_F = 0.5 * (heatindex_F + AirTF);

	if (heatindex_F >= 80.0)
	{
		heatindex_F = -42.379 + (2.04901523 * AirTF) + (10.14333127 * RH) - (0.22475541 * AirTF * RH) -
			(0.00683783 * AirTF * AirTF) - (0.05481717 * RH * RH) + (0.00122874 * AirTF * AirTF * RH) +
			(0.00085282 * AirTF * RH * RH) - (0.00000199 * AirTF * AirTF * RH * RH);
	}
	else if (RH < 13.0 && AirTF > 80.0 && AirTF < 112.0)
	{
		double hi_adjust = ((13.0 - RH) / 4.0) * std::pow(((17.0 - std::abs(AirTF - 95.0)) / 17.0), 0.5);
		heatindex_F = heatindex_F - hi_adjust;
	}
	else if (RH > 85.0 && AirTF > 80.0 && AirTF < 87.0)
	{
		double hi_adjust = ((RH - 85.0) / 10.0) * ((87.0 - AirTF) / 5.0);
		heatindex_F = heatindex_F + hi_adjust;
	}
	else
	{
		heatindex_F = AirTF;
	}

	//HeatIndex2 terms are from Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009)
	//Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009). "Extreme high temperatures and hospital admissions for respiratory and cardiovascular diseases." Epidemiology 20(5): 738-746.
	//Basu, R., W.Y.Feng and B.D.Ostro(2008). "Characterizing temperature and mortality in nine California counties." Epidemiology 19(1) : 138 - 145.
	//Steadman, R. G. (1979). "The Assessment of Sultriness. Part I: A Temperature-Humidity Index Based on Human Physiology and Clothing Science." Journal of Applied Meteorology 18(7): 861-873.
	double heatindex2_C = -2.653 + (0.994 * AirTC) + 0.0153 * pow(TdC,2);

	//get wind speed
	double windSpeedMPH = windSpeed*3600.0/1609.34;; //wspd(jday, hr)
	double windchill_F;

	//Wind Chill Index Method, NOAA NWS
	//The National Weather Service Wind Chill Temperature Index udpated in 2001, https://www.weather.gov/oun/safety-winter-windchill
	if (AirTF < 50.0 && windSpeedMPH > 3.0)
	{
		windchill_F = 35.74 + (0.6215 * AirTF) - (35.75 * std::pow(windSpeedMPH, 0.16)) + (0.4275 * AirTF *std::pow(windSpeedMPH, 0.16));
	}
	else
	{
		windchill_F = AirTF;
	}

	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTAvg"] += BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirT"] / BlockGroupIDToItsFolder[blockGroupID]->ParamDict["folderCount"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["TdAvg"] += BlockGroupIDToItsFolder[blockGroupID]->VarDict["Td"] / BlockGroupIDToItsFolder[blockGroupID]->ParamDict["folderCount"];
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndexAvg"] += ((heatindex_F - 32.0) * (5.0/9.0) + 273.15);
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndex2Avg"] += heatindex2_C + 273.15 ;
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WindChillAvg"] += ((windchill_F - 32.0) * (5.0/9.0) + 273.15);
	#define max(a, b) ((a) > (b) ? (a) : (b))
    #define min(a, b) ((a) < (b) ? (a) : (b))
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTMax"] = max(BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTMax"], BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirT"] / BlockGroupIDToItsFolder[blockGroupID]->ParamDict["folderCount"]);
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTMin"] = min(BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirTMin"], BlockGroupIDToItsFolder[blockGroupID]->VarDict["AirT"] / BlockGroupIDToItsFolder[blockGroupID]->ParamDict["folderCount"]);
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndexMax"] = max(BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndexMax"], ((heatindex_F - 32.0) * (5.0/9.0) + 273.15));
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndex2Max"] = max(BlockGroupIDToItsFolder[blockGroupID]->VarDict["HeatIndex2Max"], (heatindex2_C + 273.15));
	BlockGroupIDToItsFolder[blockGroupID]->VarDict["WindChillMin"] = min(BlockGroupIDToItsFolder[blockGroupID]->VarDict["WindChillMin"], ((windchill_F - 32.0) * (5.0/9.0) + 273.15));

}
