/*                                                                   
 *  InterpolationCal.h - interpolates the hourly temperatures. This     
 *                       may take up a bit of space on the stack. This  
 *                       can be changed later, but for now I'm going to 
 *                       leave it as is.                                
 *                                                                      
 *  Language:     Visual Studio 2010 C++                                
 *                                                                      
 *  Application:  iTree-Temperature model                               
 *  Author:       Steven Maio                                           
 *                stevenmaio.321@gmail.com
 *                                                                      
 */

#ifndef ICTCINTERPOLATOR_H
#define ICTCINTERPOLATOR_H

#include "BlockGroupHandler.h"
#include "../Inputs/Inputs.h"
#include <stdio.h>
#include <fstream>
#include <vector>
#include <map>
#include <sstream>

#define ICTC_RANGE 101


class ICTCInterpolator
{
	private:
		double icInterpolationFactor;
		double tcInterpolationFactor;
		double svInterpolationFactor;
		double icvInterpolationFactor;
		double icMinInterpolationFactor;
		double icMaxInterpolationFactor;
		double tcMinInterpolationFactor;
		double tcMaxInterpolationFactor;
		double svMinInterpolationFactor;
		double svMaxInterpolationFactor;
	public:
		ICTCInterpolator();
		static const double NORMALIZATION_FACTOR;
		static const std::string IC0TC0_AIR_TEMP;
		static const std::string IC0TC0_DEW_TEMP;
		static const std::string IC0TC100_AIR_TEMP;
		static const std::string IC0TC100_DEW_TEMP;
		static const std::string IC100TC0_AIR_TEMP;
		static const std::string IC100TC0_DEW_TEMP;
		double computeICVInterpolationFactor(int icValue);
		double computeTCInterpolationFactor(int icValue, int tcValue);
		double computeICInterpolationFactor(int icValue, int tcValue);
		double computeSVInterpolationFactor(int icValue, int tcValue);
		double interpolateHourlyTemperature(double svt, double ict, double tct, int icValue, int tcValue);
		void interpolateData(Inputs* inputs);
};

#endif // !ICTCINTERPOLATOR_H