//////////////////////////////////////////////////////////////////////////	
//  TemperatureOutputWriter.h - Provides methods to write Temperature	//
//					             related output files				    //
/////////////////////////////////////////////////////////////////////////
/*



Module Operations:
==================
Here are some example config file options for specifying temperature output. There are three types of 
output and options for each type are describe below.
1. BlockGroup Outputs

		Example A: Print all block group hourly files and block group daily files. To only print daily files set 
		PrintBlockGroupDailyAndHourly to false or remove the tag. 
				<TemperatureExecutionParams>
					<RefWeatherLocation>row#,col#</RefWeatherLocation>
					<PrintBlockGroupRange>All</PrintBlockGroupRange>
					<PrintBlockGroupDailyAndHourly>True</PrintBlockGroupDailyAndHourly>
				</TemperatureExecutionParams>
		Example B: Print block group files that fall with the ranges specified 8,8 would print only block group and 
		block group daily files for group 8.  1,5 would print 1,2,3,4,5. To specify a new block group range such such as
		22-23 
				<TemperatureExecutionParams>
					<RefWeatherLocation>row#,col#</RefWeatherLocation>
					<PrintBlockGroupRange0>1,5</PrintBlockGroupRange0>
					<PrintBlockGroupRange1>8,8</PrintBlockGroupRange1>
				</TemperatureExecutionParams>
		you would add the tag.
				<TemperatureExecutionParams>
					<RefWeatherLocation>row#,col#</RefWeatherLocation>
					<PrintBlockGroupRange0>1,5</PrintBlockGroupRange0>
					<PrintBlockGroupRange1>8,8</PrintBlockGroupRange1>
					<PrintBlockGroupRange2>22,23</PrintBlockGroupRange2>
				</TemperatureExecutionParams>

2. RowCol Output

			Example C: 
			Print Row#Col#Heat.txt and Row#Col#Hydro.txt
				for all locations in the DEM for the time range specified. June 24, 2015 from the 
				1st hour/time step to the last hour/timestep of that day; and June 26-28, 2015.
			  <TemperatureExecutionParams>
				  <RefWeatherLocation>1,42</RefWeatherLocation>
				  <PrintRowCol>All</PrintRowCol>
				  <RowColOutputPeriod0>2015062400,2015062423</RowColOutputPeriod0>
				  <RowColOutputPeriod1>2015062600,2015062823</RowColOutputPeriod1>
			  </TemperatureExecutionParams>
			  
			Example D: Print Row#Col#Heat.txt 
				for specific locations in the DEM for all timesteps in the simulation (RowColOutputPeriod = All).

			  <TemperatureExecutionParams>
				  <RefWeatherLocation>1,42</RefWeatherLocation>
				  <PrintRowCol0>5,3</PrintRowCol0>
				  <PrintRowCol1>2,2</PrintRowCol1>
				  <RowColOutputPeriod>All</RowColOutputPeriod>
			  </TemperatureExecutionParams>

 3.TimeStep Output 

				 Example E:
				 For each timestep within 2015062400,2015062423 range, print a file for a specific variable such as 
				 AirT#.txt (where # is the timestep#). When PrintTimeBasedResults is set to All, All variable files 
				 will be created containing that value for each location in the DEM. 
				 (See function TemperatureOutputWriter::timeStepResultsFileProcessor for complete list of variables.)
				 <TemperatureExecutionParams>
					<RefWeatherLocation>1,42</RefWeatherLocation>
					<PrintTimeBasedResults>All</PrintTimeBasedResults>
					<TimeBasedOutputPeriod0>2015062400,2015062423</TimeBasedOutputPeriod0>
				 <TemperatureExecutionParams>
				
				When PrintAllTimeBasedResults is set to AirT, only AirT#.txt files will be printed for 
				the following three time periods.

				Example F:
				 <TemperatureExecutionParams>
					<RefWeatherLocation>1,42</RefWeatherLocation>
					<PrintTimeBasedResults>AirT</PrintTimeBasedResults>
					<TimeBasedOutputPeriod0>2015062400,2015062423</TimeBasedOutputPeriod0>
					<TimeBasedOutputPeriod1>2015062600,2015062723</TimeBasedOutputPeriod1>
					<TimeBasedOutputPeriod2>2015062800,2015063000</TimeBasedOutputPeriod2>
				 <TemperatureExecutionParams>

				In the below example, all variable files AirT#.txt, TD#.txt, ect. files will be printed 
				for every timestep in the simulation. 

				 Example G:
				 <TemperatureExecutionParams>
					<RefWeatherLocation>1,42</RefWeatherLocation>
					<PrintTimeBasedResults>All</PrintTimeBasedResults>
					<TimeBasedOutputPeriod>All</TimeBasedOutputPeriod>
				 <TemperatureExecutionParams>


Public Interface & Description:
=================
1.  BlockGroup Outputs
*Block group related output functions for hourly data
	Ex. BlockGroup#.txt '#' = placeholder for blockgroup number
	YYYYMMDDHH    Ta    Td
	2015062100   0  0
	2015062101   295.187  294.368
	2015062102   295.292  295.008

	static void createBlockGroupHourlyFile(std::string outDirectory, int blockGroupIndex);
		-The above function creates the file and adds the header line
	static void writeToBlockGroupHourlyFile(std::string outDirectory, std::string date, DataFolder *blockGroupFolder);
		-The above function writes output for a particular block group for particular hour

*Block group related output functions for daily data (HeatIndex terms are from Rothfusz, Lans P. National Weather Service Technical Attachment SR 90-23 (1990), HeatIndex2 terms are from Lin, S., M. Luo, R. J. Walker, X. Liu, S. A. Hwang and R. Chinery (2009) 
	Ex. BlockGroupDaily#.txt  '#' = placeholder for blockgroup number
	YYYYMMDD    RawTairMax    RawTairMin    RawTairAvg    RawTdewAvg    HeatIndexMax    HeatIndexAvg    HeatIndex2Max    HeatIndex2Avg    WindchillMin    WindchillAvg
	20150621   303.559  0  286.672  282.121  273.15  273.15  273.15  273.15  273.15  273.15
	static void createBlockGroupDailyFile(std::string outDirectory, int blockGroupIndex);
	-The above function creates the file and adds the header line
	static void writeToBlockGroupDailyFile(std::string outDirectory, std::string date, DataFolder *blockGroupFolder);
	-The above function writes output for a particular block group for a specific day
	-Outputs include post-processed temperature metrics used for temperature-health analysis (HeatIndex, WindChill, min mean max)

*Helper Functions
static void generateBlockFiles(std::map<int, DataFolder*> BlockGroupIDToItsfolder, std::string outDirectory, bool createDailyFiles);
-The above function organizes the creation of all blockgroup and blockgroupdaily files

2. RowCol Output

*Functions related to the creation of Row#Col#Heat.txt # is a placeholder for DEM location row number and col number 
	ts    Ta    Td   Ea    ImpNR    TreeNR    ShortVegNR    SoilNR    ImpH    ImpLE    TreeH    TreeLE    TreeLEE    TreeLET    ShortH    ShortLE   ShortLEE    ShortLET    SoilH    SoilLE  H   LE
	2015062409   297.025  288.089   0.012781   68.0026  74.0514  60.2795  0  85.4408  0  -171.964  222.164  189.708  32.4559  -180.26  240.489  187.167  53.3219  0  0  35.794  43.7463
	static void createHeatRowColFile(int i, int j, std::string outDirectory);
	-The above function creates the file and adds the header line
	static void writeToHeatRowColFile(int timeStep, int i, int j, DataFolder *folder, std::string outDirectory);
	-The above function writes output for a specific DEM location for a specific time step
*Functions related to the creation of Row#Col#Hydro.txt # is a placeholder for DEM location row number and col number
These are currently not in use!!!
	-static void createHydroRowColFile(int i, int j, std::string outDirectory, Inputs *input);
	-static void writeToHydroRowColFile(int i, int j, DataFolder *folder, std::string outDirectory);
*Helper functions for generating row#col# files
	-static void rowColFileBatchProcessor(int timeStep, int organizerLocationIndex, DataFolder *folder, std::string outDirectory, Inputs *input);
	The above function determines if the data for a location at a specific time step is written to its 
	corresponding heat file
	-static void generateRowColFiles(DataOrganizer *organizer, Inputs *input);
	The above function creates all rowcol files for the simulation run

3. Time Based output

*Functions related to the creation of timestep files containing the specified variable value for each location
in the DEM.
	-static void createTimeStepResultFile(std::string outputVarName, std::string date, Inputs *input);
	The above function creates the file for a specific result and date
	-static void writeToTimeStepResultFile(std::string outputVarName, std::string date, std::string & outDirectory, double val, bool newLine);
	The above function adds the value for a specific location's variable value to appropriate file. The value is written
	to the same DEM row/col location.
	-static void timeStepResultsFileProcessor(int timeStep, int organizerLocationIndex, DataFolder *folder, std::string outDirectory, Inputs *input);
	The above function coordinates the creation of all result files and addition of each locations results to created files

Maintenance History:
====================


*/

#ifndef TemperatureOutputWriter_H
#define TemperatureOutputWriter_H
#include "../Inputs/Inputs.h"
#include "../DataFolder.h"
#include "../DataOrganizer.h"
#include <map>

class TemperatureOutputWriter
{
public:
	//Generate block group and block group daily files
	static void generateBlockFiles(std::map<int, DataFolder*> BlockGroupIDToItsfolder, std::string outDirectory, bool needHourlyBGFiles, bool createDailyFiles);
	static void outputLandCoverDetails(int index, std::string outDirectory, DataFolder * folder);
	//Related functions to create files containing hourly data for block groups i.e. BlockGroup#.txt
	static void createBlockGroupHourlyFile(std::string outDirectory, int blockGroupIndex);
	static void writeToBlockGroupHourlyFile(std::string outDirectory, std::string date, DataFolder *blockGroupFolder);

	//Related functions to create files containing daily data for block groups i.e. BlockGroupDaily#.txt
	static void createBlockGroupDailyFile(std::string outDirectory, int blockGroupIndex);
	static void writeToBlockGroupDailyFile(std::string outDirectory, std::string date, DataFolder *blockGroupFolder, int timeStep);

	//Related output functions for generating location based files containing hourly data i.e. Row#Col#Hydro.txt & Row#Col#Heat.txt
	static void rowColFileBatchProcessor(int timeStep, int organizerLocationIndex, DataFolder *folder, std::string outDirectory, Inputs *input);
	static void generateRowColFiles(DataOrganizer *organizer, Inputs *input);
    
	//Row#Col#Hydro.txt Currently not used
	static void createHydroRowColFile(int i, int j, std::string outDirectory, Inputs *input);
	static void writeToHydroRowColFile(int ts, int i, int j, DataFolder *folder, std::string outDirectory); 
	//Row#Col#Heat.txt
	static void createHeatRowColFile(int i, int j, std::string outDirectory);
	static void writeToHeatRowColFile(int timeStep, int i, int j, DataFolder *folder, std::string outDirectory);
	
	//Related output functions for generating a file containing a specific variable/output timestep value for each 
	//location in the DEM.dat i.e. __#.txt where __is the variable name such as AirT and # is the timestep AirT3.txt

	static void createTimeStepResultFile(std::string outputVarName, int date, Inputs *input);
	static void writeToTimeStepResultFile(std::string outputVarName, int date, std::string & outDirectory, DataOrganizer *organizer, Inputs *input);
	static void timeStepResultsFileProcessor(int timeStep, DataOrganizer *organizer, std::string outDirectory, Inputs *input);


};

#endif