#include "PerviousAreaThroughFlowCalc.h"

// PerviousAreaThroughFlow function 
// History: Introduced by June Wang for use in iTree V4
//          Modified by Yang Yang
// References: 1. Wang, Jun, Theodore A. Endreny, and David J. Nowak, 2008.
//                Mechanistic Simulation of Tree Effects in an Impervious Water Balance Model 
//                Journal of the American Water Resources Association (JAWRA) 44(1): 75-85
//                DOI: 10.1111/j.1752-1688.2007.00139.x
// Authors: June Wang, Yang Yang, Shannon Conley

void PerviousAreaThroughFlowCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	perviousAreaThroughFlow(input, folder, timeStep);
	storeValues(input, folder, timeStep);
}


void PerviousAreaThroughFlowCalc::perviousAreaThroughFlow(Inputs *input, DataFolder *folder, int timeStep)
{
	double treeThruFlowSWE = folder->VarDict["treeThruFlowSWE"];
	double shortVegThruFlowSWE = folder->VarDict["shortVegThruFlowSWE"];
	double TotalVegThruFlow = (folder->VarDict["treeThruFlowW"] + treeThruFlowSWE) * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	TotalVegThruFlow +=  (folder->VarDict["shortVegThruFlowW"] + shortVegThruFlowSWE) * folder->ParamDict["ShortVegetationCover_frac"];
	folder->VarDict["TotalVegThruFlow"] = TotalVegThruFlow;
}


void PerviousAreaThroughFlowCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["totalVegThruFlowStore"] += folder->VarDict["TotalVegThruFlow"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		folder->VarVecDict["TotalVegThruFlow"].emplace_back(folder->VarDict["TotalVegThruFlow"]);
	}
}