
#include "UnsatZonePerviousFlowCalc.h"

void UnsatZonePerviousFlowCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		statisticalCalculate(input, folder, timeStep);
	}
	else
	{
		spatialCalculate(input, folder, timeStep);
	}
}

void UnsatZonePerviousFlowCalc::spatialCalculate(Inputs *input, DataFolder *folder, int timeStep)
{
	// temporary variable for the saturation excess pervious area flow 
	double satExcessFlow = 0.0;
	satExcessFlow = input->IndexArea[0] * (folder->VarVecDict["satExRunoff_TI"][0]);
	
	folder->VarDict["satExRunoff"] = satExcessFlow;

	storeValues(input, folder, timeStep);
}


void UnsatZonePerviousFlowCalc::statisticalCalculate(Inputs *input, DataFolder *folder, int timeStep)
{

	// temporary variable for the saturation excess pervious area flow 
	double satExcessFlow = 0.0;
	// temporary variable for the amount of saturated area in the index value ib (lower in the vector than index value ia)
	double ibFA ;                  // saturated fractional area of olandQ[ib]
	// Setting the sum of pervious area flow for the unsaturated zone (across all TI bins) to 0.0
	folder->VarDict["satExRunoff"] = 0.0;
	// setting the sum of ????? to 0.0
    input->RepoDict["AcmfA"]=0.0;




	for(int ia=1; ia<input->IndexAreaSize; ia++)
	{ 
		// setting the temporary variable for the saturation excess pervious area flow to 0.0
		satExcessFlow = 0.0;
		// setting the saturated fraction of lower TI vale to 0.0
		ibFA = 0.0;
		// Setting ib to be the index value below ia in the vector - this means ib will be a higher TI value - more likely to be saturated (higher a / lower Tan B)
		int ib = ia -1;

		//if ( pervious flow for index value ia is > 0.0)
		if (folder->VarVecDict["satExRunoff_TI"][ia] > 0.0 )                 // both limits are saturated - because if ia is saturated, ib is too
		{
			// pervious flow = fraction of watershed area of index value ia * (pervious flow from ia + pervious flow from ib) / 2.0
			satExcessFlow = input->IndexArea[ia] * (folder->VarVecDict["satExRunoff_TI"][ia] + folder->VarVecDict["satExRunoff_TI"][ib])/2.0;
			// updating the fraction of the watershed that is saturated?  Adding in the area from ia?
			input->RepoDict["AcmfA"] += input->IndexArea[ia];
		}
		else if (folder->VarVecDict["satExRunoff_TI"][ib] > 0.0)            // check if lower limit saturated ( higher ln(a/tanB) ) - if ia isn't saturated, ib might be
		{
			// the fraction of saturated area in ib  = fraction of watershed area of index value ia * pervious flow from ib / (pervious flow from ib - pervious flow form ia)
			// Q: if we enter this "else-if" statement, pervious flow from ia is either negative of 0.  So subtracting it from pervious flow from ib will increase the denominator (or leave it the same if 0)
			// and the ibFA value will either be just plain equal to fraction of watershed in ia * pervious flow form ib or smaller if pervious flow is negative
			ibFA = input->IndexArea[ia] * folder->VarVecDict["satExRunoff_TI"][ib]/(folder->VarVecDict["satExRunoff_TI"][ib] - folder->VarVecDict["satExRunoff_TI"][ia]) ;
			// This appears to average the pervious flow coming from ib (and therefore form ia?) between the original value and the newly (above) calculated value
			satExcessFlow = ibFA * folder->VarVecDict["satExRunoff_TI"][ib]/2.0 ;
			//updating the fraction of the watershed that is saturated?  Adding in the area from ia?
            input->RepoDict["AcmfA"]+= ibFA;
		}

		folder->VarDict["satExRunoff"] += satExcessFlow;                // total over landflow of the catchment for one time step

	}
	
	//Reza: Here we calculate the effective saturated excess runoff from the contributing area as a function of the "satExRunoff"
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		input->RepoDict["satExRunoff_temp"] = folder->VarDict["satExRunoff"];
	}
	if (folder->ParamStringDict["Type"] != "BulkArea") {

		input->RepoDict["satExRunoffContArea"] = input->RepoDict["satExRunoff_temp"] *
			((folder->ParamDict["upSlopeArea"] * //(1.0 - folder->ParamDict["upSlopeAreaImpFrac"]) *
				folder->ParamDict["ImpAreaTreated_frac"]));// /
		//		(input->SimulationNumericalParams["CatchmentArea_m2"] - (folder->ParamDict["Area"] * folder->ParamDict["giNumber"])));
		// convert to the area of the GI
		input->RepoDict["satExRunoffContArea_GI"] = (input->RepoDict["satExRunoffContArea"]  /
			(folder->ParamDict["Area"] * folder->ParamDict["giNumber"]));
		//folder->VarDict["waterToPerDep"] += input->RepoDict["satExRunoffContArea"];
	}
	//if (folder->ParamStringDict["Type"] == "BulkArea") {
	//	folder->VarDict["satExRunoff"] -= input->RepoDict["satExRunoffContArea"];
	//	if (folder->VarDict["satExRunoff"] < 0.0) {
	//		folder->VarDict["satExRunoff"] = 0.0;
	//	}
	//}

	//Reza: converting the saturated excess runoff from the contibuting area to the area of the GIs. 
	if (folder->ParamStringDict["Type"] != "BulkArea") {
		for (int ia = 0; ia < input->IndexAreaSize; ia++) {
			folder->VarDict["Zi"] = folder->VarVecDict["localSMD_TI"][folder->ParamDict["binNumber"]] / folder->ParamDict["effPorosity"];
			//folder->VarDict["Zi"] = input->RepoDict["local_BA_TI"] / folder->ParamDict["effPorosity"];
		}
		//folder->VarDict["Zi"] /= input->IndexAreaSize;         
	}


	//folder->VarDict["adjSatExRunoff"] = folder->VarDict["satExRunoff"]; //* (1-folder->ParamDict["InfiltExcessGovernedArea_frac"]); 
	//Reza
	//if (folder->ParamStringDict["Type"] != "BulkArea") {
	//	folder->VarDict["adjGI_satExRunoff"] = folder->VarDict["satExRunoff"];
	//	folder->VarDict["satExRunoff"] = 0;
	//}

	storeValues(input, folder, timeStep);

}


void UnsatZonePerviousFlowCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	
	
	folder->VarDict["totalSatExRunoff"] += folder->VarDict["satExRunoff"];

	//STORE ADJUSTED VALUE HERE AS WELL

	//Arpit
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
		folder->VarVecDict["satExRunoff"].emplace_back(folder->VarDict["satExRunoff"]);

}