#include "UnsatZoneSoilMoistureDeficitPowDecay.h"


void UnsatZoneSoilMoistureDeficitPowDecayCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	
	double mM = folder->ParamDict["m"];      // Watershed soil scaling parameter
	double nN = folder->ParamDict["n"];      // power governing decay of saturated hydraulic conductivity
	double Q0 = folder->ParamDict["Q0_mph"];     // Baseflow estimate
	double T0 = folder->ParamDict["T0_m2ph"];      // Soil transmissivity at saturation, specific to each structure


	// This for loop runs for each member of the Topographic Index Distribution - typically 30 values
   	for(int ia=0; ia < input->IndexAreaSize; ia++)
	{
		if (timeStep == 0)
		{
			folder->VarVecDict["localSMD_TI"].push_back(0.0);
		}
		// the local (unsaturated) soil mositure deficit = (average SMD + watershed scaling param (m) * (Baseflow/Transmissivity) ^ (1/n) * ( Average TI - local TI)
		if(input->IndexAreaSize > 1)
			folder->VarVecDict["localSMD_TI"][ia] = folder->VarDict["aveSMD"] + mM * pow(Q0/T0, 1/nN)*(input->AverageIndex - input->Index[ia]); // real for coresponding index
		else
			folder->VarVecDict["localSMD_TI"][ia] = folder->VarDict["aveSMD"] + mM * pow(Q0/T0, 1/nN)*(input->AverageIndex - input->Index[folder->VarDict["position"]]); // real for coresponding index


		// Updating the local SMD - cannot be greater than M - the watershed scaling factor - set to M
		if(folder->VarVecDict["localSMD_TI"][ia] > mM)
			folder->VarVecDict["localSMD_TI"][ia] = mM;

		// Updating the local SMD - cannot be less then 0 - set to 0
		if(folder->VarVecDict["localSMD_TI"][ia] < 0.0 )         
			folder->VarVecDict["localSMD_TI"][ia] = 0.0;

	}

}



void UnsatZoneSoilMoistureDeficitPowDecayCalc::storeValues(Inputs * input, DataFolder *folder, int timeStep)
{

	// Nothing to store as of right now
}