#include "WaterOnImperviousAreaCalc.h"
#include <iostream>


void WaterOnImperviousAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	double totalImperviousArea;
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		totalImperviousArea = folder->ParamDict["TotalImperviousCover_frac"];
	}
	else {
		totalImperviousArea = folder->ParamDict["ImperviousCover_frac"];
	}
	double WaterOnImperviousAreaVal = 0.0;
	
 	if(totalImperviousArea)     
	{
		//cte UrP is from hydroprime Dec11, used to rewrite equation for WaterOnImperviousAreaVal
		//UrP = repo->treeInterceptRepo.getTreeThroughfall() * repo->TreeOnImp / (repo->TreeOnImp + repo->ImperP) + repo->P * (1 - repo->TreeOnImp / (repo->TreeOnImp + repo->ImperP));

		WaterOnImperviousAreaVal = folder->VarDict["waterUnderTree"] * folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] / (folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"]);
		WaterOnImperviousAreaVal += folder->VarDict["waterOpenArea"] * folder->ParamDict["ImperviousCover_frac"] / (folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"]);

		/* cte Feb 17, 2020 seems ratio is different than urP computation in Dec11 code TempUrbanCalc::calculate
		WaterOnImperviousAreaVal = folder->VarDict["waterUnderTree"] * folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
		WaterOnImperviousAreaVal += folder->VarDict["waterOpenArea"] * folder->ParamDict["ImperviousCover_frac"];
		*/
		//cte - this is division by a fraction which seems incorrect; the model then uses waterOnImperviousArea in ImperviousDepressionStorageCalc.cpp (August 2019)
		//WaterOnImperviousAreaVal /= totalImperviousArea;

		folder->VarDict["waterOnImperviousArea"] = WaterOnImperviousAreaVal;
	}
	else
	{
		folder->VarDict["waterOnImperviousArea"] = 0.0;
	}

	storeValues(input, folder, timeStep);
}

void WaterOnImperviousAreaCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		folder->VarVecDict["waterOnImpArea"].emplace_back(folder->VarDict["waterOnImperviousArea"]);
	}
}
